@echo off

rem // iOS build environment master script by Pierre-Marie Baty <pm@pmbaty.com>
rem // Windows version -- for the UNIX version, see build.sh

setlocal enableextensions enabledelayedexpansion
goto startup

:usage
echo ===============================================================================
echo        iOS Build Environment's command-line builder by Pierre-Marie Baty
echo ===============================================================================
echo Builds ^(and optionally deploys^) an iOS project from the command line.
echo.
echo   Usage: %SCRIPT_NAME% [path] [options]
echo.
echo Valid options:
echo.
echo   -toolchain ^"path^" - build environment toolchain path ^(guessed if unset^)
echo   -sdk ^"path^" - SDK root path with libclang_rt library ^(guessed if unset^)
echo   -keychain ^"path^" - path to certificates and private keys ^(guessed if unset^)
echo   -xcname ^"filename^" - extensionless Xcode project bundle name ^(optional^)
echo   -xcpname ^"Project Name^" - Xcode target name to build from xcname ^(optional^)
echo   -xcconf ^"Configuration Name^" - Xcode target configuration ^(optional^)
echo   -name ^"Project Name^" - name of the project to build ^(guessed if unset^)
echo   -archs ^"cpu1 [cpu2] [...]^" - target CPUs to build ^(armv7, armv7s, arm64, etc^)
echo   -family ^<iphone^|ipad^|all^> - target specific device family (defaults to all)
echo   -config ^"Name^" - build configuration name ^(defaults to ^"Default^" if unset^)
echo   -miniosver N - minimum OS version
echo   -builddir ^"path^" - build directory prefix ^(defaults to ^"build^")
echo   -cppflags ^"flags^" - additional preprocessor-specific flags*
echo   -cflags ^"flags^" - additional compiler-specific flags ^(C, Objective-C^)*
echo   -cxxflags ^"flags^" - additional compiler-specific flags ^(C++, Objective-C++^)*
echo   -pch ^"file1 [file2] [...]^" - list of precompiled header files to include
echo   -lld - use LLVM's lld linker instead of Apple's ld64
echo   -ldflags ^"flags^" - additional linker-specific flags*
echo   -type ### - ^"crt^", ^"bundle^", ^"lib^", ^"dylib^", ^"framework^" ^(guessed if unset^)
echo   -autoresolve - attempt to resolve undefined Objective-C classes automatically
echo   -strip [none^|bitcode] - strip output binary options to reduce code size
echo   -res ^"file1 [file2] [...]^" - list of resource files or directories to embed
echo   -[no]pngcrush - optimize icons and splash screen PNG images for iOS [or not]
echo   -[no]symlink - optimize splash screens by symlinking identical ones [or not]
echo   -embed ^"^<dylib1^|framework1^> [dylib2^|framework2] [...]^" - binaries to embed
echo   -entitlements ^"pathname^" - path to an optional entitlements property list
echo   -prepkgscript ^"pathname^" - path to an optional pre-packaging script
echo   -identity ^"cert[:...]:privatekey:passphrase^" - signer identity string**
echo   -provision ^"file.mobileprovision^" - pathname of provisioning profile to embed
echo   -artwork ^"image file^" - specify a 1024x1024 App Store artwork image
echo   -autoversion - use an automatic build version number scheme ^(DDDD.HH.MM^)
echo   -ipa - create an iOS Packaged Archive ^(.ipa^)
echo   -deploy [multiple] [flashcode] - deploy .ipa OTA ^(requires Internet^)
echo   -deb - create a Cydia package ^(for jailbroken devices^)
echo   -rebuild - rebuild everything from scratch ^(ignore file modification times^)
echo   -nodeps - skip rebuilding the dependencies, if some are found
echo   -nopods - skip building the pods, if a Podfile is found
echo   -nocache - do not use the compiler cache ^(ccache^)
echo   -cleanmodules - clean the compiler cache ^(ccache^), forcing a global rebuild
echo   -multicore - use several CPU cores ^(default: all^) for compilation
echo   -cores # - limit multithread compilation to a specific number of CPU cores
echo   -remotelog ^<myself^|IP^> - send standard error and backtraces to specific host
echo.
echo *: to specify extra compiler/linker flags that start with a dash safely, it is
echo    recommended to pass them using environment variables instead. Those are:
echo.
echo      IOS_CPPFLAGS[_TargetName] - additional preprocessor-specific flags
echo      IOS_CFLAGS[_TargetName]   - additional C/Objective-C compiler flags
echo      IOS_CXXFLAGS[_TargetName] - additional C++/Objective-C++ compiler flags
echo      IOS_LDFLAGS[_TargetName]  - additional linker flags
echo      IOS_IDENTITY  - signer identity string ^(see below^)
echo      IOS_PROVISION - full pathname of the provisioning profile to embed
echo      IOS_EMBEDDED_BINARIES - additional files to embed in the app or bundle
echo.
echo    When a Xcode target name is defined with -xcpname, it is possible to define
echo    target- or dependency-specific CPPFLAGS, CFLAGS, CXXFLAGS and LDFLAGS by
echo    appending '_TargetName' to the environment variable's name; all non-alpha-
echo    numeric characters in TargetName must be turned into underscores. Example:
echo.
echo       target "Unity-iPhone" CFLAGS = %%IOS_CFLAGS%% + %%IOS_CFLAGS_Unity_iPhone%%
echo.
echo **: the signer identity string is a colon-separated list of X.509 certificates
echo     forming a trust chain that you want to include in your digital signature.
echo     A typical identity string contains at least 3 items: your own certificate,
echo     your private key and its passphrase. If you don't supply intermediary
echo     certificates after yours, the code signer will attempt to pick them up
echo     automatically from the keychain directory.
echo.
echo        Example: ^"my.cer:my.key:p@ssw0rd^"
echo.
echo     If no signing identity is provided, the default identity is used; if no
echo     default identity has been setup, then only a pseudo-signature is performed.
goto :exit_success


:startup

rem // DO NOT switch to the UTF-8 codepage, IT BREAKS THE find.exe UTILITY AND POSSIBLY OTHERS!
rem // chcp 65001 > nul

rem // tell the world who we are
title iOS Build Environment

rem // figure out where we are
set SCRIPT_NAME=%~nx0
set SCRIPT_ARGS=%*
set SCRIPT_DIRECTORY=%~dp0

rem // for safety, rebuild a minimalistic, default PATH. This is the default PATH for Windows 10.
set PATH=%SystemRoot%\system32;%SystemRoot%;%SystemRoot%\System32\Wbem;%SYSTEMROOT%\System32\WindowsPowerShell\v1.0\

rem // for safety, clear the OpenSSL environment variables
rem // source: https://www.openssl.org/docs/manmaster/man7/openssl-env.html
set OPENSSL=
set OPENSSL_HOME=
set OPENSSL_CONF=
set OPENSSL_CONF_INCLUDE=
set OPENSSL_CONFIG=
set OPENSSL_ENGINES=
set OPENSSL_MALLOC_FD=
set OPENSSL_MALLOC_FAILURES=
set OPENSSL_MODULES=
set OPENSSL_WIN32_UTF8=
set RANDFILE=
set SSL_CERT_DIR=
set SSL_CERT_FILE=
set TSGET=
set OPENSSL_ia32cap=
set OPENSSL_sparcv9cap=
set OPENSSL_ppccap=
set OPENSSL_armcap=
set OPENSSL_s390xcap=
set OPENSSL_riscvcap=
set NO_PROXY=
set HTTPS_PROXY=
set HTTP_PROXY=

rem // create a temporary output directory
rem // NOTE: do NOT use %TEMP% here, as it may point to a pathname containing non-ANSI characters! (e.g. "C:\Users\<ChineseNoun>\AppData\Local\Temp")
rem // use the C:\iOS\TEMP directory instead, and create it if needed
set TEMPDIR=C:\iOS\TEMP\!SCRIPT_NAME!.%RANDOM%-%RANDOM%
rmdir /s /q "!TEMPDIR!" > nul 2>&1
mkdir "!TEMPDIR!" > nul 2>&1

rem // handy shortcut to redirect anything into an environment variable called TEMPVAR. Relies on !TEMPDIR!. Usage: any_command > %TO_TEMPVAR%
set TO_TEMPVAR="^!TEMPDIR^!\output.txt" 2^> nul ^& set TEMPVAR=^& set /p TEMPVAR=^< "^!TEMPDIR^!\output.txt" ^& del /f /q "^!TEMPDIR^!\output.txt" ^> nul 2^>^&1

rem // cleanup the variables we'll use (CPPFLAGS, CFLAGS, CXXFLAGS and LDFLAGS are inherited after IOS_CPPFLAGS, IOS_CFLAGS, IOS_CXXFLAGS and IOS_LDFLAGS respectively)
set TOOLCHAIN_PATH=
set SDK_PATH=
set SDK_RUNTIME_LIB=
set KEYCHAIN_PATH=
set XCNAME=
set TARGET_NAME=
set CONFIGURATION=
set NAME=
set ARCHS=all
set FAMILY=
set CONFIG=Default
set BUILD_NUMBER=
set MINOSVER=
set CPPFLAGS=!IOS_CPPFLAGS!
set CFLAGS=!IOS_CFLAGS!
set CXXFLAGS=!IOS_CXXFLAGS!
set PCH=
if not "_!IOS_LDFLAGS!_"=="__" echo !IOS_LDFLAGS!>> "!TEMPDIR!\LDFLAGS.txt"& rem // non-target specific LDFLAGS
set ENTRYPOINT=
set RES=
set SKIP_DEPENDENCIES=
set SKIP_COCOAPODS=
set WANT_STRIP=true
set WANT_STRIP_BITCODE=
set WANT_PNGCRUSH=
set WANT_SYMLINKS=
if not "_!IOS_EMBEDDED_BINARIES!_"=="__" echo !IOS_EMBEDDED_BINARIES!>> "!TEMPDIR!\EMBEDDED_BINARIES.txt"
set ENTITLEMENTS_PLIST=
set PREPKG_SCRIPT=
set IDENTITY=!IOS_IDENTITY!
set PROVISION=!IOS_PROVISION!
set ARTWORK=
set WANT_LLVMLINKER=
set WANT_AUTOVERSION=
set WANT_AUTORESOLVE=
set WANT_IPA=
set WANT_DEPLOY=
set WANT_DEB=
set NEEDS_REBUILD=
set WANT_REBUILDALL=
set WANT_CLEANCCACHE=
set WANT_MULTICORE=
set WANT_CCACHE=true
set WORKING_DIR=
set CORES=
set REMOTELOG_IP=
set DEBUG_ARG=
set DEBUG_WAIT=0
set BUILD_DIR=build

rem /////////////////////////////
rem // BEGIN arguments collection


set SKIP_NEXT_ARG=
:parse
shift

rem // if we were told to skip this argument, acknowledge and skip it
if "_!SKIP_NEXT_ARG!_"=="_yes_" set SKIP_NEXT_ARG=& goto parse

rem // collect argument, remove leading and ending quotes and evaluate
set ARG=###%0###
set ARG=!ARG:"###=!
set ARG=!ARG:###"=!
if not "_!ARG!_"=="__" set ARG=!ARG:###=!

rem // no more args ? it means the end of the command line has been reached
if "_!ARG!_"=="__" goto parseok

rem // support both "-option" and "/option" syntaxes
set IS_FLAG=
if "_!ARG:~0,1!_"=="_-_" set IS_FLAG=true
if "_!ARG:~0,1!_"=="_/_" set IS_FLAG=true

rem // if current arg is not a flag, then it's a target directory
if "_!IS_FLAG!_"=="__" set WORKING_DIR=!ARG!

rem // else current arg is an option flag
if "_!IS_FLAG!_"=="_true_" (
	rem // collect possible 2nd and 3rd arguments, removing any leading and ending quotes
	set ARG2=###%1###
	set ARG2=!ARG2:"###=!
	set ARG2=!ARG2:###"=!
	if not "_!ARG2!_"=="__" set ARG2=!ARG2:###=!
	set ARG3=###%2###
	set ARG3=!ARG3:"###=!
	set ARG3=!ARG3:###"=!
	if not "_!ARG3!_"=="__" set ARG3=!ARG3:###=!
	rem // identify option flag and send it into the right variable
	if "_!ARG:~1!_"=="_?_"            goto usage
	if "_!ARG:~1!_"=="_help_"         goto usage
	if "_!ARG:~1!_"=="_toolchain_"    set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set TOOLCHAIN_PATH=!ARG2!
	if "_!ARG:~1!_"=="_sdk_"          set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set SDK_PATH=!ARG2!
	if "_!ARG:~1!_"=="_keychain_"     set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set KEYCHAIN_PATH=!ARG2!
	if "_!ARG:~1!_"=="_xcname_"       set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set XCNAME=!ARG2!
	if "_!ARG:~1!_"=="_xcpname_"      set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set TARGET_NAME=!ARG2!
	if "_!ARG:~1!_"=="_xcconf_"       set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set CONFIGURATION=!ARG2!
	if "_!ARG:~1!_"=="_builddir_"     set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set BUILD_DIR=!ARG2!
	if "_!ARG:~1!_"=="_name_"         set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set NAME=!ARG2!
	if "_!ARG:~1!_"=="_archs_"        set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set ARCHS=!ARG2!
	if "_!ARG:~1!_"=="_family_"       set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set FAMILY=!ARG2!
	if "_!ARG:~1!_"=="_config_"       set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set CONFIGURATION=!ARG2!
	if "_!ARG:~1!_"=="_miniosver_"    set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set MINOSVER=!ARG2!& rem // miniosver is a legacy argument
	if "_!ARG:~1!_"=="_minosver_"     set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set MINOSVER=!ARG2!
	if "_!ARG:~1!_"=="_cppflags_"     set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set CPPFLAGS=!CPPFLAGS! !ARG2!
	if "_!ARG:~1!_"=="_cflags_"       set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set CFLAGS=!CFLAGS! !ARG2!
	if "_!ARG:~1!_"=="_cxxflags_"     set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set CXXFLAGS=!CXXFLAGS! !ARG2!
	if "_!ARG:~1!_"=="_pch_"          set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set "PCH=!ARG2!"
	if "_!ARG:~1!_"=="_lld_"          set WANT_LLVMLINKER=true
	if "_!ARG:~1!_"=="_ldflags_"      set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" echo !ARG2!>> "!TEMPDIR!\LDFLAGS.txt"
	if "_!ARG:~1!_"=="_strip_" (
		if "_!ARG2!_"=="_bitcode_" set SKIP_NEXT_ARG=yes& set WANT_STRIP_BITCODE=true
		if "_!ARG2!_"=="_none_"    set SKIP_NEXT_ARG=yes& set WANT_STRIP=false
	)
	if "_!ARG:~1!_"=="_type_"         set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set ENTRYPOINT=!ARG2!
	if "_!ARG:~1!_"=="_res_"          set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set "RES=!ARG2!"
	if "_!ARG:~1!_"=="_pngcrush_"     set WANT_PNGCRUSH=true
	if "_!ARG:~1!_"=="_nopngcrush_"   set WANT_PNGCRUSH=false
	if "_!ARG:~1!_"=="_symlink"       set WANT_SYMLINKS=true
	if "_!ARG:~1!_"=="_nosymlink"     set WANT_SYMLINKS=false
	if "_!ARG:~1!_"=="_embed_"        set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" echo !ARG2!>> "!TEMPDIR!\EMBEDDED_BINARIES.txt"
	if "_!ARG:~1!_"=="_entitlements_" set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set ENTITLEMENTS_PLIST=!ARG2!
	if "_!ARG:~1!_"=="_prepkgscript_" set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set PREPKG_SCRIPT=!ARG2!
	if "_!ARG:~1!_"=="_identity_"     set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set IDENTITY=!ARG2!
	if "_!ARG:~1!_"=="_provision_"    set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set PROVISION=!ARG2!
	if "_!ARG:~1!_"=="_artwork_"      set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set ARTWORK=!ARG2!
	if "_!ARG:~1!_"=="_autoversion_"  set WANT_AUTOVERSION=true
	if "_!ARG:~1!_"=="_autoresolve_"  set WANT_AUTORESOLVE=true
	if "_!ARG:~1!_"=="_ipa_"          set WANT_IPA=true
	if "_!ARG:~1!_"=="_deploy_" (
		rem // this arg has several syntaxes, either "-deploy", or "-deploy [yes|no]", or "-deploy multiple", or "-deploy [flashcode|qrcode|datamatrix]", or "-deploy multiple [flashcode|qrcode|datamatrix]"
		if "_!ARG2!_"=="_yes_"        set SKIP_NEXT_ARG=yes
		if "_!ARG2!_"=="_no_"         set SKIP_NEXT_ARG=yes
		if "_!ARG2!_"=="_multiple_"   set SKIP_NEXT_ARG=yes
		if "_!ARG2!_"=="_flashcode_"  set SKIP_NEXT_ARG=yes
		rem // NOTE: "qrcode" and "datamatrix" are deprecated options. Kept here for backwards compatibility.
		if "_!ARG2!_"=="_qrcode_"     set SKIP_NEXT_ARG=yes
		if "_!ARG2!_"=="_datamatrix_" set SKIP_NEXT_ARG=yes
		if not "_!ARG2!_"=="_no_"     set WANT_DEPLOY=true
		if "_!ARG2!_"=="_multiple_" if not "_!ARG3!_"=="_flashcode_" if not "_!ARG3!_"=="_qrcode_" if not "_!ARG3!_"=="_datamatrix_" set WANT_DEPLOY=multiple
		if "_!ARG2!_"=="_flashcode_"  if not "_!ARG3!_"=="_multiple_" set WANT_DEPLOY=flashcode
		if "_!ARG2!_"=="_qrcode_"     if not "_!ARG3!_"=="_multiple_" set WANT_DEPLOY=qrcode
		if "_!ARG2!_"=="_datamatrix_" if not "_!ARG3!_"=="_multiple_" set WANT_DEPLOY=datamatrix
		if "_!ARG2!_"=="_multiple_" if "_!ARG3!_"=="_flashcode_"  set WANT_DEPLOY=multiple flashcode&  shift
		if "_!ARG2!_"=="_multiple_" if "_!ARG3!_"=="_qrcode_"     set WANT_DEPLOY=multiple qrcode&     shift
		if "_!ARG2!_"=="_multiple_" if "_!ARG3!_"=="_datamatrix_" set WANT_DEPLOY=multiple datamatrix& shift
	)
	if "_!ARG:~1!_"=="_deb_"          set WANT_DEB=true
	if "_!ARG:~1!_"=="_rebuild_"      set WANT_REBUILDALL=true
	if "_!ARG:~1!_"=="_nodeps_"       set SKIP_DEPENDENCIES=true
	if "_!ARG:~1!_"=="_nopods_"       set SKIP_COCOAPODS=true
	if "_!ARG:~1!_"=="_nocache_"      set WANT_CCACHE=false
	if "_!ARG:~1!_"=="_cleanmodules_" set WANT_CLEANCCACHE=true
	if "_!ARG:~1!_"=="_multicore_"    set WANT_MULTICORE=true
	if "_!ARG:~1!_"=="_cores_"        set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set CORES=!ARG2!
	if "_!ARG:~1!_"=="_remotelog_"    set SKIP_NEXT_ARG=yes& if not "_!ARG2!_"=="__" set REMOTELOG_IP=!ARG2!
	if "_!ARG:~1!_"=="_debugscript_"  set DEBUG_ARG=true
)

goto parse

:parseok


rem // if we have a defined TARGET_NAME, see if we have matching CPPFLAGS, CFLAGS, CXXFLAGS and LDFLAGS to pick up
if not "_!TARGET_NAME!_"=="__" (
	rem // sanitize target name by turning all non-alphanumeric characters into an underscore
	set CONSUMABLE_INPUT=!TARGET_NAME!
	set SANITIZED_TARGETNAME=
	set ALLOWED_CHARS=abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890
	:sanitize_targetname_1morechar
	rem // as long as there are characters to go, consider the first char of CONSUMABLE_INPUT and see if it's in the allowed chars
	rem // if it's allowed, append it to the output variable, and replace it with an underscore if it's not
	if defined CONSUMABLE_INPUT (
		for /F "delims=*~ eol=*" %%C in ("!CONSUMABLE_INPUT:~0,1!") do (
			if not "!ALLOWED_CHARS:%%C=!"=="!ALLOWED_CHARS!" (
				set SANITIZED_TARGETNAME=!SANITIZED_TARGETNAME!%%C
			) else (
				set SANITIZED_TARGETNAME=!SANITIZED_TARGETNAME!_
			)
		)
		rem // consume one character from the input and try again
		set CONSUMABLE_INPUT=!CONSUMABLE_INPUT:~1!
		goto :sanitize_targetname_1morechar
	)
	rem // at this point, target name is sane enough for an environment variable name
	rem // construct the target-specific compiler and linker flags
	if defined CPPFLAGS (set CPPFLAGS=!CPPFLAGS! !IOS_CPPFLAGS_%SANITIZED_TARGETNAME%!) else set CPPFLAGS=!IOS_CPPFLAGS_%SANITIZED_TARGETNAME%!
	if defined CFLAGS   (set CFLAGS=!CFLAGS! !IOS_CFLAGS_%SANITIZED_TARGETNAME%!)       else set CFLAGS=!IOS_CFLAGS_%SANITIZED_TARGETNAME%!
	if defined CXXFLAGS (set CXXFLAGS=!CXXFLAGS! !IOS_CXXFLAGS_%SANITIZED_TARGETNAME%!) else set CXXFLAGS=!IOS_CXXFLAGS_%SANITIZED_TARGETNAME%!
	if not "_!IOS_LDFLAGS_%SANITIZED_TARGETNAME%!_"=="__" echo !IOS_LDFLAGS_%SANITIZED_TARGETNAME%!>> "!TEMPDIR!\LDFLAGS.txt"
	rem // unset the variables we no longer need
	set ALLOWED_CHARS=
	set SANITIZED_TARGETNAME=
	set CONSUMABLE_INPUT=
)


rem // unset variables we no longer need
set IS_FLAG=
set ARG2=
set ARG=

rem // display collected arguments in debug mode
if not "_%DEBUG_ARG%_"=="__" (
	rem // when in debug mode, add -v to the linker flags to make sure it's there (it doesn't matter if the flag is there already)
	echo  -v >> "!TEMPDIR!\LDFLAGS.txt"

	echo ===============================================================================
	set /p BUILDER_VERSION=<"%SCRIPT_DIRECTORY%changelog.txt"
	set BUILDER_VERSION=!BUILDER_VERSION:~19!
	echo Builder version: !BUILDER_VERSION!
	echo Working directory: %CD%
	echo Command line invokation:
	echo "%SCRIPT_DIRECTORY%%SCRIPT_NAME%" %SCRIPT_ARGS%
	echo ===============================================================================
	if "_!WANT_MULTICORE!_"=="_true_" (
		echo NOTICE: disabling parallel builds in script debug mode
		set WANT_MULTICORE=false
	)
	echo Arguments collected:
	echo ARCHS=[!ARCHS!]
	echo ARTWORK=[!ARTWORK!]
	echo BUILD_DIR=[!BUILD_DIR!]
	echo CFLAGS=[!CFLAGS!]
	echo CONFIGURATION=[!CONFIGURATION!]
	echo CORES=[!CORES!]
	echo CPPFLAGS=[!CPPFLAGS!]
	echo CXXFLAGS=[!CXXFLAGS!]
	echo|set /p=EMBEDDED_BINARIES=[
	if exist "!TEMPDIR!\EMBEDDED_BINARIES.txt" type "!TEMPDIR!\EMBEDDED_BINARIES.txt"
	echo ]
	echo ENTITLEMENTS_PLIST=[!ENTITLEMENTS_PLIST!]
	echo ENTRYPOINT=[!ENTRYPOINT!]
	echo FAMILY=[!FAMILY!]
	echo IDENTITY=[!IDENTITY!]
	echo|set /p=LDFLAGS=[
	if exist "!TEMPDIR!\LDFLAGS.txt" type "!TEMPDIR!\LDFLAGS.txt"
	echo ]
	echo MINOSVER=[!MINOSVER!]
	echo NAME=[!NAME!]
	echo NEEDS_REBUILD=[!NEEDS_REBUILD!]
	echo PCH=[!PCH!]
	echo PREPKG_SCRIPT=[!PREPKG_SCRIPT!]
	echo PROVISION=[!PROVISION!]
	echo REMOTELOG_IP=[!REMOTELOG_IP!]
	echo RES=[!RES!]
	echo SDK_PATH=[!SDK_PATH!]
	echo SDK_RUNTIME_LIB=[!SDK_RUNTIME_LIB!]
	echo SKIP_DEPENDENCIES=[!SKIP_DEPENDENCIES!]
	echo SKIP_COCOAPODS=[!SKIP_COCOAPODS!]
	echo TARGET_NAME=[!TARGET_NAME!]
	echo TOOLCHAIN_PATH=[!TOOLCHAIN_PATH!]
	echo WANT_AUTORESOLVE=[!WANT_AUTORESOLVE!]
	echo WANT_AUTOVERSION=[!WANT_AUTOVERSION!]
	echo WANT_CCACHE=[!WANT_CCACHE!]
	echo WANT_CLEANCCACHE=[!WANT_CLEANCCACHE!]
	echo WANT_LLVMLINKER=[!WANT_LLVMLINKER!]
	echo WANT_DEB=[!WANT_DEB!]
	echo WANT_DEPLOY=[!WANT_DEPLOY!]
	echo WANT_IPA=[!WANT_IPA!]
	echo WANT_MULTICORE=[!WANT_MULTICORE!]
	echo WANT_PNGCRUSH=[!WANT_PNGCRUSH!]
	echo WANT_REBUILDALL=[!WANT_REBUILDALL!]
	echo WANT_STRIP_BITCODE=[!WANT_STRIP_BITCODE!]
	echo WANT_STRIP=[!WANT_STRIP!]
	echo WANT_SYMLINKS=[!WANT_SYMLINKS!]
	echo XCNAME=[!XCNAME!]
	echo Press a key or wait %DEBUG_WAIT% seconds to continue...
	timeout /t %DEBUG_WAIT% > nul
)


rem // END arguments collection
rem ///////////////////////////


rem ///////////////////////////
rem // BEGIN arguments guessing


rem // make sure we have a valid toolchain path: convert relative paths to absolute ; if unset, deduce it from the script directory
if not "_!TOOLCHAIN_PATH!_"=="__" if not "_!TOOLCHAIN_PATH:~1,2!_"=="_:_" if not "_!TOOLCHAIN_PATH:~0,1!_"=="_\_" set TOOLCHAIN_PATH=%CD%\!TOOLCHAIN_PATH!
if "_!TOOLCHAIN_PATH!_"=="__" set TOOLCHAIN_PATH=%SCRIPT_DIRECTORY%Toolchain
if not exist "!TOOLCHAIN_PATH!\clang.exe"    echo ERROR: toolchain path does not seem to point to a valid toolchain directory.& goto :exit_error
if not exist "!TOOLCHAIN_PATH!\ld.exe"       echo ERROR: toolchain path does not seem to point to a valid toolchain directory.& goto :exit_error
if not exist "!TOOLCHAIN_PATH!\ld64.lld.exe" echo ERROR: toolchain path does not seem to point to a valid toolchain directory.& goto :exit_error
if not exist "!TOOLCHAIN_PATH!\ldid.exe"     echo ERROR: toolchain path does not seem to point to a valid toolchain directory.& goto :exit_error


rem // if we're using LLVM's linker AND the user wants link-step hints, signal the user that he should use the Apple linker instead
if not "_!WANT_LLVMLINKER!_"=="__" if not "_!WANT_AUTORESOLVE!_"=="__" set WANT_AUTORESOLVE=&echo WARNING: auto-fix feature is only available when using Apple's linker. Turning it off.


rem // make sure we have a valid SDK path: convert relative paths to absolute ; if unset, deduce it from the script directory
if not "_!SDK_PATH!_"=="__" if not "_!SDK_PATH:~1,2!_"=="_:_" if not "_!SDK_PATH:~0,1!_"=="_\_" set SDK_PATH=%CD%\!SDK_PATH!
rem // if no explicit SDK path was passed, cycle through all the known SDK directories until we find a usable one
if "_!SDK_PATH!_"=="__" set SDK_PATH=%SCRIPT_DIRECTORY%SDK
if not exist "!SDK_PATH!\Info.plist" set SDK_PATH=%SCRIPT_DIRECTORY%SDK-iOS
if not exist "!SDK_PATH!\Info.plist" set SDK_PATH=%SCRIPT_DIRECTORY%SDK-macOS
if not exist "!SDK_PATH!\Info.plist" set SDK_PATH=%SCRIPT_DIRECTORY%SDK-watchOS
if not exist "!SDK_PATH!\Info.plist" set SDK_PATH=%SCRIPT_DIRECTORY%SDK-tvOS
if not exist "!SDK_PATH!\Info.plist" set SDK_PATH=%SCRIPT_DIRECTORY%SDK-visionOS
if not exist "!SDK_PATH!\Info.plist" echo No Info.plist was found in the SDK directory. Please run the SDK Migration Assistant again.& goto :exit_sdk_missing
rem // read the SDK platform name out of the SDK and deduce the runtime library to use
"!TOOLCHAIN_PATH!\plconvert.exe" "!SDK_PATH!\Info.plist" -query Name > %TO_TEMPVAR%
if not "_!TEMPVAR!_"=="__" set SDK_PLATFORM_NAME=!TEMPVAR!
if        "_!SDK_PLATFORM_NAME!_"=="_iphoneos_"  ( set TARGET_PLATFORM_NAME=ios&     set SDK_RUNTIME_LIB=clang_rt.ios
) else if "_!SDK_PLATFORM_NAME!_"=="_macosx_"    ( set TARGET_PLATFORM_NAME=macos&   set SDK_RUNTIME_LIB=clang_rt.osx
) else if "_!SDK_PLATFORM_NAME!_"=="_watchos_"   ( set TARGET_PLATFORM_NAME=watchos& set SDK_RUNTIME_LIB=clang_rt.watchos
) else if "_!SDK_PLATFORM_NAME!_"=="_appletvos_" ( set TARGET_PLATFORM_NAME=tvos&    set SDK_RUNTIME_LIB=clang_rt.tvos
) else if "_!SDK_PLATFORM_NAME!_"=="_xros_"      ( set TARGET_PLATFORM_NAME=xros&    set SDK_RUNTIME_LIB=clang_rt.xros
) else (
	echo The platform this SDK concerns is unrecognized: !SDK_PLATFORM_NAME!. Please update your build environment.
	goto :exit_error
)
rem // read the SDK version number out of the SDK
"!TOOLCHAIN_PATH!\plconvert.exe" "!SDK_PATH!\Info.plist" -query Version > %TO_TEMPVAR%
if not "_!TEMPVAR!_"=="__" set SDK_VERSION=!TEMPVAR!
rem // SDK consistency checks
if not exist "!SDK_PATH!\lib!SDK_RUNTIME_LIB!.a"      echo The Clang run-time library has not been found. Please run the SDK Migration Assistant again.&    goto :exit_sdk_missing
if not exist "!SDK_PATH!\System\Library\Frameworks\." echo The SDK frameworks stubs have not been found. Please run the SDK Migration Assistant again.& goto :exit_sdk_missing
if not exist "!SDK_PATH!\usr\include\."               echo The SDK headers have not been found. Please run the SDK Migration Assistant again.&          goto :exit_sdk_missing
rem // SDK consistency checks (Swift-specific)
if not exist "!SDK_PATH!\..\lib\swift\!SDK_PLATFORM_NAME!\layouts-arm64.yaml" echo Some files necessary for the Swift compiler are missing from this SDK.& echo Please run the SDK Migration Assistant again ^(take care to use the latest assistant^).& goto :exit_sdk_missing
if not exist "!SDK_PATH!\..\lib\swift\apinotes\os.apinotes"                   echo Some files necessary for the Swift compiler are missing from this SDK.& echo Please run the SDK Migration Assistant again ^(take care to use the latest assistant^).& goto :exit_sdk_missing


rem // make sure we have a valid keychain path: convert relative paths to absolute; if unset, deduce it from the script directory
if not "_!KEYCHAIN_PATH!_"=="__" if not "_!KEYCHAIN_PATH:~1,1!_"=="_:_" if not "_!KEYCHAIN_PATH:~0,1!_"=="_\_" set KEYCHAIN_PATH=%CD%\!KEYCHAIN_PATH!
if "_!KEYCHAIN_PATH!_"=="__" set KEYCHAIN_PATH=%SCRIPT_DIRECTORY%Keychain
rem // signing identity and team ID: setup fallback defaults
set SIGNER=
set TEAMID=BADC0DE516
rem // figure out what type of signing identity we have: do we have no explicit identity ?
if "_!IDENTITY!_"=="__" set NO_EXPLICIT_IDENTITY=true
if "_!IDENTITY!_"=="_::_" set NO_EXPLICIT_IDENTITY=true
if "_!NO_EXPLICIT_IDENTITY!_"=="_true_" if exist "!KEYCHAIN_PATH!\Default identity.ini" (
	rem // no explicit identity was supplied AND a default identity file exists ; read signing identity from it
	type "!KEYCHAIN_PATH!\Default identity.ini" | findstr /v "^$ ^#" > "!TEMPDIR!\signing-identity.txt"
	for /f "usebackq delims=" %%l in ("!TEMPDIR!\signing-identity.txt") do (
		set LINE=%%l
		set VALUE=
		for /f "tokens=2 delims==" %%v in ("!LINE!") do set VALUE=%%v
		rem // strip leading spaces, leading tabs, trailing spaces and trailing tabs
		for /l %%a in (1,1,255) do if "!VALUE:~0,1!"==" " set VALUE=!VALUE:~1!
		for /l %%a in (1,1,255) do if "!VALUE:~0,1!"=="	" set VALUE=!VALUE:~1!
		for /l %%a in (1,1,255) do if "!VALUE:~-1!"==" "  set VALUE=!VALUE:~0,-1!
		for /l %%a in (1,1,255) do if "!VALUE:~-1!"=="	" set VALUE=!VALUE:~0,-1!
		rem // now read the values
		if "_!LINE:~0,12!_"=="_CERTIFICATES_" set DEFAULT_CERTIFICATES=!VALUE!
		if "_!LINE:~0,11!_"=="_PRIVATE_KEY_"  set DEFAULT_PRIVATE_KEY=!VALUE!
		if "_!LINE:~0,10!_"=="_PASSPHRASE_"   set DEFAULT_PASSPHRASE=!VALUE!
		if "_!LINE:~0,9!_"=="_PROVISION_"     set DEFAULT_PROVISION=!VALUE!
	)
	set IDENTITY=!DEFAULT_CERTIFICATES!:!DEFAULT_PRIVATE_KEY!:!DEFAULT_PASSPHRASE!
	if "_!PROVISION!_"=="__" if not "_!DEFAULT_PROVISION!_"=="__" set PROVISION=!DEFAULT_PROVISION!
)
rem // else is the identity explicitly "none" (i.e. a pseudo-signature is wanted ?) if so, clear identity string
if "_!IDENTITY!_"=="_none_" set IDENTITY=
rem // now, if we have a valid signing identity, identify the signer by its Common Name
if not "_!IDENTITY!_"=="__" if not "_!IDENTITY!_"=="_::_" (
	rem // split the identity string
	for %%S in ("!IDENTITY::=" "!") do (
		rem // remove leading and ending quotes and evaluate item
		set ITEM=###%%S###
		set ITEM=!ITEM:"###=!
		set ITEM=!ITEM:###"=!
		if not "_!ITEM!_"=="__" set ITEM=!ITEM:###=!
		if not "_!ITEM!_"=="__" if not "_!ITEM!_"=="_AppleIncRootCertificate.cer_" if not "_!ITEM!_"=="_AppleWWDRCA.cer_" (
			if not exist "!ITEM!" set ITEM=!KEYCHAIN_PATH!\!ITEM!
			if exist "!ITEM!" "!TOOLCHAIN_PATH!\openssl.exe" x509 -in "!ITEM!" -inform der -noout -subject -nameopt multiline 2>nul | findstr " commonName " >> "!TEMPDIR!\cert-CommonName.txt"
			if exist "!ITEM!" "!TOOLCHAIN_PATH!\openssl.exe" x509 -in "!ITEM!" -inform der -noout -subject -nameopt multiline 2>nul | findstr " organizationalUnit " >> "!TEMPDIR!\cert-OrganizationalUnit.txt"
		)
	)
	if exist "!TEMPDIR!\cert-CommonName.txt" (
		rem // read the output (i.e. the contents of the temporary file) into the variable
		set SIGNER=&set /p SIGNER=< "!TEMPDIR!\cert-CommonName.txt"
		rem // if a signer was retrieved, read what's after the " = " separator
		if not "_!SIGNER!_"=="__" (
			for /f "tokens=2 delims=^=" %%a in ("!SIGNER!") do set SIGNER=%%a
			set SIGNER=!SIGNER:~1!
		) else echo WARNING: unable to identify signer. Is your identity string correct?
	)
	if exist "!TEMPDIR!\cert-OrganizationalUnit.txt" (
		rem // read the output (i.e. the contents of the temporary file) into the variable
		set TEAMID=&set /p TEAMID=< "!TEMPDIR!\cert-OrganizationalUnit.txt"
		rem // if a team ID was retrieved, read what's after the " = " separator
		if not "_!TEAMID!_"=="__" (
			for /f "tokens=2 delims=^=" %%a in ("!TEAMID!") do set TEAMID=%%a
			set TEAMID=!TEAMID:~1!
		) else echo WARNING: unable to identify team ID. Is your identity string correct?
	)
)
rem // see if the optional provisioning profile exists in either the current directory or the keychain directory and build its absolute pathname
if not "_!PROVISION!_"=="__" (
	if exist ".\!PROVISION!" set PROVISION=%CD%\!PROVISION!&                          goto :provisionok
	if exist "!PROVISION!"                                                            goto :provisionok
	if exist "!KEYCHAIN_PATH!\!PROVISION!" set PROVISION=!KEYCHAIN_PATH!\!PROVISION!& goto :provisionok
	rem // at this point the provisioning profile specified in !PROVISION! does not exist on the filesystem
	if "_!NO_EXPLICIT_IDENTITY!_"=="_true_" (
		echo ERROR: the provisioning profile !PROVISION! specified in the default signing identity has not been found.
	) else echo ERROR: the specified provisioning profile !PROVISION! has not been found.
	goto :exit_error
)
:provisionok
set NO_EXPLICIT_IDENTITY=
set DEFAULT_CERTIFICATES=
set DEFAULT_PRIVATE_KEY=
set DEFAULT_PASSPHRASE=
set DEFAULT_PROVISION=
set LINE=
set VALUE=
set ITEM=


rem // if we were specified a working directory, change to it
if not "_!WORKING_DIR!_"=="__" (
	rem // change the working directory
	echo Entering directory !WORKING_DIR!
	cd /d "!WORKING_DIR!"
	if not "!errorlevel!"=="0" (
		rem // if directory change failed, exit
		echo NOTE: to build over a network path, please mount that path as a network drive.
		goto :exit_error
	)
)


rem // have we NOT been specified an Xcode project ? If so, see if one (and only one) exists in that directory
if "_!XCNAME!_"=="__" (
	set CANDIDATE_XCNAME=
	for /f "usebackq delims=" %%a in (`dir /b "*.xcodeproj" 2^>nul`) do (
		if not "_!CANDIDATE_XCNAME!_"=="__" (
			echo ERROR: Multiple Xcode projects exist in this directory. Specify -xcname to select the one to build.
			echo Example: -xcname !CANDIDATE_XCNAME!
			goto :exit_error
		)
		set CANDIDATE_XCNAME=%%~na
	)
	rem // a single .pbxproj directory was found, use it as the Xcode project name
	if not "_!CANDIDATE_XCNAME!_"=="__" set XCNAME=!CANDIDATE_XCNAME!
)


rem ////////////////////////////////////


rem // dumb-proof checks. No need to go further if some specified files don't exist
if not "_!ENTITLEMENTS_PLIST!_"=="__" if not exist "!ENTITLEMENTS_PLIST!" echo ERROR: the specified entitlements file !ENTITLEMENTS_PLIST! has not been found.&	goto :exit_error
if not "_!ARTWORK!_"=="__"            if not exist "!ARTWORK!"            echo ERROR: the specified store artwork image file !ARTWORK! has not been found.&     goto :exit_error
if not "_!PREPKG_SCRIPT!_"=="__"      if not exist "!PREPKG_SCRIPT!"      echo ERROR: the specified pre-packaging script !PREPKG_SCRIPT! has not been found.&   goto :exit_error


rem // does this project rely on CocoaPods ?
if exist "Podfile" (

	rem // does the user want to NOT use the CocoaPods daemon ?
	if "_!SKIP_COCOAPODS!_"=="_true_" (
		echo NOTICE: CocoaPods project but manual linking requested. Skipping pods.

	rem // else does the user NOT want to skip those dependencies?
	) else if not "_!SKIP_DEPENDENCIES!_"=="_true_" (

		rem // if we want to rebuild everything, delete the Pods directory first
		if "_!WANT_REBUILDALL!_"=="_true_" rmdir /s /q "Pods" > nul 2>&1

		rem // is the Pods directory NOT populated? (i.e. we should do a "pod install" or "pod update")
		if not exist "Pods\Pods.xcodeproj\Project.pbxproj" (
			echo.
			echo INFO: this project relies on CocoaPods to install extra packages.

			rem // is the CocoaPods daemon NOT usable yet?
			if not exist "!TOOLCHAIN_PATH!\cocoapods\cocoapods.key" (
				echo There are two ways of getting those packages required by your project.
				echo.
				echo 1. Manually by downloading, setting them up and injecting them in the build.
				echo    This is the legacy method that was recommended before the CocoaPods daemon
				echo    existed. It's still working, and covered in the builder's documentation.
				echo.
				echo 2. Automatically by using the Windows CocoaPods daemon. This daemon implements
				echo    a BSD UNIX subsystem that's compatible enough with macOS to make CocoaPods
				echo    work on Windows. You must however unlock this subsystem with a key as it's
				echo    a separate purchase.
				echo.
				choice /C 12 /N /M "Which method do you choose? "
				if "_!errorlevel!_"=="_1_" (
					echo Renaming Podfile to Podfile.bak so that this message doesn't appear again.
					move Podfile Podfile.bak
					echo.
					echo Please refer to the documentation on how to setup your project dependencies.
					echo Once this is done, you can build your project again.
					echo.
					call :open_url "%SCRIPT_DIRECTORY%readme.html#troubleshooting"
					goto :exit_error
				)
			)

			rem // either CocoaPods daemon is usable, or user wants to enable it, so start it up
			set POD_COMMAND=pod install
			echo The CocoaPods command about to be sent is "!POD_COMMAND!"
			echo [A]ccept or [C]hange? Default is to [A]ccept if no answer in 5 seconds.
			choice /T 5 /C ac /D a /N > nul
			if "_!errorlevel!_"=="_2_" set /p POD_COMMAND=Enter command to send to CocoaPods daemon: 

			echo.
			echo Delegating "!POD_COMMAND!" job to CocoaPods daemon...

			rem // tell the world CocoaPods is taking over from now on
			title iOS Build Environment - running CocoaPods...

			rem // optional argument to pod install: --repo-update
			call "!TOOLCHAIN_PATH!\cocoapods\cocoapods.cmd" !POD_COMMAND!
			if not "!errorlevel!"=="0" goto :exit_error

			rem // tell the world we've regained control
			title iOS Build Environment
		)

		rem // we use CocoaPods, and the Pods directory is populated. Fine! we can proceed...
	)
)


rem // assume we have no Info.plist until known otherwise
set INFO_PLIST=


rem // are we building a Xcode project ?
if exist "!XCNAME!.xcodeproj\project.pbxproj" (

	rem // make sure the pbxprojparse map file is re-generated
	rem // NOTE: this should be done before the first pbxprojparse call in this script. So move that line upper when necessary.
	del /f /q "!XCNAME!.xcodeproj\project.pbxproj.pbxprojparse-map" > nul 2>&1

	rem // locate the Xcode project dir and source root (not necessarily the same!)
	call :dirname "!XCNAME!.xcodeproj" XCDIR
	set XCDIR=!XCDIR:\=/!
	if "_!XCDIR!_"=="__" set XCDIR=.
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" -showroot > %TO_TEMPVAR%
	set SRCROOT=!TEMPVAR!
	if "_!SRCROOT!_"=="__" set SRCROOT=.

	rem // if we don't have a specific target, get the name of the default one from the Xcode project file
	if "_!TARGET_NAME!_"=="__" (
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" -listtargets | findstr /c:" \[default\]" > %TO_TEMPVAR%
		if not "_!TEMPVAR!_"=="__" set TARGET_NAME=!TEMPVAR:~0,-10!
		if "_!TARGET_NAME!_"=="__" (
			echo ERROR: This Xcode project has no default target. Specify -xcpname to select which target to build.
			echo The available targets are:
			"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" -listtargets
			goto :exit_error
		)
	)

	rem // if we don't have a specific configuration, get the name of the default one from the Xcode project file
	if "_!CONFIGURATION!_"=="__" (
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" -target "!TARGET_NAME!" -listconfigurations | findstr /c:" \[default\]" > %TO_TEMPVAR%
		if not "_!TEMPVAR!_"=="__" set CONFIGURATION=!TEMPVAR:~0,-10!
		if "_!CONFIGURATION!_"=="__" (
			echo ERROR: The target !TARGET_NAME! in this Xcode project has no default build configuration. Specify -xcconf to select which configuration to build.
			echo The available configurations for !TARGET_NAME! are:
			"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" -target "!TARGET_NAME!" -listconfigurations
			goto :exit_error
		)
	)

	rem // dumb-proof check. No need to go further if we're trying to build something that doesn't match our SDK.
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query SUPPORTED_PLATFORMS > %TO_TEMPVAR%
	if "_!TEMPVAR:%SDK_PLATFORM_NAME%=!_"=="_!TEMPVAR!_" (
		echo ERROR: The target !TARGET_NAME! in this Xcode project is for the '!TEMPVAR!' platform, whereas your installed SDK is for '!SDK_PLATFORM_NAME!'. Either get a '!TEMPVAR!' SDK, or change this Xcode project's target to '!SDK_PLATFORM_NAME!'.
		goto :exit_error
	)

	rem // HACK: prior to doing anything with a Xcode project, make sure it doesn't use .xib files
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -resources > nul
	if not "!errorlevel!"=="0" goto :exit_error

	rem // HACK: prior to doing anything with a Xcode project, make sure it doesn't use shell scripts
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -resources -noscripts > nul
	if not "!errorlevel!"=="0" goto :exit_error

	rem // figure out the product name from the .pbxproj file
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query PRODUCT_NAME > %TO_TEMPVAR%
	if not "_!TEMPVAR!_"=="__" set NAME=!TEMPVAR!
	if "_!NAME!_"=="__" (
		set NAME=!TARGET_NAME!
		rem echo WARNING: Xcode product name not set, defaulting to !NAME!
	)

	rem // figure out the build directory (FIXME: conflicts with -builddir script argument)
rem //	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query BUILD_DIR > %TO_TEMPVAR%
rem //	if not "_!TEMPVAR!_"=="__" set BUILD_DIR=!TEMPVAR!
rem //	if "_!BUILD_DIR!_"=="__" set BUILD_DIR=build
rem //	set BUILD_ROOT=!BUILD_DIR!
rem //	set OBJROOT=!BUILD_DIR!
rem //	set SYMROOT=!BUILD_DIR!
rem //	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query CONFIGURATION_BUILD_DIR > %TO_TEMPVAR%
rem //	if not "_!TEMPVAR!_"=="__" set CONFIGURATION_BUILD_DIR=!TEMPVAR!
rem //	if "_!CONFIGURATION_BUILD_DIR!_"=="__" set CONFIGURATION_BUILD_DIR=!BUILD_DIR!\!CONFIGURATION!
rem //	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query TARGET_BUILD_DIR > %TO_TEMPVAR%
rem //	if not "_!TEMPVAR!_"=="__" set TARGET_BUILD_DIR=!TEMPVAR!
rem //	if "_!TARGET_BUILD_DIR!_"=="__" set TARGET_BUILD_DIR=!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\!TARGET_NAME!

	rem // figure out the output entrypoint type from the .pbxproj file
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -showtype > %TO_TEMPVAR%
	if "_!TEMPVAR!_"=="__" (
		rem // special case: if this project has no com.apple.product-type known value, then there's nothing to build
		echo Nothing to build here, skipping.
		goto :exit_success
	) else if "_!TEMPVAR!_"=="_com.apple.product-type.application_"     ( set ENTRYPOINT=crt
	) else if "_!TEMPVAR!_"=="_com.apple.product-type.framework_"       ( set ENTRYPOINT=framework
	) else if "_!TEMPVAR!_"=="_com.apple.product-type.app-extension_"   ( set ENTRYPOINT=appex
	) else if "_!TEMPVAR!_"=="_com.apple.product-type.bundle_"          ( set ENTRYPOINT=bundle
	) else if "_!TEMPVAR!_"=="_com.apple.product-type.library.static_"  ( set ENTRYPOINT=lib
	) else if "_!TEMPVAR!_"=="_com.apple.product-type.library.dynamic_" ( set ENTRYPOINT=dylib
	)

	rem // figure out the Info.plist location from the .pbxproj file (if appliable. Applies to "crt", "framework", "appex" and "bundle")
	if not "_!ENTRYPOINT!_"=="_lib_" if not "_!ENTRYPOINT!_"=="_dylib_" (
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query INFOPLIST_FILE -line_format "!SRCROOT!/%%s" > %TO_TEMPVAR%
		if not "_!TEMPVAR!_"=="__" set INFO_PLIST=!TEMPVAR!
		if "_!INFO_PLIST!_"=="__" if exist "Info.plist" (
			set INFO_PLIST=Info.plist
			echo WARNING: Info.plist file location not set in Xcode project, but an Info.plist file exists in the project directory. Defaulting to it.
		)
		rem // while some bundles and frameworks might not have an Info.plist, apps and app extensions MUST have one
		if "_!INFO_PLIST!_"=="__" (
			if "_!ENTRYPOINT!_"=="_crt_" (
				echo ERROR: Info.plist file location not set in Xcode project, and no Info.plist file exists in the project directory.
				goto :exit_error
			) else if "_!ENTRYPOINT!_"=="_appex_" (
				echo ERROR: Info.plist file location not set in Xcode project, and no Info.plist file exists in the project directory.
				goto :exit_error
			)
		)
	)

	rem // compare project and build mtimes and see if this project needs to be rebuilt
	"!TOOLCHAIN_PATH!\is_newer.exe" "!XCNAME!.xcodeproj\project.pbxproj" "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\!TARGET_NAME!\srclist.txt" 2>nul && (
		if not "_%DEBUG_ARG%_"=="__" echo !NAME! will be rebuilt because its Xcode project is more recent than its build directory.
		set NEEDS_REBUILD=true
	)

	rem // if unspecified, figure out the target device architectures from the .pbxproj file
	if "_!ARCHS!_"=="_all_" (
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query ARCHS > %TO_TEMPVAR%
		if not "_!TEMPVAR!_"=="__" set ARCHS=!TEMPVAR!
	)

	rem // figure out the target device families from the .pbxproj file
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query TARGETED_DEVICE_FAMILY > %TO_TEMPVAR%
	if "_!TEMPVAR!_"=="_1_"          ( set FAMILY=iphone
	) else if "_!TEMPVAR!_"=="_2_"   ( set FAMILY=ipad
	) else if "_!TEMPVAR!_"=="_1,2_" ( set FAMILY=all
	) else                           ( set FAMILY=all
	)

	rem // figure out the minimum OS version number from the .pbxproj file
	if "_!TARGET_PLATFORM_NAME!_"=="_ios_" (
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query IPHONEOS_DEPLOYMENT_TARGET > %TO_TEMPVAR%
	) else if "_!TARGET_PLATFORM_NAME!_"=="_watchos_" (
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query WATCHOS_DEPLOYMENT_TARGET > %TO_TEMPVAR%
	) else if "_!TARGET_PLATFORM_NAME!_"=="_tvos_" (
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query TVOS_DEPLOYMENT_TARGET > %TO_TEMPVAR%
	) else if "_!TARGET_PLATFORM_NAME!_"=="_xros_" (
		rem // FIXME: not sure about this variable name?
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query XROS_DEPLOYMENT_TARGET > %TO_TEMPVAR%
	) else (
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query MACOSX_DEPLOYMENT_TARGET > %TO_TEMPVAR%
	)
	if not "_!TEMPVAR!_"=="__" set MINOSVER=!TEMPVAR!

	rem // figure out the C compiler language standard from the .pbxproj file (ignore if "compiler-default")
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query GCC_C_LANGUAGE_STANDARD -line_format "-std=%%s" > %TO_TEMPVAR%
	if not "_!TEMPVAR!_"=="__" if not "_!TEMPVAR!_"=="_compiler-default_" set CFLAGS=!CFLAGS! !TEMPVAR!

	rem // figure out the C++ compiler language standard from the .pbxproj file (ignore if "compiler-default")
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query CLANG_CXX_LANGUAGE_STANDARD -line_format "-std=%%s" > %TO_TEMPVAR%
	if not "_!TEMPVAR!_"=="__" if not "_!TEMPVAR!_"=="_compiler-default_" set CXXFLAGS=!CXXFLAGS! !TEMPVAR!

	rem // figure out the C language compiler optimization level from the .pbxproj file
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query GCC_OPTIMIZATION_LEVEL -line_format "-O%%s" > %TO_TEMPVAR%
	if not "_!TEMPVAR!_"=="__" set CPPFLAGS=!CPPFLAGS! !TEMPVAR!

	rem // figure out whether duplicate symbols are allowed or disabled
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query GCC_NO_COMMON_BLOCKS -line_format " \"%%s\"" > %TO_TEMPVAR%
	if not "_!TEMPVAR!_"=="_NO_" set CPPFLAGS=!CPPFLAGS! -fcommon

	rem // figure out the GCC_PREPROCESSOR_DEFINITIONS, OTHER_CFLAGS and OTHER_CPLUSPLUSFLAGS from the .pbxproj file (the former affects the latter)
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query GCC_PREPROCESSOR_DEFINITIONS -line_format " -D\"%%s\"" > %TO_TEMPVAR%
	if not "_!TEMPVAR!_"=="__" set CPPFLAGS=!CPPFLAGS! !TEMPVAR!

	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query OTHER_CFLAGS -line_format " \"%%s\"" > %TO_TEMPVAR%
	if not "_!TEMPVAR!_"=="__" set CPPFLAGS=!CPPFLAGS! !TEMPVAR!

	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query OTHER_CPLUSPLUSFLAGS -line_format " \"%%s\"" > %TO_TEMPVAR%
	if not "_!TEMPVAR!_"=="__" set CXXFLAGS=!CXXFLAGS! !TEMPVAR!

	rem // figure out whether ARC is globally disabled from the .pbxproj file ONLY if the contrary is NOT specified in the user-supplied flags
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query CLANG_ENABLE_OBJC_ARC > %TO_TEMPVAR%
	if "_!TEMPVAR!_"=="_NO_" (
		rem // ARC is explicitly disabled
		rem // this two-pass check is necessary because testing a substitution on an undefined environment variable yields... the string to substitute itself. Stupid Microsoft.
		if "_!IOS_CPPFLAGS!_"=="__" set CPPFLAGS=!CPPFLAGS! -fno-objc-arc
		if "_!IOS_CPPFLAGS:-fobjc-arc=!_"=="_!IOS_CPPFLAGS!_" set CPPFLAGS=!CPPFLAGS! -fno-objc-arc

		rem // figure out whether we allow weak references in non-ARC code ONLY if the contrary is NOT specified in the user-supplied flags
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query CLANG_ENABLE_OBJC_WEAK > %TO_TEMPVAR%
		if "_!TEMPVAR!_"=="_NO_" (
			rem // weak references in non-ARC code are explicitly disabled
			rem // this two-pass check is necessary because testing a substitution on an undefined environment variable yields... the string to substitute itself. Stupid Microsoft.
			if "_!IOS_CPPFLAGS!_"=="__" set CPPFLAGS=!CPPFLAGS! -fno-objc-weak
			if "_!IOS_CPPFLAGS:-fobjc-weak=!_"=="_!IOS_CPPFLAGS!_" set CPPFLAGS=!CPPFLAGS! -fno-objc-weak
		) else (
			rem // weak references in non-ARC code are NOT disabled (i.e. either "YES", which means explicitly enabled, or no value at all, which means enabled by default)
			rem // this two-pass check is necessary because testing a substitution on an undefined environment variable yields... the string to substitute itself. Stupid Microsoft.
			if "_!IOS_CPPFLAGS!_"=="__" set CPPFLAGS=!CPPFLAGS! -fobjc-weak
			if "_!IOS_CPPFLAGS:-fno-objc-weak=!_"=="_!IOS_CPPFLAGS!_" set CPPFLAGS=!CPPFLAGS! -fobjc-weak
		)
	) else (
		rem // ARC is NOT disabled (i.e. either "YES", which means explicitly enabled, or no value at all, which means enabled by default)
		rem // this two-pass check is necessary because testing a substitution on an undefined environment variable yields... the string to substitute itself. Stupid Microsoft.
		if "_!IOS_CPPFLAGS!_"=="__" set CPPFLAGS=!CPPFLAGS! -fobjc-arc
		if "_!IOS_CPPFLAGS:-fno-objc-arc=!_"=="_!IOS_CPPFLAGS!_" set CPPFLAGS=!CPPFLAGS! -fobjc-arc
	)

	rem // semantic includes (@import ...) should now be enabled by default, UNLESS the contrary is specified in the user-supplied flags
	rem // Note that this WILL disable ccache, because ccache doesn't know how to parse clang modules (TODO: implement this in my own ccache port)
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query CLANG_ENABLE_MODULES > %TO_TEMPVAR%
	if not "_!TEMPVAR!_"=="_NO_" (
		rem // semantic includes are not explicitly disabled, so enable them
		rem // WARNING: ONLY C SEMANTICS MODULES ARE ENABLED BECAUSE THE C++ ONES ARE STILL UNSTABLE AND APPLE SILENTLY DISABLE THEM (credits: Stefan Vasilev)
		rem //          read: https://github.com/facebook/infer/commit/1be7827981048291331343be06b0e84be45592c3
		rem //          read: https://lists.llvm.org/pipermail/cfe-commits/Week-of-Mon-20140929/115672.html
		rem // this two-pass check is necessary because testing a substitution on an undefined environment variable yields... the string to substitute itself. Stupid Microsoft.
		if "_!IOS_CPPFLAGS!_"=="__" set CPPFLAGS=!CPPFLAGS! -fmodules -fno-cxx-modules
		if "_!IOS_CPPFLAGS:-fno-modules=!_"=="_!IOS_CPPFLAGS!_" set CPPFLAGS=!CPPFLAGS! -fmodules -fno-cxx-modules
	)

	rem // figure out the precompiled header flags from the .pbxproj file (NOTE: their location seems to be relative to XCDIR and not SOURCE_ROOT...)
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query GCC_PREFIX_HEADER -line_format " \"!XCDIR!/%%s\"" > %TO_TEMPVAR%
	if not "_!TEMPVAR!_"=="__" set PCH=!PCH! !TEMPVAR!

	rem // figure out the framework search paths, the dylibs against which to link, and the extra linker flags from the .pbxproj file
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query FRAMEWORK_SEARCH_PATHS -line_format " -F\"%%s\"\n" >> "!TEMPDIR!\LDFLAGS.txt"
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -dylibs >> "!TEMPDIR!\LDFLAGS.txt"
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" OTHER_LDFLAGS_FRAMEWORK= -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query OTHER_LDFLAGS -line_format " %%s" >> "!TEMPDIR!\LDFLAGS.txt"
	echo.>> "!TEMPDIR!\LDFLAGS.txt"

	rem // figure out the frameworks against which to link from the .pbxproj file. Note that on iOS, "Foundation" and "UIKit" seem to be implicit (as seen in the PlayCanvas example)
	echo -framework Foundation -framework UIKit>> "!TEMPDIR!\FRAMEWORKS_LDFLAGS.txt"
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -frameworks -line_format " %%s" >> "!TEMPDIR!\FRAMEWORKS_LDFLAGS.txt"

	rem // if not specified yet, figure out whether we want to strip the output file
	if "_!WANT_STRIP!_"=="__" (
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query COPY_PHASE_STRIP > %TO_TEMPVAR%
		if "_!TEMPVAR!_"=="_YES_" (
			set WANT_STRIP=true
			if "_!WANT_STRIP_BITCODE!_"=="__" (
				set WANT_STRIP_BITCODE=true
				"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query STRIP_BITCODE_FROM_COPIED_FILES > %TO_TEMPVAR%
				if "_!TEMPVAR!_"=="_NO_" set WANT_STRIP_BITCODE=false
			)
		)
	)

	rem // figure out whether extra binaries (frameworks or dylibs) need to be embedded from the .pbxproj file
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -embeddedbinaries -line_format "\"%%s\"\n" >> "!TEMPDIR!\EMBEDDED_BINARIES.txt"

	rem // figure out whether other resources need to be added from the .pbxproj file, EXCLUDING LOCALIZED RESOURCES (they'll be handled later)
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -resources -variant none -line_format " \"%%s\"" > %TO_TEMPVAR%
	if not "_!TEMPVAR!_"=="__" set RES=!RES! !TEMPVAR!

	rem // if not specified yet, figure out whether to crush PNG files (default: yes)
	if "_!WANT_PNGCRUSH!_"=="__" (
		set WANT_PNGCRUSH=true
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query IPHONE_OPTIMIZE_OPTIONS > %TO_TEMPVAR%
		if not "_!TEMPVAR!_"=="__" if not "_!TEMPVAR:-skip-PNGs=!_"=="_!TEMPVAR!_" set WANT_PNGCRUSH=
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query COMPRESS_PNG_FILES > %TO_TEMPVAR%
		if "_!TEMPVAR!_"=="_NO_" set WANT_PNGCRUSH=
	)

	rem // see if an entitlements .plist file is specified in the .pbxproj file
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query CODE_SIGN_ENTITLEMENTS > %TO_TEMPVAR%
	if not "_!TEMPVAR!_"=="__" if exist "!TEMPVAR!" (
		rem // if not "_!ENTITLEMENTS_PLIST!_"=="__" (
		rem // 	echo Notice: automatically merging extra entitlements from [...]
		rem // 	TODO: merge .plist files together
		rem // )
		set ENTITLEMENTS_PLIST=!TEMPVAR!
	)

	rem // if the current configuration is called "Debug" AND we don't have any -g flag already, enable debug symbols generation with -gmodules
	if "_!CONFIGURATION!_"=="_Debug_" echo " !CPPFLAGS! !CFLAGS! !CXXFLAGS!"|findstr /c:" -g" >nul 2>&1 || set CPPFLAGS=!CPPFLAGS! -gmodules

	rem // resolve $(SRCROOT) and $(PROJECT_DIR) into "." in CPPFLAGS, CFLAGS, CXXFLAGS, PCH and LDFLAGS
	rem // resolve $(inherited) to nothing in CPPFLAGS, CFLAGS, CXXFLAGS, PCH and LDFLAGS
	rem // FIXME: are the two cases above still a problem now that pbxprojparse resolves environment variables ?
	rem // then remove empty quotes in CPPFLAGS, CFLAGS, CXXFLAGS, PCH and LDFLAGS to clean up a bit
	if not "_!CPPFLAGS!_"=="__" (
		set CPPFLAGS=!CPPFLAGS:$^(SRCROOT^)=.!
		set CPPFLAGS=!CPPFLAGS:$^(PROJECT_DIR^)=.!
		set CPPFLAGS=!CPPFLAGS:$^(inherited^)=!
		set CPPFLAGS=!CPPFLAGS:^"^"=!
	)
	if not "_!CFLAGS!_"=="__" (
		set CFLAGS=!CFLAGS:$^(SRCROOT^)=.!
		set CFLAGS=!CFLAGS:$^(PROJECT_DIR^)=.!
		set CFLAGS=!CFLAGS:$^(inherited^)=!
		set CFLAGS=!CFLAGS:^"^"=!
	)
	if not "_!CXXFLAGS!_"=="__" (
		set CXXFLAGS=!CXXFLAGS:$^(SRCROOT^)=.!
		set CXXFLAGS=!CXXFLAGS:$^(PROJECT_DIR^)=.!
		set CXXFLAGS=!CXXFLAGS:$^(inherited^)=!
		rem // this case has been seen: OTHER_CXXFLAGS is defined in the .pbxproj containing OTHER_CFLAGS, but OTHER_CFLAGS already are in CPPFLAGS
		set CXXFLAGS=!CXXFLAGS:$^(OTHER_CFLAGS^)=!
		set CXXFLAGS=!CXXFLAGS:^"^"=!
	)
	if not "_!PCH!_"=="__" (
		set PCH=!PCH:$^(SRCROOT^)=.!
		set PCH=!PCH:$^(PROJECT_DIR^)=.!
		set PCH=!PCH:$^(inherited^)=!
		set PCH=!PCH:^"^"=!
	)

	rem // sometimes ld is passed options in the Clang syntax, i.e. "-Wl,-rpath -Wl,./Frameworks" instead of "-rpath ./Frameworks". Just fix that.
	type "!TEMPDIR!\LDFLAGS.txt"|findstr /c:"-Wl," && (
		echo WARNING: this Xcode project defines linker flags in compiler format ^(-Wl,...^). Attempting to fix.
		copy /b "!TEMPDIR!\LDFLAGS.txt" "!TEMPDIR!\LDFLAGS.txt.tmp" > nul
		"!TOOLCHAIN_PATH!\sed.exe" -e "s/-Wl,//g" "!TEMPDIR!\LDFLAGS.txt.tmp" > "!TEMPDIR!\LDFLAGS.txt"
		del /f /q "!TEMPDIR!\LDFLAGS.txt.tmp"
	)
)

rem ////////////////////////////////////


rem // are we NOT building a library AND is no Info.plist explicitly defined BUT an Info.plist file exists here ? if so, assume it's the Info.plist we need
if not "_!ENTRYPOINT!_"=="_lib_" if not "_!ENTRYPOINT!_"=="_dylib_" if "_!INFO_PLIST!_"=="__" if exist "Info.plist" (
	set INFO_PLIST=Info.plist
	echo WARNING: Info.plist file location not set but an Info.plist file exists in the project directory, defaulting to it.
)


rem // make sure project name is set. If not, see if an Info.plist file exist and extract name from it
if "_!NAME!_"=="__" if exist "!INFO_PLIST!" (
	rem // read CFBundleName from Info.plist and strip quotes
	"!TOOLCHAIN_PATH!\plconvert.exe" "!INFO_PLIST!" /query CFBundleName > %TO_TEMPVAR%
	if not "_!TEMPVAR!_"=="__" set NAME=!TEMPVAR:"=!
	rem // if it exists, make sure the project name does NOT contain a variable, clear it if it does
	if not "_!NAME!_"=="__" (
		echo _!NAME!_|findstr /c:"\$" 1>nul
		if "!errorlevel!"=="0" set NAME=
	)
	rem // if CFBundleName is not set or contained an unresolved variable, fallback to CFBundleDisplayName
	if "_!NAME!_"=="__" (
		rem // read CFBundleDisplayName from Info.plist and strip quotes
		"!TOOLCHAIN_PATH!\plconvert.exe" "!INFO_PLIST!" /query CFBundleDisplayName > %TO_TEMPVAR%
		if not "_!TEMPVAR!_"=="__" set NAME=!TEMPVAR:"=!
		rem // if the project name was STILL not found in the Info.plist, bomb out
		if "_!NAME!_"=="__" echo ERROR: project name not specified and neither CFBundleName nor CFBundleDisplayName are defined in the Info.plist.& goto :exit_error
		rem // make sure the project name does NOT contain a variable, bomb out if it does
		echo _!NAME!_|findstr /c:"\$" 1>nul
		if "!errorlevel!"=="0" echo ERROR: project name not specified and neither CFBundleName nor CFBundleDisplayName contain an explicit value in the Info.plist.& goto :exit_error
	)
)
if "_!NAME!_"=="__" (
	echo ERROR: project name not specified and guessing failed. Use -? for usage.
	goto :exit_error
)


rem // make sure we have a valid target name (most often from Xcode project)
if "_!TARGET_NAME!_"=="__" (
	rem // set TARGET_NAME to be the same as project name if it's unset
	set TARGET_NAME=!NAME!
)


rem // make sure CPU sets is set, default to armv7+arm64 if not
set WANT_ALL_ARCHS=no
if "_!ARCHS!_"=="__"                 set WANT_ALL_ARCHS=yes
if "_!ARCHS!_"=="_all_"              set WANT_ALL_ARCHS=yes
if "_!ARCHS!_"=="_%ARCHS_STANDARD%_" set WANT_ALL_ARCHS=yes
if "_!WANT_ALL_ARCHS!_"=="_yes_" (
	if "_!TARGET_PLATFORM_NAME!_"=="_macos_" (
		rem // when targeting macOS, build for Intel x64 and ARM64 CPUs
		set ARCHS=x86_64 arm64
	) else (
		rem // when targeting everything else, build for ARM CPUs in 32 and 64-bit variants
		set ARCHS=arm64 armv7
	)
	echo WARNING: CPU sets not specified and guessing failed, defaulting to !ARCHS: =+!.
)
set WANT_ALL_ARCHS=


rem // make sure minimum OS version is set, is in the format X.X, and is equal or higher than 6.0
if "_!MINOSVER!_"=="__" (
	if        "_!TARGET_PLATFORM_NAME!_"=="_ios_"     ( set MINOSVER=6.0
	) else if "_!TARGET_PLATFORM_NAME!_"=="_watchos_" ( set MINOSVER=1.0
	) else if "_!TARGET_PLATFORM_NAME!_"=="_tvos_"    ( set MINOSVER=1.0
	) else if "_!TARGET_PLATFORM_NAME!_"=="_xros_"    ( set MINOSVER=1.0
	) else                                            ( set MINOSVER=10.6
	)
	echo WARNING: minimum !TARGET_PLATFORM_NAME! version not specified and guessing failed, defaulting to !MINOSVER!.
)
if "_!MINOSVER:.=!_"=="_!MINOSVER!_" set MINOSVER=!MINOSVER!.0
for /f "tokens=1 delims=." %%v in ("!MINOSVER!") do set MINOSVER_MAJOR=%%v
rem // IMPORTANT: do NOT use double quotes "!MINOSVER!" LSS "6.0" for numeric compare, else cmd would do a STRING compare
if        "_!TARGET_PLATFORM_NAME!_"=="_ios_"     ( if !MINOSVER_MAJOR! LSS 6 set MINOSVER=6.0
) else if "_!TARGET_PLATFORM_NAME!_"=="_watchos_" ( if !MINOSVER_MAJOR! LSS 1 set MINOSVER=1.0
) else if "_!TARGET_PLATFORM_NAME!_"=="_tvos_"    ( if !MINOSVER_MAJOR! LSS 1 set MINOSVER=1.0
) else if "_!TARGET_PLATFORM_NAME!_"=="_xros_"    ( if !MINOSVER_MAJOR! LSS 1 set MINOSVER=1.0
) else                                            ( if !MINOSVER_MAJOR! LSS 10 set MINOSVER=10.0
)


rem // make sure entrypoint type is set. If not already set AND if we have an Info.plist, deduce it from CFBundlePackageType
if "_!ENTRYPOINT!_"=="__" if exist "!INFO_PLIST!" (
	rem // read CFBundlePackageType from Info.plist and strip quotes
	"!TOOLCHAIN_PATH!\plconvert.exe" "!INFO_PLIST!" /query CFBundlePackageType > %TO_TEMPVAR%
	if not "_!TEMPVAR!_"=="__" set TEMPVAR=!TEMPVAR:"=!
	rem // this logic is the one of https://developer.apple.com/library/ios/documentation/general/Reference/InfoPlistKeyReference/Articles/CoreFoundationKeys.html#//apple_ref/doc/uid/20001431-111321
	if not "_!TEMPVAR!_"=="__"     set ENTRYPOINT=bundle
	if     "_!TEMPVAR!_"=="_APPL_" set ENTRYPOINT=crt
	if     "_!TEMPVAR!_"=="_FMWK_" set ENTRYPOINT=framework
	echo WARNING: entrypoint type not specified, deduced as !ENTRYPOINT! from !INFO_PLIST!.
)
if "_!ENTRYPOINT!_"=="__" (
	set ENTRYPOINT=crt
	echo WARNING: entrypoint type not specified, defaulting to !ENTRYPOINT!.
)


rem // if building an iOS application (CRT entrypoint), make sure device family is set, default to "all" (iPhones + iPads) if not
if "_!SDK_PLATFORM_NAME!_"=="_iphoneos_" if "_!ENTRYPOINT!_"=="_crt_" if "_!FAMILY!_"=="__" (
	set FAMILY=all
	echo WARNING: target device family not specified and guessing failed, defaulting to !FAMILY!.
)


rem // determine output file name: if it's a library, append the correct prefix and suffix if necessary
set OUTFILE=!NAME!
if "_!ENTRYPOINT!_"=="_lib_" (
	rem // see if it does NOT begin in "lib" and ends in ".a", if so, prepend "lib" and append ".a" suffix
	if not "_!OUTFILE:~0,3!_"=="_lib_" set OUTFILE=lib!OUTFILE!
	if not "_!OUTFILE:~-2!_"=="_.a_" set OUTFILE=!OUTFILE!.a
)
if "_!ENTRYPOINT!_"=="_dylib_" (
	rem // see if it does NOT begin in "lib" and ends in ".dylib", if so, prepend "lib" and append ".dylib" suffix
	if not "_!OUTFILE:~0,3!_"=="_lib_" set OUTFILE=lib!OUTFILE!
	if not "_!OUTFILE:~-6!_"=="_.dylib_" set OUTFILE=!OUTFILE!.dylib
)


rem // are we building a Unity project ?
if exist "!XCNAME!.xcodeproj\project.pbxproj" if "_!XCNAME!_"=="_Unity-iPhone_" (
	rem // for Unity projects, suppress all warnings and optimize for speed + size unless previously specified. Also, Microsoft extensions must be enabled (for Mono I suppose.)
	set CPPFLAGS=!CPPFLAGS! -w
	if "_ !CPPFLAGS: -O=!_"=="_ !CPPFLAGS!_" set CPPFLAGS=!CPPFLAGS! -Os
	set CPPFLAGS=!CPPFLAGS! -fms-extensions

	rem // if we are building the main application, add the default Unity resources (FIXME: work this out more elegantly from the .pbxproj file)
	if "_!ENTRYPOINT!_"=="_crt_" (
		set RES=!RES! *.png
		rem // older versions of Unity copy the Data folder using a post-build shell script, that's why we still need to hardcode it here if it's missing.
		set TEMPVAR=
		for %%a in (!RES!) do (
			set ITEM=%%a
			set ITEM=!ITEM:^"=!
			if "_!ITEM!_"=="_Data_" set TEMPVAR=Present
		)
		if "_!TEMPVAR!_"=="__" set RES=!RES! Data
	)

	rem // FIXME: I don't understand why this patch is needed
	rem // FACT: SOME VERSIONS of the Unity libraries export il2cpp::vm::Array::NewFull(Il2CppClass*, il2cpp_array_size_t*, il2cpp_array_size_t*) as il2cpp::vm::Array::NewFull(Il2CppClass*, unsigned long*, unsigned long*)
	rem //       SOME OTHERS ........................ export il2cpp::vm::Array::NewFull(Il2CppClass*, il2cpp_array_size_t*, il2cpp_array_size_t*) as il2cpp::vm::Array::NewFull(Il2CppClass*, int*, int*)
	rem //       ...which means that in some cases il2cpp_array_size_t is typedef'ed as unsigned long and in some other cases as int.
	rem // PROBLEM: the problem is here that the Clang compiler sometimes fails to pick up the right type for this typedef, leading to linker errors (symbols not found). I don't understand why yet.
	rem // The wokraround that I've chosen so far is to enforce the right definition with a #define at an appropriate place, i.e. after the typedef occured but before the types are used.
	rem // A good place for this patch is at the bottom of il2cpp-config.h. I'm not sure if this is a Unity bug, or mine.
	rem // ANOTHER PROBLEM is that sometimes the Unity libraries can't be dumped correctly with neither Apple's nm (error "archive member extends pasts the end of the file") nor with LLVM's
	rem // llvm-nm (error "fixed or VBR abbrev record with size > MaxChunkData") - and this one *IS* a genuine Unity bug. In these cases the right type for il2cpp_array_size_t is undecideable.
	if exist "Libraries\libil2cpp\include\il2cpp-config.h" (
		rem // guess a reasonable fallback default for this type for the case the Unity libraries symbol tables are corrupt (and pray that they be not. Gospody pomiluij...)
		set IL2CPP_ARRAY_SIZE_TYPE=size_t
		set UNITY_LIBRARY=Libraries\libiPhone-lib.a
		if exist "Libraries\libil2cpp.a" set UNITY_LIBRARY=Libraries\libil2cpp.a
		"!TOOLCHAIN_PATH!\nm.exe" -U "!UNITY_LIBRARY!" 2>nul | findstr "__ZN6il2cpp2vm5Array7NewFullEP11Il2CppClassP" > "!TEMPDIR!\nm-output.txt"
		rem // see how il2cpp::vm::Array::NewFull(Il2CppClass*, il2cpp_array_size_t*, il2cpp_array_size_t*) is exported: is il2cpp_array_size_t a long?
		type "!TEMPDIR!\nm-output.txt" | findstr "__ZN6il2cpp2vm5Array7NewFullEP11Il2CppClassPlS4_" > %TO_TEMPVAR%
		if not "_!TEMPVAR!_"=="__" set IL2CPP_ARRAY_SIZE_TYPE=long
		rem // see how il2cpp::vm::Array::NewFull(Il2CppClass*, il2cpp_array_size_t*, il2cpp_array_size_t*) is exported: is il2cpp_array_size_t an unsigned long?
		type "!TEMPDIR!\nm-output.txt" | findstr "__ZN6il2cpp2vm5Array7NewFullEP11Il2CppClassPmS4_" > %TO_TEMPVAR%
		if not "_!TEMPVAR!_"=="__" set IL2CPP_ARRAY_SIZE_TYPE=unsigned long
		rem // see how il2cpp::vm::Array::NewFull(Il2CppClass*, il2cpp_array_size_t*, il2cpp_array_size_t*) is exported: is il2cpp_array_size_t an int?
		type "!TEMPDIR!\nm-output.txt" | findstr "__ZN6il2cpp2vm5Array7NewFullEP11Il2CppClassPiS4_" > %TO_TEMPVAR%
		if not "_!TEMPVAR!_"=="__" set IL2CPP_ARRAY_SIZE_TYPE=int
		rem // see how il2cpp::vm::Array::NewFull(Il2CppClass*, il2cpp_array_size_t*, il2cpp_array_size_t*) is exported: is il2cpp_array_size_t an unsigned int?
		type "!TEMPDIR!\nm-output.txt" | findstr "__ZN6il2cpp2vm5Array7NewFullEP11Il2CppClassPjS4_" > %TO_TEMPVAR%
		if not "_!TEMPVAR!_"=="__" set IL2CPP_ARRAY_SIZE_TYPE=unsigned int
		rem // now no matter how it is defined, we know how to enforce it for the compiler in il2cpp-config.h. See if it's not already done, in which case do it
		type "Libraries\libil2cpp\include\il2cpp-config.h" | findstr "IOSBUILDENV_IL2CPP_FIX" > %TO_TEMPVAR%
		if "_!TEMPVAR!_"=="__" (
			echo.>> "Libraries\libil2cpp\include\il2cpp-config.h"
			echo #ifndef IOSBUILDENV_IL2CPP_FIX>> "Libraries\libil2cpp\include\il2cpp-config.h"
			echo #define IOSBUILDENV_IL2CPP_FIX>> "Libraries\libil2cpp\include\il2cpp-config.h"
			echo #define il2cpp_array_size_t !IL2CPP_ARRAY_SIZE_TYPE! /* iOS Build Environment patch: necessary addition to work around a compiler bug */>> "Libraries\libil2cpp\include\il2cpp-config.h"
			echo #endif // !IOSBUILDENV_IL2CPP_FIX>> "Libraries\libil2cpp\include\il2cpp-config.h"
		)
		rem // cleanup
		set IL2CPP_ARRAY_SIZE_TYPE=
		set UNITY_LIBRARY=
	)

	rem // FIXME: Unity seems to generate non-working projects on some pairs of {Editor+Xcode} versions, that crash in VerifyiPhoneSplashScreen().
	rem // As my splash screens are not RGB-swapped like Xcode does (see ios-png-normalizer), may this be related ? Strangely enough it isn't systematic.
	rem // The temporary fix seems to be to reroute this function call until Unity fixes the problem. Unfortunately many people still need to use older
	rem // editor versions, and I'm fed up of getting crash reports for a problem that isn't mine.
	rem // UPDATE: 2021-11-02, looks like this file no longer exists. Good. Keeping the patch around for a while...
	if exist "Classes\main.mm" (
		type "Classes\main.mm" | findstr "VerifyiPhoneSplashScreen" > %TO_TEMPVAR%
		if "_!TEMPVAR!_"=="__" (
			echo.>> "Classes\main.mm"
			echo #include ^<string^>>> "Classes\main.mm"
			echo void VerifyiPhoneSplashScreen^(std::string const^&^) ^{^}>> "Classes\main.mm"
		)
	)
)


if "_!SKIP_DEPENDENCIES!_"=="__" if "_!WANT_REBUILDALL!_"=="_true_" (
	rem // if we're at root level AND we want to rebuild everything, wipe out the build directory
	rem // NOTE: this must be done BEFORE building any sub-projects, hence why we do it here.
	echo  + Wiping output directory...
	rmdir /s /q "!BUILD_DIR!" > nul 2>&1
)

rem // are we building a Xcode project AND should we worry about building dependencies first ?
if exist "!XCNAME!.xcodeproj\project.pbxproj" if not "_!SKIP_DEPENDENCIES!_"=="_true_" (
	rem // if we have a CocoaPods project AND we do NOT want to skip pods, build by hand all its sub-projects
	if exist "Pods\Pods.xcodeproj\project.pbxproj" if "_!SKIP_COCOAPODS!_"=="__" (
		rem // list all the sub-projects in this Xcode project (pbxprojparse will sort them topologically) and call the build script on each of them
		rem // note that we specify the build dir as "..\build" so that the Pods are built in the same build directory as the main project
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "Pods\Pods.xcodeproj" -listtargets > "!TEMPDIR!\Pods.targetlist"
		for /f "usebackq delims=" %%f in ("!TEMPDIR!\Pods.targetlist") do (
			set COCOAPODS_PROJECTNAME=%%f
			echo Building the CocoaPods dependency project !COCOAPODS_PROJECTNAME!...
			rem // Pods don't need to be signed
			set IOS_IDENTITY=none
			set IOS_PROVISION=
			rem // we assume Pods don't need custom compiler/linker flags (FIXME: make this project-specific by dropping flags in the right environment variables)
			rem // build the Pods sub-project
			set EXTRA_FLAGS=-cppflags "-IPods/Headers/Public"
			if "_!WANT_REBUILDALL!_"=="_true_"  set EXTRA_FLAGS=!EXTRA_FLAGS! -rebuild
			if "_!WANT_LLVMLINKER!_"=="_true_"  set EXTRA_FLAGS=!EXTRA_FLAGS! -lld
			if "_!WANT_AUTORESOLVE!_"=="_true_" set EXTRA_FLAGS=!EXTRA_FLAGS! -autoresolve
			if "_!WANT_MULTICORE!_"=="_true_"   set EXTRA_FLAGS=!EXTRA_FLAGS! -multicore
			if "_!WANT_CCACHE!_"=="_false_"     set EXTRA_FLAGS=!EXTRA_FLAGS! -nocache
			if "_!WANT_CLEANCCACHE!_"=="_true_" set EXTRA_FLAGS=!EXTRA_FLAGS! -cleanmodules
			if not "_%DEBUG_ARG%_"=="__"        set EXTRA_FLAGS=!EXTRA_FLAGS! -debugscript
			set IS_BUILDING_DEPENDENCY=true
			call "!SCRIPT_DIRECTORY!\!SCRIPT_NAME!" -xcname Pods\Pods -xcpname "!COCOAPODS_PROJECTNAME!" -xcconf "!CONFIGURATION!" -minosver "!MINOSVER!" !EXTRA_FLAGS! -archs "!ARCHS!" -nodeps
			if not "!errorlevel!"=="0" echo.& goto :exit_error
			rem // cleanup
			set IS_BUILDING_DEPENDENCY=
			set COCOAPODS_PROJECTNAME=
			set WANT_CLEANCCACHE=
			set EXTRA_FLAGS=
			echo.
		)
	)

	rem // list all the dependencies in this Xcode project and call the build script on each of them
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" -target "!TARGET_NAME!" -dependencies > "!TEMPDIR!\!TARGET_NAME!.dependencies"
	for /f "usebackq delims=" %%f in ("!TEMPDIR!\!TARGET_NAME!.dependencies") do (
		set DEPENDENCY_PROJECTNAME=%%f
		rem // is this dependency an App Extension ? these need to be specifically signed
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!DEPENDENCY_PROJECTNAME!" -configuration "!CONFIGURATION!" -showtype > %TO_TEMPVAR%
		if "_!TEMPVAR!_"=="_com.apple.product-type.app-extension_" (
			rem // App Extensions have a different bundle identifier, thus they need a different provisioning profile.
			rem // By convention, an app extension's provisioning profile MUST be named xxxxxxx-AppEx.mobileprovision.
			for /f "delims=" %%i in ("!PROVISION!") do (
				set PROVISION_PATH=%%~dpi
				set PROVISION_BASENAME=%%~ni
				set PROVISION_EXTENSION=%%~xi
			)
			rem // so let's see if a ProvisioningProfile-AppEx.mobileprovision file exists
			if not exist "!PROVISION_PATH!\!PROVISION_BASENAME!-AppEx!PROVISION_EXTENSION!" (
				rem // nope, so search in the Keychain directory for a profile
				rem // - that contains the same certificate as the one we'll be using to sign
				rem // - whose Entitlements/application-identifier value matches "${TEAM_IDENTIFIER}.${BUNDLE_IDENTIFIER}"
				rem // - add other criteria here...
				rem // TODO

				rem // have we found just a single one ? if so, use it
				rem // TODO

				rem // else bomb out with an instructive error message
				echo Error: couldn't find a suitable provisioning profile for the "!DEPENDENCY_PROJECTNAME!" App Extension.
				echo App Extensions require a dedicated provisioning profile, different than the one used to sign your apps. Please put this provisioning profile alongside the one for your app and name it "!PROVISION_BASENAME!-AppEx!PROVISION_EXTENSION!".
				echo You don't need to change your signing identity. The builder will simply pick the "!PROVISION!" profile for your app, and "!PROVISION_BASENAME!-AppEx!PROVISION_EXTENSION!" for your app extension.
				echo Once this is done, you can build your app again.
				goto :exit_error
			)
			rem // App Extensions need to be individually signed. Use the same certificate as the app itself, and a specific provisioning profile
			set IOS_IDENTITY=!IDENTITY!
			set IOS_PROVISION=!PROVISION_PATH!\!PROVISION_BASENAME!-AppEx!PROVISION_EXTENSION!
		) else (
			rem // all other types of dependencies don't need to be signed
			set IOS_IDENTITY=none
			set IOS_PROVISION=
		)
		echo Building the dependency project !DEPENDENCY_PROJECTNAME!...
		set EXTRA_FLAGS=
		if "_!WANT_REBUILDALL!_"=="_true_"  set EXTRA_FLAGS=!EXTRA_FLAGS! -rebuild
		if "_!WANT_LLVMLINKER!_"=="_true_"  set EXTRA_FLAGS=!EXTRA_FLAGS! -lld
		if "_!WANT_AUTORESOLVE!_"=="_true_" set EXTRA_FLAGS=!EXTRA_FLAGS! -autoresolve
		if "_!WANT_CCACHE!_"=="_false_"     set EXTRA_FLAGS=!EXTRA_FLAGS! -nocache
		if "_!WANT_MULTICORE!_"=="_true_"   set EXTRA_FLAGS=!EXTRA_FLAGS! -multicore
		if "_!WANT_CLEANCCACHE!_"=="_true_" set EXTRA_FLAGS=!EXTRA_FLAGS! -cleanmodules
		if not "_%DEBUG_ARG%_"=="__"        set EXTRA_FLAGS=!EXTRA_FLAGS! -debugscript
		set IS_BUILDING_DEPENDENCY=true
		call "!SCRIPT_DIRECTORY!\!SCRIPT_NAME!" -xcname "!XCNAME!" -xcpname "!DEPENDENCY_PROJECTNAME!" -xcconf "!CONFIGURATION!" !EXTRA_FLAGS! -archs "!ARCHS!" -nodeps
		if not "!errorlevel!"=="0" echo.& goto :exit_error
		rem // cleanup
		set IS_BUILDING_DEPENDENCY=
		set DEPENDENCY_PROJECTNAME=
		set WANT_CLEANCCACHE=
		set EXTRA_FLAGS=
		echo.
	)
)


rem // if we don't know whether we want to strip the final binary, assume we don't
if "_!WANT_STRIP!_"=="__" set WANT_STRIP=false
if "_!WANT_STRIP_BITCODE!_"=="__" set WANT_STRIP_BITCODE=false


rem // if a deployment has been requested, make sure we build at least an IPA
if not "_!WANT_DEPLOY!_"=="__" set WANT_IPA=true


rem // if unspecified on the command-line, guess the number of CPU cores (default to 1)
if "_!CORES!_"=="__" (
	rem // NOTE: "wmic" is available on all Windows machines since Windows XP (at least from SP3)
	wmic cpu get NumberOfLogicalProcessors 2>nul | findstr /v "NumberOfLogicalProcessors" > %TO_TEMPVAR%
	rem // remove spaces from value and default to 1 core if we couldn't get the actual number of cores
	if not "_!TEMPVAR!_"=="__" set CORES=!TEMPVAR: =!
	if "_!CORES!_"=="__" set CORES=1
)


rem // remove spaces from remote log IP
if not "_!REMOTELOG_IP!_"=="__" set REMOTELOG_IP=!REMOTELOG_IP: =!
rem // if we have specified remote logging to ourselves, resolve our IP
if "_!REMOTELOG_IP!_"=="_myself_" (
	(for /f "tokens=4" %%b in ('"route print 0.0.0.0|findstr "0\.0\.0\.0""') do @echo %%b) > %TO_TEMPVAR%
	set REMOTELOG_IP=!TEMPVAR!
)


rem // if we want to use an automatic build version numbering scheme, construct a unique, incremental (time-based) build version number
if "_!WANT_AUTOVERSION!_"=="_true_" (
	wmic OS GET LocalDateTime | findstr "\." > %TO_TEMPVAR%
	rem // format to use: DDDD.HH.MM (DDDD being roughly "days since 2010", assuming each month is 31 days which is false, but right enough for our purpose...)
	set YEAR=!TEMPVAR:~0,4!
	set MONTH=!TEMPVAR:~4,2!
	if "_!MONTH:~0,1!_"=="_0_" set MONTH=!MONTH:~1!
	set DAY=!TEMPVAR:~6,2!
	if "_!DAY:~0,1!_"=="_0_" set DAY=!DAY:~1!
	set /A VER_1=^(!YEAR!-2010^)*31*12+!MONTH!*31+!DAY!
	set VER_2=!TEMPVAR:~8,2!
	set VER_3=!TEMPVAR:~10,2!
	set BUILD_NUMBER=!VER_1!.!VER_2!.!VER_3!
	set YEAR=& set MONTH=& set DAY=& set VER1=& set VER2=& set VER3=
	echo Notice: setting automatic build number to !BUILD_NUMBER!.
)


rem // if we don't have a configuration name, assume "Default"
if "_!CONFIGURATION!_"=="__" set CONFIGURATION=Default


rem // display final parameters in debug mode
if not "_%DEBUG_ARG%_"=="__" (
	echo ===============================================================================
	echo Final parameters:
	echo ARCHS=[!ARCHS!]
	echo ARTWORK=[!ARTWORK!]
	echo BUILD_NUMBER=[!BUILD_NUMBER!]
	echo CFLAGS=[!CFLAGS!]
	echo CONFIGURATION=[!CONFIGURATION!]
	echo CORES=[!CORES!]
	echo CPPFLAGS=[!CPPFLAGS!]
	echo CXXFLAGS=[!CXXFLAGS!]
	echo|set /p=EMBEDDED_BINARIES=[
	if exist "!TEMPDIR!\EMBEDDED_BINARIES.txt" type "!TEMPDIR!\EMBEDDED_BINARIES.txt"
	echo ]
	echo ENTITLEMENTS_PLIST=[!ENTITLEMENTS_PLIST!]
	echo ENTRYPOINT=[!ENTRYPOINT!]
	echo FAMILY=[!FAMILY!]
	echo IDENTITY=[!IDENTITY!]
	echo INFO_PLIST=[!INFO_PLIST!]
	echo|set /p=LDFLAGS=[
	if exist "!TEMPDIR!\LDFLAGS.txt" type "!TEMPDIR!\LDFLAGS.txt"
	echo ]
	echo MINOSVER=[!MINOSVER!]
	echo NAME=[!NAME!]
	echo NEEDS_REBUILD=[!NEEDS_REBUILD!]
	echo PCH=[!PCH!]
	echo PREPKG_SCRIPT=[!PREPKG_SCRIPT!]
	echo PROVISION=[!PROVISION!]
	echo REMOTELOG_IP=[!REMOTELOG_IP!]
	echo RES=[!RES!]
	echo SDK_PATH=[!SDK_PATH!]
	echo SDK_VERSION=[!SDK_VERSION!]
	echo SDK_PLATFORM_NAME=[!SDK_PLATFORM_NAME!]
	echo SKIP_DEPENDENCIES=[!SKIP_DEPENDENCIES!]
	echo SKIP_COCOAPODS=[!SKIP_COCOAPODS!]
	echo TARGET_NAME=[!TARGET_NAME!]
	echo TOOLCHAIN_PATH=[!TOOLCHAIN_PATH!]
	echo WANT_AUTORESOLVE=[!WANT_AUTORESOLVE!]
	echo WANT_AUTOVERSION=[!WANT_AUTOVERSION!]
	echo WANT_CCACHE=[!WANT_CCACHE!]
	echo WANT_CLEANCCACHE=[!WANT_CLEANCCACHE!]
	echo WANT_LLVMLINKER=[!WANT_LLVMLINKER!]
	echo WANT_DEB=[!WANT_DEB!]
	echo WANT_DEPLOY=[!WANT_DEPLOY!]
	echo WANT_IPA=[!WANT_IPA!]
	echo WANT_MULTICORE=[!WANT_MULTICORE!]
	echo WANT_PNGCRUSH=[!WANT_PNGCRUSH!]
	echo WANT_REBUILDALL=[!WANT_REBUILDALL!]
	echo WANT_STRIP_BITCODE=[!WANT_STRIP_BITCODE!]
	echo WANT_STRIP=[!WANT_STRIP!]
	echo WANT_SYMLINKS=[!WANT_SYMLINKS!]
	echo XCNAME=[!XCNAME!]
	echo Press a key or wait %DEBUG_WAIT% seconds to continue...
	timeout /t %DEBUG_WAIT% > nul
)


rem // END parameters guessing
rem //////////////////////////


rem // tell the world who we are and what we're doing
title iOS Build Environment - building !NAME! for !ARCHS: =+! ^(configuration: !CONFIGURATION!^) using !SDK_PLATFORM_NAME! SDK !SDK_VERSION!


rem // measure build start time (only for the top-level build)
if "_!BUILDSTART_TIME!_"=="__" set BUILDSTART_TIME=%TIME%


rem ///////////////////
rem // STEP 1. CLEAN //
rem ///////////////////


if not "_!WANT_CLEANCCACHE!_"=="__" (
	rem // prune compiler cache
	echo  + Pruning compiler caches...
	rmdir /s /q "!SDK_PATH!\.ccache" > nul 2>&1
	rmdir /s /q "!SDK_PATH!\.llvm-modules" > nul 2>&1
)

rem // make sure we have a build directory, work out one if necessary
if exist "!XCNAME!.xcodeproj\project.pbxproj" (
	rem // Xcode projects may supply one
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query CONFIGURATION_BUILD_DIR > %TO_TEMPVAR%
	if not "_!TEMPVAR!_"=="__" set TARGET_BUILD_DIR=!TEMPVAR!
)
if "_!TARGET_BUILD_DIR!_"=="__" set TARGET_BUILD_DIR=!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\!TARGET_NAME!

rem // work out the output directory and wipe it
echo  + Preparing output directory...
mkdir "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!" > nul 2>&1
if        "_!ENTRYPOINT!_"=="_framework_" ( set OUTDIR=!TARGET_BUILD_DIR!\!NAME!.framework
) else if "_!ENTRYPOINT!_"=="_appex_"     ( set OUTDIR=!TARGET_BUILD_DIR!\!NAME!.appex
) else if "_!ENTRYPOINT!_"=="_bundle_"    ( set OUTDIR=!TARGET_BUILD_DIR!\!NAME!.bundle
) else if "_!ENTRYPOINT!_"=="_lib_"       ( set OUTDIR=!TARGET_BUILD_DIR!\!NAME!.lib
) else if "_!ENTRYPOINT!_"=="_dylib_"     ( set OUTDIR=!TARGET_BUILD_DIR!\!NAME!.dylib
) else if exist "!INFO_PLIST!"            ( set OUTDIR=!TARGET_BUILD_DIR!\!NAME!.app
) else                                    ( set OUTDIR=!TARGET_BUILD_DIR!\!NAME!.out
)
rmdir /s /q "!OUTDIR!" > nul 2>&1
if exist "!OUTDIR!\nul" (
	rem // something prevents us to do that. Maybe a permissions problem, or a stupid Thumbs.db ? Nuke the shit out of that.
	ping -n 1 127.0.0.1 > nul
	attrib -r -s -h -a /S /D "!OUTDIR!" > nul
	rmdir /s /q "!OUTDIR!" > nul 2>&1
	if exist "!OUTDIR!\nul" echo ERROR: unable to clean output directory "!OUTDIR!". Check permissions and delete it manually, then try again.& goto :exit_error
)
rmdir /s /q "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedLibraries"        > nul 2>&1
rmdir /s /q "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks"       > nul 2>&1
rmdir /s /q "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries"           > nul 2>&1
rmdir /s /q "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks"          > nul 2>&1
rmdir /s /q "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\XCFrameworkIntermediates" > nul 2>&1

rem // have a list of available libraries and frameworks at hand
dir *.a           /b /s 2>nul | findstr /v /c:"\\!CONFIGURATION!-!SDK_PLATFORM_NAME!\\" >  "!TEMPDIR!\supplied-libraries.out"
dir *.dylib       /b /s 2>nul | findstr /v /c:"\\!CONFIGURATION!-!SDK_PLATFORM_NAME!\\" >> "!TEMPDIR!\supplied-libraries.out"
dir *.framework   /b /s 2>nul | findstr /v /c:"\\!CONFIGURATION!-!SDK_PLATFORM_NAME!\\" | findstr /v /c:".xcframework\\" >  "!TEMPDIR!\supplied-frameworks.out"
dir *.xcframework /b /s 2>nul | findstr /v /c:"\\!CONFIGURATION!-!SDK_PLATFORM_NAME!\\" >  "!TEMPDIR!\supplied-xcframeworks.out"
pushd "!BUILD_DIR!"
dir *.a         /b /s 2>nul | findstr /v /c:"\!CONFIGURATION!-!SDK_PLATFORM_NAME!\!TARGET_NAME!\" | findstr /v "\.app\\" >  "!TEMPDIR!\built-libraries.out"
dir *.dylib     /b /s 2>nul | findstr /v /c:"\!CONFIGURATION!-!SDK_PLATFORM_NAME!\!TARGET_NAME!\" | findstr /v "\.app\\" >> "!TEMPDIR!\built-libraries.out"
dir *.framework /b /s 2>nul | findstr /v /c:"\!CONFIGURATION!-!SDK_PLATFORM_NAME!\!TARGET_NAME!\" | findstr /v "\.app\\ \\XCFrameworkIntermediates\\" > "!TEMPDIR!\built-frameworks.out"
popd
rem // do we have .xcframework's (outside of the build directory) ? If so, create a XCFrameworkIntermediates directory and lay them out there
if        "_!SDK_PLATFORM_NAME!_"=="_iphoneos_"         ( set XCFRAMEWORK_SUPPORTEDPLATFORM=ios&       set XCFRAMEWORK_SUPPORTEDPLATFORMVARIANT=
) else if "_!SDK_PLATFORM_NAME!_"=="_iphonesimulator_"  ( set XCFRAMEWORK_SUPPORTEDPLATFORM=ios&       set XCFRAMEWORK_SUPPORTEDPLATFORMVARIANT=simulator
) else if "_!SDK_PLATFORM_NAME!_"=="_watchos_"          ( set XCFRAMEWORK_SUPPORTEDPLATFORM=watchos&   set XCFRAMEWORK_SUPPORTEDPLATFORMVARIANT=
) else if "_!SDK_PLATFORM_NAME!_"=="_watchsimulator_"   ( set XCFRAMEWORK_SUPPORTEDPLATFORM=watchos&   set XCFRAMEWORK_SUPPORTEDPLATFORMVARIANT=simulator
) else if "_!SDK_PLATFORM_NAME!_"=="_appletvos_"        ( set XCFRAMEWORK_SUPPORTEDPLATFORM=appletvos& set XCFRAMEWORK_SUPPORTEDPLATFORMVARIANT=
) else if "_!SDK_PLATFORM_NAME!_"=="_appletvsimulator_" ( set XCFRAMEWORK_SUPPORTEDPLATFORM=appletvos& set XCFRAMEWORK_SUPPORTEDPLATFORMVARIANT=simulator
) else if "_!SDK_PLATFORM_NAME!_"=="_xros_"             ( set XCFRAMEWORK_SUPPORTEDPLATFORM=xros&      set XCFRAMEWORK_SUPPORTEDPLATFORMVARIANT=
) else if "_!SDK_PLATFORM_NAME!_"=="_xrsimulator_"      ( set XCFRAMEWORK_SUPPORTEDPLATFORM=xros&      set XCFRAMEWORK_SUPPORTEDPLATFORMVARIANT=simulator
) else                                                  ( set XCFRAMEWORK_SUPPORTEDPLATFORM=macos&     set XCFRAMEWORK_SUPPORTEDPLATFORMVARIANT=
)
set SUPPLIED_COUNT=0
for /f "usebackq delims=" %%a in ("!TEMPDIR!\supplied-xcframeworks.out") do (
	set XCFRAMEWORK_DIR=%%a
	set XCFRAMEWORK_NAME=%%~na
	rem echo evaluating .xcframework: !XCFRAMEWORK_NAME!
	rem // verify Info.plist presence and search through each AvailableLibrary in this .xcframework's Info.plist until we find something suitable
	if exist "!XCFRAMEWORK_DIR!\Info.plist" for %%b in (0 1 2 3 4 5 6 7 8 9) do if not exist "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\XCFrameworkIntermediates\!XCFRAMEWORK_NAME!" (
		set AVAILABLELIBRARIES_INDEX=%%b
		rem // verify that SupportedPlatform key is present and matches what we need
		"!TOOLCHAIN_PATH!\plconvert.exe" "!XCFRAMEWORK_DIR!\Info.plist" /query AvailableLibraries[!AVAILABLELIBRARIES_INDEX!]/SupportedPlatform > %TO_TEMPVAR%
		if "_!errorlevel!_"=="_0_" if "_!TEMPVAR!_"=="_!XCFRAMEWORK_SUPPORTEDPLATFORM!_" (
			rem // verify that SupportedPlatformVariant matches what we need
			"!TOOLCHAIN_PATH!\plconvert.exe" "!XCFRAMEWORK_DIR!\Info.plist" /query AvailableLibraries[!AVAILABLELIBRARIES_INDEX!]/SupportedPlatformVariant > %TO_TEMPVAR%
			if "_!TEMPVAR!_"=="_!XCFRAMEWORK_SUPPORTEDPLATFORMVARIANT!_" (
				rem // get the list of architectures this available library supports and verify that not one target arch is missing in this available library
				set MISSING_ARCHS=0
				"!TOOLCHAIN_PATH!\plconvert.exe" "!XCFRAMEWORK_DIR!\Info.plist" /query AvailableLibraries[!AVAILABLELIBRARIES_INDEX!]/SupportedArchitectures > "!TEMPDIR!\plconvert.out"
				for %%c in (!ARCHS!) do (
					findstr "%%c" "!TEMPDIR!\plconvert.out" > nul 2>&1 || set /a MISSING_ARCHS=!MISSING_ARCHS!+1
				)
				del /f /q "!TEMPDIR!\plconvert.out" > nul 2>&1
				if "_!MISSING_ARCHS!_"=="_0_" (
					rem // this library looks good, read its identifier and figure out its .framework directory's path
					"!TOOLCHAIN_PATH!\plconvert.exe" "!XCFRAMEWORK_DIR!\Info.plist" /query AvailableLibraries[!AVAILABLELIBRARIES_INDEX!]/LibraryIdentifier > %TO_TEMPVAR%
					set XCFRAMEWORK_SUBDIR=!XCFRAMEWORK_DIR!\!TEMPVAR!
					rem // create the XCFrameworkIntermediates directory if it doesn't exist yet and drop a symlink there
					mkdir "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\XCFrameworkIntermediates" > nul 2>&1
					echo|set /p="!XCFRAMEWORK_SUBDIR!" > "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\XCFrameworkIntermediates\!XCFRAMEWORK_NAME!"
					rem // create the BuiltFrameworks directory if it doesn't exist yet and drop a symlink there
					mkdir "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks" > nul 2>&1
					echo|set /p="!XCFRAMEWORK_SUBDIR!\!XCFRAMEWORK_NAME!.framework" > "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks\!XCFRAMEWORK_NAME!.framework"
					rem // now add this path to the *COMPILER* frameworks search paths (in case this framework has a Headers directory)
					set CPPFLAGS=!CPPFLAGS! -F"!XCFRAMEWORK_SUBDIR:\=/!"
					if not "_!DEBUG_ARG!_"=="__" echo Mapped framework !XCFRAMEWORK_NAME! into XCFrameworkIntermediates
					set /a SUPPLIED_COUNT=!SUPPLIED_COUNT!+1
					rem // cleanup
					set XCFRAMEWORK_SUBDIR=
				)
				rem // cleanup
				set MISSING_ARCHS=
			)
		)
		rem // cleanup
		set AVAILABLELIBRARIES_INDEX=
	)
	rem // cleanup
	set XCFRAMEWORK_NAME=
	set XCFRAMEWORK_DIR=
)
if "_!DEBUG_ARG!_"=="__" echo    + XCFrameworks: !SUPPLIED_COUNT! supplied
rem // cleanup
set XCFRAMEWORK_SUPPORTEDPLATFORMVARIANT=
set XCFRAMEWORK_SUPPORTEDPLATFORM=
rem // define the LC_BUILD_VERSION load command platform value to look for in supplied dylibs/frameworks (values from Apple's mach-o/loader.h)
if        "_!SDK_PLATFORM_NAME!_"=="_macosx_"    ( set LC_VERSION_MIN_CMDNAME=LC_VERSION_MIN_MACOSX&   set LC_BUILD_VERSION_PLATFORM=platform 1
) else if "_!SDK_PLATFORM_NAME!_"=="_iphoneos_"  ( set LC_VERSION_MIN_CMDNAME=LC_VERSION_MIN_IPHONEOS& set LC_BUILD_VERSION_PLATFORM=platform 2
) else if "_!SDK_PLATFORM_NAME!_"=="_appletvos_" ( set LC_VERSION_MIN_CMDNAME=LC_VERSION_MIN_TVOS&     set LC_BUILD_VERSION_PLATFORM=platform 3
) else if "_!SDK_PLATFORM_NAME!_"=="_watchos_"   ( set LC_VERSION_MIN_CMDNAME=LC_VERSION_MIN_WATCHOS&  set LC_BUILD_VERSION_PLATFORM=platform 4
)
rem // map libraries that live outside the build directory into a SuppliedLibraries directory
set SUPPLIED_COUNT=0
for /f "usebackq delims=" %%a in ("!TEMPDIR!\supplied-libraries.out") do (
	set LIBRARY_PATHNAME=%%a
	set LIBRARY_NAME=%%~nxa
	rem // if not mapped already, verify that all the architectures we need are present in this library
	if not exist "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedLibraries\!LIBRARY_NAME!" "!TOOLCHAIN_PATH!\lipo.exe" "!LIBRARY_PATHNAME!" -verify_arch !ARCHS! 2>nul && (
		rem // also verify that it is for the target architecture that matches our SDK. For this we look for either of 2 load commands: LC_VERSION_MIN_<platform> or the newer LC_BUILD_VERSION which has a <platform> member
		"!TOOLCHAIN_PATH!\otool.exe" -l "!LIBRARY_PATHNAME!" > "!TEMPDIR!\loadcmds.out"
		set IS_SUITABLE=false
		find "!LC_VERSION_MIN_CMDNAME!" "!TEMPDIR!\loadcmds.out" > nul && set IS_SUITABLE=true
		if "_!IS_SUITABLE!_"=="_false_" powershell Select-String -Path "!TEMPDIR!\loadcmds.out" -Pattern "LC_BUILD_VERSION" -Context 0,2|find "!LC_BUILD_VERSION_PLATFORM!" > nul && set IS_SUITABLE=true
		if "_!IS_SUITABLE!_"=="_true_" (
			rem // create the SuppliedLibraries directory if it doesn't exist yet and drop a symlink there
			mkdir "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedLibraries" > nul 2>&1
			echo|set /p="!LIBRARY_PATHNAME!" > "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedLibraries\!LIBRARY_NAME!"
			if not "_!DEBUG_ARG!_"=="__" echo Mapped library !LIBRARY_NAME! into SuppliedLibraries
			set /a SUPPLIED_COUNT=!SUPPLIED_COUNT!+1
		)
		rem // cleanup
		set IS_SUITABLE=
		del /f /q "!TEMPDIR!\loadcmds.out" > nul 2>&1
	)
	rem // cleanup
	set LIBRARY_NAME=
	set LIBRARY_PATHNAME=
)
rem // map libraries that live inside the build directory into a BuiltLibraries directory
set BUILT_COUNT=0
for /f "usebackq delims=" %%a in ("!TEMPDIR!\built-libraries.out") do (
	set LIBRARY_PATHNAME=%%a
	set LIBRARY_NAME=%%~nxa
	rem // if not mapped already, verify that all the architectures we need are present in this library
	if not exist "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries\!LIBRARY_NAME!" "!TOOLCHAIN_PATH!\lipo.exe" "!LIBRARY_PATHNAME!" -verify_arch !ARCHS! 2>nul && (
		rem // also verify that it is for the target architecture that matches our SDK. For this we look for either of 2 load commands: LC_VERSION_MIN_<platform> or the newer LC_BUILD_VERSION which has a <platform> member
		"!TOOLCHAIN_PATH!\otool.exe" -l "!LIBRARY_PATHNAME!" > "!TEMPDIR!\loadcmds.out"
		set IS_SUITABLE=false
		find "!LC_VERSION_MIN_CMDNAME!" "!TEMPDIR!\loadcmds.out" > nul && set IS_SUITABLE=true
		if "_!IS_SUITABLE!_"=="_false_" powershell Select-String -Path "!TEMPDIR!\loadcmds.out" -Pattern "LC_BUILD_VERSION" -Context 0,2|find "!LC_BUILD_VERSION_PLATFORM!" > nul && set IS_SUITABLE=true
		if "_!IS_SUITABLE!_"=="_true_" (
			rem // create the BuiltLibraries directory if it doesn't exist yet and drop a symlink there
			mkdir "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries" > nul 2>&1
			echo|set /p="!LIBRARY_PATHNAME!" > "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries\!LIBRARY_NAME!"
			if not "_!DEBUG_ARG!_"=="__" echo Mapped library !LIBRARY_NAME! into BuiltLibraries
			set /a BUILT_COUNT=!BUILT_COUNT!+1
		)
		rem // cleanup
		set IS_SUITABLE=
		del /f /q "!TEMPDIR!\loadcmds.out" > nul 2>&1
	)
	rem // cleanup
	set LIBRARY_NAME=
	set LIBRARY_PATHNAME=
)
if "_!DEBUG_ARG!_"=="__" echo    + Libraries: !SUPPLIED_COUNT! supplied, !BUILT_COUNT! built
rem // map frameworks that live outside the build directory into a SuppliedFrameworks directory
set SUPPLIED_COUNT=0
for /f "usebackq delims=" %%a in ("!TEMPDIR!\supplied-frameworks.out") do (
	set FRAMEWORK_PATH=%%a
	set FRAMEWORK_NAME=%%~na
	rem // if not mapped already, verify that all the architectures we need are present in this framework (NOTE THAT WE EXCLUDE SIMULATOR VARIANTS HERE!)
	if not exist "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks\!FRAMEWORK_NAME!.framework" echo "!FRAMEWORK_PATH:\=/!"|findstr /v /c:"-simulator/" > nul && "!TOOLCHAIN_PATH!\lipo.exe" "!FRAMEWORK_PATH!\!FRAMEWORK_NAME!" -verify_arch !ARCHS! 2>nul && (
		rem // also verify that it is for the target architecture that matches our SDK. For this we look for either of 2 load commands: LC_VERSION_MIN_<platform> or the newer LC_BUILD_VERSION which has a <platform> member
		"!TOOLCHAIN_PATH!\otool.exe" -l "!FRAMEWORK_PATH!\!FRAMEWORK_NAME!" > "!TEMPDIR!\loadcmds.out"
		set IS_SUITABLE=false
		find "!LC_VERSION_MIN_CMDNAME!" "!TEMPDIR!\loadcmds.out" > nul && set IS_SUITABLE=true
		if "_!IS_SUITABLE!_"=="_false_" powershell Select-String -Path "!TEMPDIR!\loadcmds.out" -Pattern "LC_BUILD_VERSION" -Context 0,2|find "!LC_BUILD_VERSION_PLATFORM!" > nul && set IS_SUITABLE=true
		if "_!IS_SUITABLE!_"=="_true_" (
			rem // create the BuiltFrameworks directory if it doesn't exist yet and drop a symlink there
			mkdir "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks" > nul 2>&1
			echo|set /p="!FRAMEWORK_PATH!" > "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks\!FRAMEWORK_NAME!.framework"
			if not "_!DEBUG_ARG!_"=="__" echo Mapped framework !FRAMEWORK_NAME! into SuppliedFrameworks
			set /a SUPPLIED_COUNT=!SUPPLIED_COUNT!+1
		)
		rem // cleanup
		set IS_SUITABLE=
		del /f /q "!TEMPDIR!\loadcmds.out" > nul 2>&1
	)
	rem // cleanup
	set FRAMEWORK_NAME=
	set FRAMEWORK_PATH=
)
rem // map frameworks that live inside the build directory into a BuiltFrameworks directory
set BUILT_COUNT=0
for /f "usebackq delims=" %%a in ("!TEMPDIR!\built-frameworks.out") do (
	set FRAMEWORK_PATH=%%a
	set FRAMEWORK_NAME=%%~na
	rem // if not mapped already, verify that all the architectures we need are present in this framework
	if not exist "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks\!FRAMEWORK_NAME!.framework" "!TOOLCHAIN_PATH!\lipo.exe" "!FRAMEWORK_PATH!\!FRAMEWORK_NAME!" -verify_arch !ARCHS! 2>nul && (
		rem // also verify that it is for the target architecture that matches our SDK. For this we look for either of 2 load commands: LC_VERSION_MIN_<platform> or the newer LC_BUILD_VERSION which has a <platform> member
		"!TOOLCHAIN_PATH!\otool.exe" -l "!FRAMEWORK_PATH!\!FRAMEWORK_NAME!" > "!TEMPDIR!\loadcmds.out"
		set IS_SUITABLE=false
		find "!LC_VERSION_MIN_CMDNAME!" "!TEMPDIR!\loadcmds.out" > nul && set IS_SUITABLE=true
		if "_!IS_SUITABLE!_"=="_false_" powershell Select-String -Path "!TEMPDIR!\loadcmds.out" -Pattern "LC_BUILD_VERSION" -Context 0,2|find "!LC_BUILD_VERSION_PLATFORM!" > nul && set IS_SUITABLE=true
		if "_!IS_SUITABLE!_"=="_true_" (
			rem // create the BuiltFrameworks directory if it doesn't exist yet and drop a symlink there
			mkdir "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks" > nul 2>&1
			echo|set /p="!FRAMEWORK_PATH!" > "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks\!FRAMEWORK_NAME!.framework"
			if not "_!DEBUG_ARG!_"=="__" echo Mapped framework !FRAMEWORK_NAME! into BuiltFrameworks
			set /a BUILT_COUNT=!BUILT_COUNT!+1
		)
		rem // cleanup
		set IS_SUITABLE=
		del /f /q "!TEMPDIR!\loadcmds.out" > nul 2>&1
	)
	rem // cleanup
	set FRAMEWORK_NAME=
	set FRAMEWORK_PATH=
)
if "_!DEBUG_ARG!_"=="__" echo    + Frameworks: !SUPPLIED_COUNT! supplied, !BUILT_COUNT! built
rem // now add those paths to the linker's search paths
if exist "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedLibraries\."  echo  -L"!BUILD_DIR!/!CONFIGURATION!-!SDK_PLATFORM_NAME!/SuppliedLibraries"  >> "!TEMPDIR!\LDFLAGS.txt"
if exist "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks\." echo  -F"!BUILD_DIR!/!CONFIGURATION!-!SDK_PLATFORM_NAME!/SuppliedFrameworks" >> "!TEMPDIR!\LDFLAGS.txt"
if exist "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries\."     echo  -L"!BUILD_DIR!/!CONFIGURATION!-!SDK_PLATFORM_NAME!/BuiltLibraries"     >> "!TEMPDIR!\LDFLAGS.txt"
if exist "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks\."    echo  -F"!BUILD_DIR!/!CONFIGURATION!-!SDK_PLATFORM_NAME!/BuiltFrameworks"    >> "!TEMPDIR!\LDFLAGS.txt"
rem // and cleanup the things we no longer need
del /f /q "!TEMPDIR!\supplied-xcframeworks.out" "!TEMPDIR!\supplied-frameworks.out" "!TEMPDIR!\supplied-libraries.out" "!TEMPDIR!\built-frameworks.out" "!TEMPDIR!\built-libraries.out" > nul 2>&1
set LC_BUILD_VERSION_PLATFORM=
set LC_VERSION_MIN_CMDNAME=
set SUPPLIED_COUNT=
set BUILT_COUNT=

if not "_!WANT_REBUILDALL!_"=="__" set NEEDS_REBUILD=true
if not "_!NEEDS_REBUILD!_"=="__" (
	rem // prune any previous compilation results
	echo  + Pruning compilation results...
	rmdir /s /q "!TARGET_BUILD_DIR!" > nul 2>&1
)

rem // (re)create the intermediate directory if needed
rem // note that with enableextensions, mkdir also creates all the intermediate directories as needed
mkdir "!TARGET_BUILD_DIR!" > nul 2>&1
type NUL > "!TARGET_BUILD_DIR!\target-agnostic.compflags"
type NUL > "!TARGET_BUILD_DIR!\incpaths.compflags"


rem //////////////////////////////
rem // STEP 2. COMPILE AND LINK //
rem //////////////////////////////


rem // beware of the dreaded 2048 byte-max command line size.

rem // if we requested a minimum iOS version greater than 10 AND we want armv7, this won't do it, so disable *ALL* armv7* builds
if "_!SDK_PLATFORM_NAME:~0,6!_"=="_iphone_" if !MINOSVER_MAJOR! GTR 10 if not "_!ARCHS:armv7=!_"=="_!ARCHS!_" (
	echo Notice: NOT building 32-bit slice^(s^) because minimum iOS version ^(!MINOSVER!^) prohibits it.
	set NEWARCHS=
	for %%a in (!ARCHS!) do (
		set ARCH=%%a
		if "_!ARCH:armv7=!_"=="_!ARCH!_" set NEWARCHS=!NEWARCHS! !ARCH!
	)
	set ARCHS=!NEWARCHS!
)

rem // prepare compiler flags
rem // FIXME: figure out if some machines need -I/usr/include -I/System/Library/Frameworks (these are normally implicit)
echo -U________CommonFlags__FromBuildCmd_______________________________>>                          "!TARGET_BUILD_DIR!\target-agnostic.compflags"
rem // we no longer to hack around the clang version check now that we're using Apple's branch.
rem // echo -U__STDC_HOSTED__ -U__clang_major__ -D__clang_major__=99>>                                    "!TARGET_BUILD_DIR!\target-agnostic.compflags"
rem // build the __IPHONE_OS_VERSION_MIN_REQUIRED definition (10000 * major + 100 * minor + 1 * revision)
for /f "tokens=1 delims=." %%v in ("!MINOSVER!") do set MINOSVER_MAJOR=%%v
for /f "tokens=2 delims=." %%v in ("!MINOSVER!") do set MINOSVER_MINOR=%%v
for /f "tokens=3 delims=." %%v in ("!MINOSVER!") do set MINOSVER_REVIS=%%v
if "_!MINOSVER_MINOR!_"=="__" set MINOSVER_MINOR=0
if "_!MINOSVER_REVIS!_"=="__" set MINOSVER_REVIS=0
set /a OSVERSIONMINREQUIRED=10000*!MINOSVER_MAJOR!+100*!MINOSVER_MINOR!+1*!MINOSVER_REVIS!>nul
if "_!SDK_PLATFORM_NAME:~0,6!_"=="_iphone_" (
	rem // FIXME: not sure whether the first one (-DIPHONE) is necessary...
	echo -DIPHONE>>                                                                     "!TARGET_BUILD_DIR!\target-agnostic.compflags"
	echo -DTARGET_OS_IPHONE -D__IPHONE_OS_VERSION_MIN_REQUIRED=!OSVERSIONMINREQUIRED!>> "!TARGET_BUILD_DIR!\target-agnostic.compflags"
	echo -miphoneos-version-min=!MINOSVER_MAJOR!.!MINOSVER_MINOR!.!MINOSVER_REVIS!>>    "!TARGET_BUILD_DIR!\target-agnostic.compflags"
) else if "_!SDK_PLATFORM_NAME:~0,5!_"=="_watch_" (
	echo -DTARGET_OS_WATCH>>                                                            "!TARGET_BUILD_DIR!\target-agnostic.compflags"
	echo -mwatchos-version-min=!MINOSVER_MAJOR!.!MINOSVER_MINOR!.!MINOSVER_REVIS!>>     "!TARGET_BUILD_DIR!\target-agnostic.compflags"
) else if "_!SDK_PLATFORM_NAME:~0,2!_"=="_tv_" (
	echo -DTARGET_OS_APPLETV>>                                                          "!TARGET_BUILD_DIR!\target-agnostic.compflags"
	echo -mtvos-version-min=!MINOSVER_MAJOR!.!MINOSVER_MINOR!.!MINOSVER_REVIS!>>        "!TARGET_BUILD_DIR!\target-agnostic.compflags"
) else if "_!SDK_PLATFORM_NAME:~0,2!_"=="_xr_" (
	echo -DTARGET_OS_VISION>>                                                           "!TARGET_BUILD_DIR!\target-agnostic.compflags"
	rem // FIXME: not sure about this one...
	echo -mxros-version-min=!MINOSVER_MAJOR!.!MINOSVER_MINOR!.!MINOSVER_REVIS!>>        "!TARGET_BUILD_DIR!\target-agnostic.compflags"
) else (
	echo -DTARGET_OS_MAC -D__MAC_OS_X_VERSION_MIN_REQUIRED=!OSVERSIONMINREQUIRED!>>     "!TARGET_BUILD_DIR!\target-agnostic.compflags"
	echo -mmacosx-version-min=!MINOSVER_MAJOR!.!MINOSVER_MINOR!.!MINOSVER_REVIS!>>      "!TARGET_BUILD_DIR!\target-agnostic.compflags"
)
set OSVERSIONMINREQUIRED=
echo -Wno-unused-command-line-argument>>                                                    "!TARGET_BUILD_DIR!\target-agnostic.compflags"
echo -fdiagnostics-format=msvc>>                                                            "!TARGET_BUILD_DIR!\target-agnostic.compflags"
echo -fconstant-cfstrings>>                                                                 "!TARGET_BUILD_DIR!\target-agnostic.compflags"
rem // the following line works around a bug in some versions of Google Firebase, and is harmless in all the other cases
echo -DTARGET_OS_WATCHOS=TARGET_OS_WATCH>>                                                  "!TARGET_BUILD_DIR!\target-agnostic.compflags"
rem // the "visibility=hidden" flag reduces the size of the final binary, but don't do that for anything else than a BINARY.
if "_!ENTRYPOINT!_"=="_crt_" echo -fvisibility=hidden -fvisibility-inlines-hidden>>         "!TARGET_BUILD_DIR!\target-agnostic.compflags"
rem // if we requested a minimum iOS version lesser than 8, enable old-style Objective-C objc_msgSend() prototypes
if "_!SDK_PLATFORM_NAME:~0,6!_"=="_iphone_" if !MINOSVER_MAJOR! LSS 8 (
	echo Notice: building for iOS ^<= 7. Enabling old-style Objective-C message prototypes.
	rem // note: the extra -U is necessary here because the character '1' next to '>>' would else be interpreted as a stdout redirection
	echo -DOBJC_OLD_DISPATCH_PROTOTYPES=1 -U____becauseWeAreBuildingForiOS7____>>       "!TARGET_BUILD_DIR!\target-agnostic.compflags"
)

rem // if we want remote logging, reroute the program's entrypoint
if not "_!REMOTELOG_IP!_"=="__" (
	set CPPFLAGS=!CPPFLAGS! -Dmain^(argc,argv^)=__attribute__^(^(visibility^(\"default\"^)^)^)_rdc_main^(argc,argv^) -DREMOTE_CONSOLE=\"!REMOTELOG_IP!\"
	echo -exported_symbol *_rdc_main* >> "!TEMPDIR!\LDFLAGS.txt"
)

rem // locate the C++ includes and add their search path appropriately. Look in new path first, old path next.
rem // NOTE: -I requires an ABSOLUTE PATH
set CXX_DIRFLAG=
if "_!CXX_DIRFLAG!_"=="__" if exist "!SDK_PATH!\lib\c++\v1\cstdio"         set CXX_DIRFLAG=-I"!SDK_PATH!\lib\c++\v1"
if "_!CXX_DIRFLAG!_"=="__" if exist "!SDK_PATH!\usr\include\c++\v1\cstdio" set CXX_DIRFLAG=-I"!SDK_PATH!\usr\include\c++\v1"
rem // now add the C++ includes search path to the compiler flags, turning backslashes into forward slashes
set CXXFLAGS=!CXXFLAGS! !CXX_DIRFLAG:\=/!

rem // account for inherited CPPFLAGS
if exist "!TEMPDIR!\INHERITED_CPPFLAGS.txt" (
	echo -U________CommonFlags__FromBuildCmd__Inherited____________________>>           "!TARGET_BUILD_DIR!\target-agnostic.compflags"
	type "!TEMPDIR!\INHERITED_CPPFLAGS.txt" >>                                          "!TARGET_BUILD_DIR!\target-agnostic.compflags"
)

rem // split PCH variable in words and build the PCH related flags
for %%p in (!PCH!) do echo  -include %%p>>                                                  "!TARGET_BUILD_DIR!\target-agnostic.compflags"

rem // are linker flags defined ? if so, build the target-agnostic.ldflags file with them, else create an empty file
if exist "!TEMPDIR!\LDFLAGS.txt" (
	rem // HACK: some filtering is necessary here. Unity il2cpp projects sometimes produce garbage in OTHER_LDFLAGS.
	echo s^|\x25CONFIGURATION_BUILD_DIR\x25/il2cpp\.a^|!BUILD_DIR!/!CONFIGURATION!-!SDK_PLATFORM_NAME!/BuiltLibraries/il2cpp.a^| > "!TARGET_BUILD_DIR!\target-agnostic.ldflags.sed"
	type "!TEMPDIR!\LDFLAGS.txt"|"!TOOLCHAIN_PATH!\sed.exe" -f "!TARGET_BUILD_DIR!\target-agnostic.ldflags.sed" > "!TARGET_BUILD_DIR!\target-agnostic.ldflags"
) else (
	copy /y nul "!TARGET_BUILD_DIR!\target-agnostic.ldflags" > nul
)

rem // if requested AND if we're using the Apple linker, add our handy special flag to instruct the linker to automatically identify missing symbols
if not "_!WANT_AUTORESOLVE!_"=="__" if "_!WANT_LLVMLINKER!_"=="__" echo -hint_missing>> "!TARGET_BUILD_DIR!\target-agnostic.ldflags"

rem // are the frameworks defined ? if so, build the frameworks.ldflags file with them, else create one from scratch with all the known and existing frameworks
if exist "!TEMPDIR!\FRAMEWORKS_LDFLAGS.txt" (
	copy /y "!TEMPDIR!\FRAMEWORKS_LDFLAGS.txt" "!TARGET_BUILD_DIR!\frameworks.ldflags" > nul 2>&1
) else (
	copy /y nul "!TARGET_BUILD_DIR!\frameworks.ldflags" > nul

	rem // look up in the Toolchain/frameworks.db file for the frameworks we may use according to the platform for which we're building
	if exist "!TOOLCHAIN_PATH!\frameworks.db" (
		if        "_!SDK_PLATFORM_NAME!_"=="_iphoneos_"  ( set SEARCHKEY=iOS
		) else if "_!SDK_PLATFORM_NAME!_"=="_macosx_"    ( set SEARCHKEY=macOS
		) else if "_!SDK_PLATFORM_NAME!_"=="_watchos_"   ( set SEARCHKEY=watchOS
		) else if "_!SDK_PLATFORM_NAME!_"=="_appletvos_" ( set SEARCHKEY=tvOS
		) else if "_!SDK_PLATFORM_NAME!_"=="_xros_"      ( set SEARCHKEY=visionOS
		)
		set /a MYVERS=10000*!MINOSVER_MAJOR!+100*!MINOSVER_MINOR!+1*!MINOSVER_REVIS!>nul
		rem // read one line of that file after the other
		for /f "usebackq delims=" %%a in ("!TOOLCHAIN_PATH!\frameworks.db") do (
			set LINE=%%a
			rem // identify framework name
			for /f "tokens=1 delims=	" %%a in ("!LINE!") do set FWNAME=%%a
			rem // is it includable at all, i.e. does it have either a binary file or a text symbol description file ?
			set IS_INCLUDABLE=false
			if exist "!SDK_PATH!\System\Library\Frameworks\!FWNAME!.framework\!FWNAME!"     set IS_INCLUDABLE=true
			if exist "!SDK_PATH!\System\Library\Frameworks\!FWNAME!.framework\!FWNAME!.tbd" set IS_INCLUDABLE=true
			if "_!IS_INCLUDABLE!_"=="_true_" (
				rem // read one availability tag after the other and identify the one we're looking for
				set FWVERS=
				for /f "tokens=2,3,4,5,6 delims=^	" %%a in ("!LINE!") do (
					set AVAILABILITY=%%a
					for /f "tokens=1,2 delims= " %%a in ("!AVAILABILITY!") do if "_%%a"=="_!SEARCHKEY!" set FWVERS=%%b
				)
				rem // does this line concern our platform ? if so, strip the + from the version tag and see if we should include that framework
				if not "_!FWVERS!_"=="__" (
					for /f "tokens=1,2 delims=+ " %%a in ("!FWVERS!") do set /a FWVERS=10000*%%a+100*%%b+1*%%c>nul 2>&1
					if !FWVERS! LEQ !MYVERS! (
						rem // we should include this framework: do so
						if not "_!DEBUG_ARG!_"=="__" echo Notice: will auto-link against system framework: !FWNAME! ^(min version: !FWVERS! ^<= !MYVERS!^)
						echo -framework !FWNAME!>> "!TARGET_BUILD_DIR!\frameworks.ldflags"
					)
				)
			)
		)
		set AVAILABILITY=
		set SEARCHKEY=
		set FWVERS=
		set MYVERS=
		set LINE=
	) else echo Warning: !TOOLCHAIN_PATH!\frameworks.db file is missing, automatic addition of system frameworks will be disabled.
)


rem // prepare list of thin files
set THINFILES_INPUT=


rem //////////////////////////
rem // pre-parse project files

echo  + Pre-parsing project files...

rem // are we building a Xcode project ? if so, it may have a headers build phase, which we must construct before building the sources themselves
if exist "!XCNAME!.xcodeproj\project.pbxproj" (
	rem // create the headers\[Project|Private|Public]\ProjectName directories if they doesn't exist yet
	if not exist "!TARGET_BUILD_DIR!\headers-project\!NAME!\." mkdir "!TARGET_BUILD_DIR!\headers-project\!NAME!" > nul
	if not exist "!TARGET_BUILD_DIR!\headers-private\!NAME!\." mkdir "!TARGET_BUILD_DIR!\headers-private\!NAME!" > nul
	if not exist "!TARGET_BUILD_DIR!\headers-public\!NAME!\."  mkdir "!TARGET_BUILD_DIR!\headers-public\!NAME!"  > nul
	rem // list the header files to rearrange (PBXHeadersBuildPhase)
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -headers -visibility project > "!TARGET_BUILD_DIR!\headers-project\filelist.txt"
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -headers -visibility private > "!TARGET_BUILD_DIR!\headers-private\filelist.txt"
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -headers -visibility public  > "!TARGET_BUILD_DIR!\headers-public\filelist.txt"
	rem // for each header file, create a symlink (as our versions of clang and ld64 support emulated POSIX symlinks, an emulated symlink will be enough instead of a copy)
	for /f "usebackq delims=" %%a in ("!TARGET_BUILD_DIR!\headers-project\filelist.txt") do (
		set HEADER_ABSPATHNAME=%%~fa
		set HEADER_NAME=%%~nxa
		echo|set /p="!HEADER_ABSPATHNAME!" > "!TARGET_BUILD_DIR!\headers-project\!NAME!\!HEADER_NAME!"
	)
	for /f "usebackq delims=" %%a in ("!TARGET_BUILD_DIR!\headers-private\filelist.txt") do (
		set HEADER_ABSPATHNAME=%%~fa
		set HEADER_NAME=%%~nxa
		echo|set /p="!HEADER_ABSPATHNAME!" > "!TARGET_BUILD_DIR!\headers-private\!NAME!\!HEADER_NAME!"
	)
	for /f "usebackq delims=" %%a in ("!TARGET_BUILD_DIR!\headers-public\filelist.txt") do (
		set HEADER_ABSPATHNAME=%%~fa
		set HEADER_NAME=%%~nxa
		echo|set /p="!HEADER_ABSPATHNAME!" > "!TARGET_BUILD_DIR!\headers-public\!NAME!\!HEADER_NAME!"
	)
	rem // finally, make sure the compiler looks for headers in these directories, in this order: Project headers, then Private headers, then Public headers
	echo -U________IncludePaths__FromXcode_____BuiltHeadersPaths______>> "!TARGET_BUILD_DIR!\incpaths.compflags"
	echo -I"!TARGET_BUILD_DIR:\=/!/headers-project" >> "!TARGET_BUILD_DIR!\incpaths.compflags"
	echo -I"!TARGET_BUILD_DIR:\=/!/headers-private" >> "!TARGET_BUILD_DIR!\incpaths.compflags"
	echo -I"!TARGET_BUILD_DIR:\=/!/headers-public"  >> "!TARGET_BUILD_DIR!\incpaths.compflags"
)

rem // build the list of source files in the intermediate directory
if exist "!XCNAME!.xcodeproj\project.pbxproj" (
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -sources > "!TARGET_BUILD_DIR!\srclist.txt"
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -staticlibs >> "!TARGET_BUILD_DIR!\srclist.txt"
) else (
	"!TOOLCHAIN_PATH!\list-source-files.exe" "!NAME!" "Source Files" "!OUTDIR!" > "!TARGET_BUILD_DIR!\srclist.txt"
)

rem // if we want remote logging, add the remote debug console magic to the list of source files
if not "_!REMOTELOG_IP!_"=="__" echo !TOOLCHAIN_PATH!/extra/rdc.m>> "!TARGET_BUILD_DIR!\srclist.txt"

rem // if we are building a Xcode project, setup the include paths now. These can be quite long,
rem // so store them in a file instead of in an environment variable to work around the command line size limit.
if exist "!XCNAME!.xcodeproj\project.pbxproj" (
	rem // figure out both the implicit and explicit compiler include paths from the .pbxproj file
	echo -U________IncludePaths__FromXcode_____ExplicitFrameworkPaths______>> "!TARGET_BUILD_DIR!\incpaths.compflags"
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -frameworkpaths                 -line_format " -F\"%%s\"\n" >> "!TARGET_BUILD_DIR!\incpaths.compflags"
	echo -U________IncludePaths__FromXcode_____FRAMEWORK_SEARCH_PATHS______>> "!TARGET_BUILD_DIR!\incpaths.compflags"
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query FRAMEWORK_SEARCH_PATHS   -line_format " -F\"%%s\"\n" >> "!TARGET_BUILD_DIR!\incpaths.compflags"
	echo -U________IncludePaths__FromXcode_____HEADER_SEARCH_PATHS_________>> "!TARGET_BUILD_DIR!\incpaths.compflags"
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query HEADER_SEARCH_PATHS      -line_format " -I\"%%s\"\n" >> "!TARGET_BUILD_DIR!\incpaths.compflags"
	echo -U________IncludePaths__FromXcode_____USER_HEADER_SEARCH_PATHS____>> "!TARGET_BUILD_DIR!\incpaths.compflags"
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query USER_HEADER_SEARCH_PATHS -line_format " -I\"%%s\"\n" >> "!TARGET_BUILD_DIR!\incpaths.compflags"
	echo -U________IncludePaths__FromXcode_____SRCROOT_____________________>> "!TARGET_BUILD_DIR!\incpaths.compflags"
	echo  -I"!SRCROOT!">>                                                                                                                                                                                                 "!TARGET_BUILD_DIR!\incpaths.compflags"
	echo -U________IncludePaths__FromXcode_____ImplicitHeaderPaths_________>> "!TARGET_BUILD_DIR!\incpaths.compflags"
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -headerpaths                    -line_format " -idirafter\"%%s\"\n" >> "!TARGET_BUILD_DIR!\incpaths.compflags"

	rem // Unity-specific hacks
	if "_!XCNAME!_"=="_Unity-iPhone_" (
		rem // HACK: for some reason, this include path is nowhere to be found in some Unity Xcode project files... Unsure if it's still the case, leaving it here for compatibility reasons.
		if exist "Libraries\libil2cpp\include\." (
			findstr "Libraries/libil2cpp/include" "!TARGET_BUILD_DIR!\incpaths.compflags" >nul 2>&1 || (
				rem // only patch when necessary
				echo -U________IncludePaths__FromBuildCmd__UnitySpecificHack___________>> "!TARGET_BUILD_DIR!\incpaths.compflags"
				echo  -ILibraries/libil2cpp/include >> "!TARGET_BUILD_DIR!\incpaths.compflags"
			)
		)

		rem // HACK: the NativeGallery Unity plugin makes an invalid pointer assignment in its iOS code, which Xcode's clang silently allows,
		rem // which is certainly an error. Replicate this behaviour in our compiler by hotpatching the plugin.
		rem // FIXME: this assignment also exists in other plugins. Find a way to make clang swallow it.
		if exist "Libraries\Plugins\NativeGallery\iOS\NativeGallery.mm" findstr "\.delegate = self;" "Libraries\Plugins\NativeGallery\iOS\NativeGallery.mm" > nul 2>&1 && (
			findstr /c:"iOS Build Environment patch" "Libraries\Plugins\NativeGallery\iOS\NativeGallery.mm" > nul 2>&1 || (
				rem // only patch when necessary
				copy "Libraries\Plugins\NativeGallery\iOS\NativeGallery.mm" "Libraries\Plugins\NativeGallery\iOS\NativeGallery.mm.bak"
				"!TOOLCHAIN_PATH!\sed.exe" -e "s/\.delegate = self;/\.delegate = (id)self; \/\* iOS Build Environment patch\: pointer assignment is invalid without a cast \*\//g" "Libraries\Plugins\NativeGallery\iOS\NativeGallery.mm.bak" > "Libraries\Plugins\NativeGallery\iOS\NativeGallery.mm"
			)
		)

		rem // HACK: it looks like Unity doesn't correctly generate the code to interface with the Facebook SDK. One of their methods is defined in Objective-C
		rem // and they call it as extern C. I'm not sure whose fault it is (Facebook plugin provider or Unity), but hotpatching might declutter my mailbox.
		if exist "Libraries\FacebookSDK\SDK\Editor\iOS\FBUnityInterface.mm" findstr /c:"\-\ \(void\)IOSFBEnableProfileUpdatesOnAccessTokenChange\:\(BOOL\)enable" "Libraries\FacebookSDK\SDK\Editor\iOS\FBUnityInterface.mm" > nul 2>&1 && findstr /c:"[FBSDKProfile\ enableUpdatesOnAccessTokenChange:enable]" "Libraries\FacebookSDK\SDK\Editor\iOS\FBUnityInterface.mm" > nul 2>&1 && (
			findstr /c:"iOS Build Environment patch" "Libraries\FacebookSDK\SDK\Editor\iOS\FBUnityInterface.mm" > nul 2>&1 || (
				rem // only patch when necessary
				echo extern "C" void IOSFBEnableProfileUpdatesOnAccessTokenChange^(BOOL enable^) { [FBSDKProfile enableUpdatesOnAccessTokenChange:enable]; } /* iOS Build Environment patch: seemingly missing C interface */>> "Libraries\FacebookSDK\SDK\Editor\iOS\FBUnityInterface.mm"
			)
		)

		rem // HACK: Google's VR SDK doesn't add its library and header search paths to the main .pbxproj file. Also, it handles
		rem // resources through a shell script, which we can't evaluate. ATM we hardcode them here.
		if exist "Pods\Headers\Public\GVRSDK\." if "_!SKIP_COCOAPODS!_"=="__" (
			echo -LPods\GVRSDK\Libraries -lGVRSDK>> "!TARGET_BUILD_DIR!\target-agnostic.ldflags"
			echo -U________IncludePaths__FromBuildCmd__GVRSDKSpecificHack__________>> "!TARGET_BUILD_DIR!\incpaths.compflags"
			echo -idirafter"Pods/Headers/Public/GVRSDK">>                             "!TARGET_BUILD_DIR!\incpaths.compflags"
			echo  -ObjC -framework GLKit>> "!TARGET_BUILD_DIR!\frameworks.ldflags"
			set RES=!RES! Pods\GVRSDK\Resources\*.bundle
		)

		rem // HACK: Microsoft's Azure Spatial Anchors SDK uses a shell script to binary embed its binary framework, which it supplies
		rem // as a pre-compiled pod (defeating the purpose of CocoaPods but whatever.) ATM we need to hardcode it in the embeddable binaries.
		rem // TODO: as of 2021-02-25 with version 3.31.4 is this still necessary ? compiling and linking should be OK, but what about embedding ? VERIFY
		rem // ASSUMPTION: by fixing rpaths in the app's binaries we should fetch the framework from its original location, so no need to hack anything any longer here
		rem //if exist "Pods\AzureSpatialAnchors\bin\frameworks\AzureSpatialAnchors.framework\AzureSpatialAnchors" if "_!SKIP_COCOAPODS!_"=="__" (
		rem //	rem // FIXME: impossible: find a more elegant and genericizable way of doing this sort of things...
		rem //	echo Pods\AzureSpatialAnchors\bin\frameworks\AzureSpatialAnchors.framework>> "!TEMPDIR!\EMBEDDED_BINARIES.txt"
		rem //)
	)
)

rem // if we have Pods dummy frameworks, remove them from the linker options. Same thing for dummy libraries from input files.
findstr "Pods" "!TARGET_BUILD_DIR!\frameworks.ldflags" > nul 2>&1 && (
	copy /b "!TARGET_BUILD_DIR!\frameworks.ldflags" "!TEMPDIR!\frameworks.ldflags.tmp" > nul
	type "!TEMPDIR!\frameworks.ldflags.tmp" | findstr /v "Pods-Unity-iPhone Pods_Unity_iPhone Pods-UnityFramework Pods_UnityFramework" > "!TARGET_BUILD_DIR!\frameworks.ldflags"
	del /f /q "!TEMPDIR!\frameworks.ldflags.tmp"
)
findstr "Pods" "!TARGET_BUILD_DIR!\srclist.txt" > nul 2>&1 && (
	copy /b "!TARGET_BUILD_DIR!\srclist.txt" "!TEMPDIR!\srclist.txt.tmp" > nul
	type "!TEMPDIR!\srclist.txt.tmp" | findstr /v "Pods-Unity-iPhone Pods_Unity_iPhone Pods-UnityFramework Pods_UnityFramework" > "!TARGET_BUILD_DIR!\srclist.txt"
	del /f /q "!TEMPDIR!\srclist.txt.tmp"
)
if exist "Pods\." if "_!SKIP_COCOAPODS!_"=="__" (
	rem // FIXME: when some Firebase frameworks are built from Pods, it's likely that the Security framework is necessary. It may not always be explicitly included (FIND OUT WHY)
	rem // FIXME: also, the necessary framework and library specifications are missing from the Xcode project file AND the .xcconfig file! (FIND OUT WHERE XCODE TAKES THEM FROM)
rem // 20230527: the problem automagically disappeared once LLVM modules were enabled!?
rem	set SHOULD_FIX_GOOGLE_POO=
rem	if    "_!COCOAPODS_PROJECTNAME!_"=="_GoogleDataTransport_" ( set SHOULD_FIX_GOOGLE_POO=true
rem	) else if "_!COCOAPODS_PROJECTNAME!_"=="_GoogleUtilities_" ( set SHOULD_FIX_GOOGLE_POO=true
rem	)
rem	if "_!SHOULD_FIX_GOOGLE_POO!_"=="_true_" (
rem		findstr /c:"-framework Security"            "!TARGET_BUILD_DIR!\frameworks.ldflags" > nul 2>&1 || echo -framework Security>>            "!TARGET_BUILD_DIR!\frameworks.ldflags"
rem		findstr /c:"-framework SystemConfiguration" "!TARGET_BUILD_DIR!\frameworks.ldflags" > nul 2>&1 || echo -framework SystemConfiguration>> "!TARGET_BUILD_DIR!\frameworks.ldflags"
rem		findstr /c:"-framework CoreTelephony"       "!TARGET_BUILD_DIR!\frameworks.ldflags" > nul 2>&1 || echo -framework CoreTelephony>>       "!TARGET_BUILD_DIR!\frameworks.ldflags"
rem		if exist "Pods\nanopb\."       findstr /c:"-framework nanopb"      "!TARGET_BUILD_DIR!\frameworks.ldflags" > nul 2>&1 || echo -framework nanopb>>      "!TARGET_BUILD_DIR!\frameworks.ldflags"
rem		if exist "Pods\PromisesObjC\." findstr /c:"-framework FBLPromises" "!TARGET_BUILD_DIR!\frameworks.ldflags" > nul 2>&1 || echo -framework FBLPromises>> "!TARGET_BUILD_DIR!\frameworks.ldflags"
rem		findstr /c:"-lz" "!TARGET_BUILD_DIR!\target-agnostic.ldflags" > nul 2>&1 || echo -lz >> "!TARGET_BUILD_DIR!\target-agnostic.ldflags"
rem	)
	rem // FIXME: FirebaseCore 10.x requires these extra frameworks at link stage (FIND OUT WHERE XCODE TAKES THEM FROM)
	if "_!COCOAPODS_PROJECTNAME!_"=="_FirebaseCore_" (
		if exist "Pods\GoogleUtilities\."      findstr /c:"-framework GoogleUtilities"      "!TARGET_BUILD_DIR!\frameworks.ldflags" > nul 2>&1 || echo -framework GoogleUtilities>>      "!TARGET_BUILD_DIR!\frameworks.ldflags"
		if exist "Pods\FirebaseCoreInternal\." findstr /c:"-framework FirebaseCoreInternal" "!TARGET_BUILD_DIR!\frameworks.ldflags" > nul 2>&1 || echo -framework FirebaseCoreInternal>> "!TARGET_BUILD_DIR!\frameworks.ldflags"
	)
)

rem // finally, add the current directory to the header search paths (this seems to be implicit even on Xcode projects)
if exist "!XCNAME!.xcodeproj\project.pbxproj" (
	rem // if this is a Xcode project, add the Xcode project directory
	echo -U________IncludePaths__FromBuildCmd__ProjectDirectory____________>> "!TARGET_BUILD_DIR!\incpaths.compflags"
	echo -I"!XCDIR!">>                                                        "!TARGET_BUILD_DIR!\incpaths.compflags"
)
echo -U________IncludePaths__FromBuildCmd__CurrentDirectory____________>>         "!TARGET_BUILD_DIR!\incpaths.compflags"
echo -I.>>                                                                        "!TARGET_BUILD_DIR!\incpaths.compflags"


rem // assume project contains no Objective-C code until told otherwise and reset state variables
set OBJC_LDFLAGS=
set LAST_BUILT_OBJFILE=
set BUILD_FAILED=
set SHOULD_CREATE_DSYM=

rem // cleanup build directory
for %%a in (!ARCHS!) do ( 
	set ARCH=%%a

	rem // (re)create the target-specific intermediate directory if needed and cleanup previous leftovers
	mkdir "!TARGET_BUILD_DIR!\!ARCH!" > nul 2>&1
	del /f /q "!TARGET_BUILD_DIR!\!ARCH!\compilation-failed.txt" > nul 2>&1
	copy /y nul "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags" > nul
	copy /y nul "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt" > nul
)

rem // now build the project
echo  + Building !OUTFILE! for !SDK_PLATFORM_NAME! ^(configuration: !CONFIGURATION!^)

rem // Unity-specific IL2CPP step
if exist "!XCNAME!.xcodeproj\project.pbxproj" if "_!XCNAME!_"=="_Unity-iPhone_" if "_!TARGET_NAME!_"=="_GameAssembly_" (
	rem // starting with Unity 2022.2.1f1, the il2cpp library is compiled on the host into a GameAssembly.a static library using a Unity-supplied pipeline.
	rem // this is done through a PBXShellScriptBuildPhase in the Xcode project and obviously, Unity assumes it's running on a Mac.
	rem // Example of such a script (de-inlined:)
	rem //    HOST_ARCH=$(uname -m)
	rem //    IL2CPP_DIR="$PROJECT_DIR/Il2CppOutputProject/IL2CPP/build/deploy_$HOST_ARCH"
	rem //    IL2CPP="$IL2CPP_DIR/il2cpp"
	rem //    chmod +x "$IL2CPP"
	rem //    chmod +x "$IL2CPP_DIR/bee_backend/mac-$HOST_ARCH/bee_backend"
	rem //    if [ "$CONFIGURATION" = "Debug" ]; then
	rem //        IL2CPP_CONFIG="Debug"
	rem //    else
	rem //        IL2CPP_CONFIG="Release"
	rem //    fi
	rem //    "$IL2CPP" --compile-cpp \
	rem //              --platform=iOS \
	rem //              --architecture=arm64 \
	rem //              --outputpath="libGameAssembly.a" \
	rem //              --libil2cpp-static \
	rem //              --baselib-directory="$PROJECT_DIR/Libraries" \
	rem //              --additional-defines=IL2CPP_DEBUG=0 \
	rem //              --incremental-g-c-time-slice=3 \
	rem //              --configuration=Release \
	rem //              --dotnetprofile=unityaot-macos \
	rem //              --profiler-report \
	rem //              --profiler-output-file="$CONFIGURATION_TEMP_DIR/artifacts/profile.json" \
	rem //              --print-command-line \
	rem //              --external-lib-il2-cpp="$PROJECT_DIR/Libraries/libil2cpp.a" \
	rem //              --data-folder="C:/Users/indon/Unity/MapBoxOrcAdventure/Library/Bee/artifacts/iOS/il2cppOutput/data" \
	rem //              --generatedcppdir="Il2CppOutputProject/Source/il2cppOutput" \
	rem //              --architecture=arm64 \
	rem //              --outputpath="$CONFIGURATION_BUILD_DIR/libGameAssembly.a" \
	rem //              --cachedirectory="$CONFIGURATION_TEMP_DIR/artifacts/arm64" \
	rem //              --configuration="$IL2CPP_CONFIG"
	rem // To work this around, cross-compilation ability was added to il2cpp in January 2022 using the --tool-chain-path and --sysroot-path arguments.
	rem // Also, luckily, il2cpp.exe can overwrite its own parameters, so we're gonna just use that to fix the bogus ones.
	rem // retrieve the Unity runtime version from the "iPhone-target" top target
	set IL2CPP=
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" -target "iPhone-target" -configuration "!CONFIGURATION!" -query UNITY_RUNTIME_VERSION > %TO_TEMPVAR%
	set UNITY_RUNTIME_VERSION=!TEMPVAR!
	rem // first, see if this project embeds an il2cpp compiler
	if exist "Il2CppOutputProject\IL2CPP\build\deploy_win64\il2cpp.exe" set IL2CPP=Il2CppOutputProject\IL2CPP\build\deploy_win64\il2cpp.exe
	rem // have we NOT found it ?
	if not exist "!IL2CPP!" (
		rem // locate this Unity installation, and foremost its IL2CPP host compiler executable
		for /F "usebackq tokens=3,*" %%a IN (`reg query "HKEY_CURRENT_USER\Software\Unity Technologies\Installer\Unity !UNITY_RUNTIME_VERSION!" /v "Location x64" 2^>nul ^| find "Location x64"`) do (
			if exist "%%b\Editor\Data\il2cpp\build\deploy\il2cpp.exe" set IL2CPP=%%b\Editor\Data\il2cpp\build\deploy\il2cpp.exe
		)
		rem // in case we haven't found it, look in "%ProgramFiles%\Unity" (default Unity install path) and "%ProgramFiles%\Unity !UNITY_RUNTIME_VERSION!"
		if not exist "!IL2CPP!" if exist "%ProgramFiles%\Unity\Hub\Editor\!UNITY_RUNTIME_VERSION!\Editor\Data\il2cpp\build\deploy\il2cpp.exe" set IL2CPP=%ProgramFiles%\Unity\Hub\Editor\!UNITY_RUNTIME_VERSION!\Editor\Data\il2cpp\build\deploy\il2cpp.exe
		if not exist "!IL2CPP!" if exist "%ProgramFiles%\Unity !UNITY_RUNTIME_VERSION!\Editor\Data\il2cpp\build\deploy\il2cpp.exe" set IL2CPP=%ProgramFiles%\Unity !UNITY_RUNTIME_VERSION!\Editor\Data\il2cpp\build\deploy\il2cpp.exe
		rem // have we found it ? if so, it's a good idea to make a local copy of it (75 Mb) in the Xcode project tree itself
		rem // this will make the Xcode project buildable on machines where Unity isn't installed.
		if exist "!IL2CPP!" (
			echo    + Creating portable copy of Unity il2cpp compiler: !UNITY_RUNTIME_VERSION!
			xcopy "!IL2CPP!\.." "Il2CppOutputProject\IL2CPP\build\deploy_win64" /d /e /i /y > nul || goto :exit_error
			set IL2CPP=Il2CppOutputProject\IL2CPP\build\deploy_win64\il2cpp.exe
			rem // drop a note about the Unity version it is for file search purpose
			echo Unity il2cpp build tools !UNITY_RUNTIME_VERSION! were copied locally on %DATE% to deploy_win64> "Il2CppOutputProject\IL2CPP\build\!UNITY_RUNTIME_VERSION!"
		)
	)
	rem // have we NOT found it ?
	if not exist "!IL2CPP!" (
		echo Error: this Unity project requires the Unity !UNITY_RUNTIME_VERSION! IL2CPP host compiler installed on your machine. Please install Unity version !UNITY_RUNTIME_VERSION!, then try again.
		goto :exit_error
	)
	echo    + Using Unity il2cpp compiler: !UNITY_RUNTIME_VERSION!
	rem // make sure we can address the SDK and toolchain using pathnames without spaces. This is necessary because il2cpp doesn't support spaces in its --tool-chain-path and --sysroot-path arguments.
	if not exist "C:\iOS\builder\SDK\Info.plist" (
		echo Error: the C:\iOS\builder symlink doesn't point to %SCRIPT_DIRECTORY%. You may have deleted it. Please either create it anew or reinstall the build environment.
		goto :exit_error
	)
	if not exist "C:\iOS\builder\Toolchain\clang.exe" (
		echo Error: the C:\iOS\builder symlink doesn't point to %SCRIPT_DIRECTORY%. You may have deleted it. Please either create it anew or reinstall the build environment.
		goto :exit_error
	)
	rem // make sure we have only one single arch -- currently Unity builds the il2cpp target in a hardcoded way.
	if not "_!ARCHS: =!_"=="_!ARCHS!_" (
		echo Error: building the il2cpp library for multiple targets is unsupported yet. Please report this problem.
		goto :exit_error
	)
	set IL2CPP_ARCH=!ARCHS!
	rem // figure out the il2cpp command-line from this project. Note that turning backslashes into forward slashes is mandatory for pbxprojparse not to interpret them as escape characters!
	rem // 1. start with an "@echo off"
	rem // 2. create the il2cpp build state directory (mimics what the shell script in the Xcode project does)
	rem // 3. create a symbolic link to the Il2CppOutputProject source directory in the il2cpp build state directory (mimics what the shell script in the Xcode project does)
	rem // 4. create a symbolic link to the Libraries source directory in the il2cpp build state directory (mimics what the shell script in the Xcode project does)
	rem // 5. write the il2cpp.exe invokation line, this way:
	rem //    5.1 extract the shell script associated with the GameAssembly target from the Xcode project and do the proper variable replacements
	rem //    5.2 isolate the relevant line, which necessarily contains the " --compile-cpp" substring
	rem //    5.3 do platform-specific transformations with sed (see below)
	rem // 6. append a "exit /b %errorlevel%" to the il2cpp invokation script
	set IL2CPP_CONFIG=ReleaseSize
	if "_!CONFIGURATION!_"=="_Debug_" set IL2CPP_CONFIG=Debug
	mkdir "!TARGET_BUILD_DIR!\temp" > nul 2>&1
	rem // HEAD of the il2cpp.cmd script
	(
		rem // steps 1, 2, 3 and 4 of the above list
		rem // for the Il2CppOutputProject and Libraries symlinks, try to emulate them as NTFS junctions first, then directory symlinks, and if it fails then fallback to a robocopy
		rem // (emulated POSIX symlinks would be ideal, but unfortunately they aren't handled by the Unity il2cpp tools).
		echo @echo off
		echo if not exist "!TARGET_BUILD_DIR!\temp\artifacts\!IL2CPP_ARCH!\buildstate\." mkdir "!TARGET_BUILD_DIR!\temp\artifacts\!IL2CPP_ARCH!\buildstate"
		echo if not exist "!TARGET_BUILD_DIR!\temp\artifacts\!IL2CPP_ARCH!\buildstate\Il2CppOutputProject" mklink /j "!TARGET_BUILD_DIR!\temp\artifacts\!IL2CPP_ARCH!\buildstate\Il2CppOutputProject" "%CD%\Il2CppOutputProject" ^> nul 2^>^&1
		echo if not exist "!TARGET_BUILD_DIR!\temp\artifacts\!IL2CPP_ARCH!\buildstate\Il2CppOutputProject" mklink /d "!TARGET_BUILD_DIR!\temp\artifacts\!IL2CPP_ARCH!\buildstate\Il2CppOutputProject" "%CD%\Il2CppOutputProject" ^> nul 2^>^&1
		echo if not exist "!TARGET_BUILD_DIR!\temp\artifacts\!IL2CPP_ARCH!\buildstate\Il2CppOutputProject" robocopy "%CD%\Il2CppOutputProject" "!TARGET_BUILD_DIR!\temp\artifacts\!IL2CPP_ARCH!\buildstate\Il2CppOutputProject" /is /e /njh /ndl /nfl /njs ^> nul 2^>^&1
		echo if not exist "!TARGET_BUILD_DIR!\temp\artifacts\!IL2CPP_ARCH!\buildstate\Libraries" mklink /j "!TARGET_BUILD_DIR!\temp\artifacts\!IL2CPP_ARCH!\buildstate\Libraries" "%CD%\Libraries" ^> nul 2^>^&1
		echo if not exist "!TARGET_BUILD_DIR!\temp\artifacts\!IL2CPP_ARCH!\buildstate\Libraries" mklink /d "!TARGET_BUILD_DIR!\temp\artifacts\!IL2CPP_ARCH!\buildstate\Libraries" "%CD%\Libraries" ^> nul 2^>^&1
		echo if not exist "!TARGET_BUILD_DIR!\temp\artifacts\!IL2CPP_ARCH!\buildstate\Libraries" robocopy "%CD%\Libraries" "!TARGET_BUILD_DIR!\temp\artifacts\!IL2CPP_ARCH!\buildstate\Libraries" /is /e /njh /ndl /nfl /njs ^> nul 2^>^&1
	) > "!TARGET_BUILD_DIR!\il2cpp.cmd"
	rem // BODY of the il2cpp.cmd script
	rem // steps 5.1 and 5.2: extract the il2cpp command line invokation from the Xcode project
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" -target "GameAssembly" -configuration "!CONFIGURATION!" -shellscripts ^
                                            IL2CPP="!IL2CPP:\=/!" ^
                                            IL2CPP_CONFIG="!IL2CPP_CONFIG!" ^
                                            PROJECT_DIR="!CD:\=/!" ^
                                            CONFIGURATION_BUILD_DIR="!CD:\=/!/!TARGET_BUILD_DIR:\=/!" ^
                                            CONFIGURATION_TEMP_DIR="!CD:\=/!/!TARGET_BUILD_DIR:\=/!/temp" | find " --compile-cpp" > "!TARGET_BUILD_DIR!\il2cpp.cmdline.xcode"
	rem // have we NOT found it ?
	find " --compile-cpp" "!TARGET_BUILD_DIR!\il2cpp.cmdline.xcode" > nul 2>&1 || (
		echo Error: unable to figure out the IL2CPP command line. Please report this problem.
		goto :exit_error
	)
	rem // step 5.3: transform the il2cpp command line that we read out of the Xcode project file
	rem // NOTE: cmd.exe would break here because of quotes. We have to prepare a sed command file.
	(
		rem // remove the --profiler-report il2cpp option
		echo s^| --profiler-report^|^|g

		rem // disable or enable verbosity options depending on whether we want debug output
		if     "_!DEBUG_ARG!_"=="__" echo s^| --print-command-line^|^|g
		if not "_!DEBUG_ARG!_"=="__" echo s^|$^| --verbose^|g

		rem // replace --data-folder=xxxxx with one that is guaranteed to not contain spaces
		echo s^| --data-folder="[^"]+.^|^|g
		echo s^|$^| --data-folder=C:/iOS/TEMP/il2cppOutput/data^|

		rem // also replace arguments that come as relative paths with their absolute path equivalents
		rem // TODO: programmatically retrieve these folder names from the Xcode project file, in case they ever change
		rem // DISABLED: THIS JUST DOES NOT WORK AS DOCUMENTED. Instead we setup links to the Libraries and Il2CppOutputProject directories in the il2cpp build state dir.
		rem echo s^| --baselib-directory="[^"]+.^|^|g
		rem echo s^|$^| --baselib-directory="%CD:\=/%/Libraries"^|
		rem echo s^| --custom-il2-cpp-root="[^"]+.^|^|g
		rem echo s^|$^| --custom-il2-cpp-root="%CD:\=/%Il2CppOutputProject/"^|g

		rem // specify toolchain path, sysroot path and extra compiler flags for clang++
		echo s^|$^| --tool-chain-path=C:/iOS/builder/Toolchain^|
		echo s^|$^| --sysroot-path=C:/iOS/builder/SDK^|
		echo s^|$^| --compiler-flags="-Wno-nonportable-include-path -Wno-unused-command-line-argument -Wno-deprecated-non-prototype -target !IL2CPP_ARCH!-apple-ios"^|

		rem // also, de-verbose il2cpp if we're not in verbose mode
		if "_!DEBUG_ARG!_"=="__" echo s;$; ^| find /v "##### CommandLine" ^| find /v "clang++";
	) > "!TARGET_BUILD_DIR!\il2cpp.cmdline.sed"
	"!TOOLCHAIN_PATH!\sed.exe" -r -f "!TARGET_BUILD_DIR!\il2cpp.cmdline.sed" "!TARGET_BUILD_DIR!\il2cpp.cmdline.xcode" >> "!TARGET_BUILD_DIR!\il2cpp.cmd"
	rem // TAIL of the il2cpp script
	(
		rem // step 6 of the above list
		echo if errorlevel 1 exit /b 1
		echo exit /b 0
	) >> "!TARGET_BUILD_DIR!\il2cpp.cmd"
	rem // now call the il2cpp host compiler and pass it two additional flags for the sysroot and target triple
	echo    + Calling the Unity il2cpp compiler to build the game assembly library...
	echo      PLEASE NOTE: this step MAY take time, and Unity doesn't always display
	echo      its progress. You can tell it's working by checking your CPU activity.
	echo      It should display !CORES! instances of clang.exe associated with il2cpp.exe.
	echo      ======================= il2cpp output BEGIN =======================
	if not "_!DEBUG_ARG!_"=="__" (
		echo|set /p=Calling: 
		type "!TARGET_BUILD_DIR!\il2cpp.cmd"|find " --compile-cpp"
	)
	call "!TARGET_BUILD_DIR!\il2cpp.cmd" || (
		echo      ======================== il2cpp output END ========================
		echo Error: the Unity il2cpp compiler returned a non-success exit status.
		goto :exit_error
	)
	echo      ======================== il2cpp output END ========================
	if not exist "!TARGET_BUILD_DIR!\libGameAssembly.a" (
		echo Error: the Unity il2cpp compiler did not produce the expected output binary.
		goto :exit_error
	)
	rem // now move that library in place and delete the objects list file so that we don't attempt to create it ourselves
	if exist "!TARGET_BUILD_DIR!\libil2cpp.a" del /f /q "!TARGET_BUILD_DIR!\libil2cpp.a" > nul
	move /y "!TARGET_BUILD_DIR!\libGameAssembly.a" "!TARGET_BUILD_DIR!\!IL2CPP_ARCH!\!OUTFILE!.!IL2CPP_ARCH!" > nul
	if exist "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt" del /f /q "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt" > nul
	set THINFILES_INPUT=!THINFILES_INPUT! -arch !IL2CPP_ARCH! "!TARGET_BUILD_DIR!\!IL2CPP_ARCH!\!OUTFILE!.!IL2CPP_ARCH!"
)

rem // prepare a list of Swift source files (they'll need to be compiled altogether)
findstr ".swift$" "!TARGET_BUILD_DIR!\srclist.txt" > "!TARGET_BUILD_DIR!\swift-srclist.txt" && (

	rem // if it worked, it means we have at least some. See if we are building a mixed-language target (Swift + something else)
	rem // This will be the case if the Swift source list is smaller than the full source list
	fc "!TARGET_BUILD_DIR!\swift-srclist.txt" "!TARGET_BUILD_DIR!\srclist.txt" > nul 2>&1 || (

		rem // Swift source files must be compiled first so as to generate a C-compatible header file for use in mixed-language scenarii
rem // WIP		echo  + Creating Swift ObjC header...

		rem // create a specific "emit header" Swift compiler command line flags file and run swiftc

		rem // the list of Swift source files to parse together
		type "!TARGET_BUILD_DIR!\swift-srclist.txt"                                                 > "!TARGET_BUILD_DIR!\emitheader.compflags"
		rem // the location of the corresponding object files (with their -o command-line prefix)
		"!TOOLCHAIN_PATH!\sed.exe" -e "s|^^^|-o |g" -e "s|\.swift$|.swift.obj.emitheader|g" "!TARGET_BUILD_DIR!\swift-srclist.txt" >> "!TARGET_BUILD_DIR!\emitheader.compflags"
		rem // FIXME: document
rem		echo -import-underlying-module                                                             >> "!TARGET_BUILD_DIR!\emitheader.compflags"
		rem // enable ObjC interoperability, bringing in some knowledge of the Darwin ObjC runtime
		echo -enable-objc-interop                                                                  >> "!TARGET_BUILD_DIR!\emitheader.compflags"
		rem // have color diagnostics just like Clang does
		echo -color-diagnostics                                                                    >> "!TARGET_BUILD_DIR!\emitheader.compflags"
		rem // don't run diagnostic passes
		echo -disable-diagnostic-passes                                                            >> "!TARGET_BUILD_DIR!\emitheader.compflags"
		rem // don't optimize
		echo -disable-sil-perf-optzns                                                              >> "!TARGET_BUILD_DIR!\emitheader.compflags"
		rem // enable memory safety
		echo -enforce-exclusivity=checked                                                          >> "!TARGET_BUILD_DIR!\emitheader.compflags"
		rem // as it says
		echo -warn-on-potentially-unavailable-enum-case                                            >> "!TARGET_BUILD_DIR!\emitheader.compflags"
		rem // enable "let pattern = /.../" literals in Swift source code to be treated as regular expressions (available in Swift-5.8+)
		echo -enable-bare-slash-regex                                                              >> "!TARGET_BUILD_DIR!\emitheader.compflags"
		rem // pass the location where the Clang headers and layout files will be
		echo -resource-dir "!SDK_PATH!/../lib/swift"                                               >> "!TARGET_BUILD_DIR!\emitheader.compflags"
		rem // pass the location of the Swift modules in the SDK
		echo -I "!SDK_PATH!/usr/lib/swift"                                                         >> "!TARGET_BUILD_DIR!\emitheader.compflags"
		rem // pass the location of our 3rd party frameworks
		echo -F "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks"               >> "!TARGET_BUILD_DIR!\emitheader.compflags"
		echo -F "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks"                  >> "!TARGET_BUILD_DIR!\emitheader.compflags"
		echo -F "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\XCFrameworkIntermediates"         >> "!TARGET_BUILD_DIR!\emitheader.compflags"
		rem // create an Objective-C header for these Swift files
		echo -emit-objc-header-path "!TARGET_BUILD_DIR!\headers-project\!NAME!\!NAME!-Swift.h"     >> "!TARGET_BUILD_DIR!\emitheader.compflags"
		rem // give the name of the module we're creating
		echo -module-name "!NAME!"                                                                 >> "!TARGET_BUILD_DIR!\emitheader.compflags"
		rem // number of threads to use
		echo -num-threads !CORES!                                                                  >> "!TARGET_BUILD_DIR!\emitheader.compflags"

		rem // call the Swift compiler and make sure it finishes successfully
rem // WIP		if not "_!DEBUG_ARG!_"=="__" echo Calling: swiftc -frontend -target arm64-apple-ios!MINOSVER! -sdk "!SDK_PATH!" -target-sdk-version "!SDK_VERSION!" -module-cache-path "!SDK_PATH!\.llvm-modules" -c @"!TARGET_BUILD_DIR!\emitheader.compflags"
rem // WIP		"!TOOLCHAIN_PATH!\swiftc.exe"                     -frontend -target arm64-apple-ios!MINOSVER! -sdk "!SDK_PATH!" -target-sdk-version "!SDK_VERSION!" -module-cache-path "!SDK_PATH!\.llvm-modules" -c @"!TARGET_BUILD_DIR!\emitheader.compflags"
rem // WIP		if not "!errorlevel!"=="0" goto :exit_error
	)
)

rem // NON-SWIFT SOURCE FILES
rem // for each source file...
for /f "usebackq delims=" %%f in ("!TARGET_BUILD_DIR!\srclist.txt") do (

	rem // identify source file and whether it's an external reference (XRef)
	set SRCFILE_VERBATIM=%%f
	set SRCFILE=!SRCFILE_VERBATIM:/=\!
	if "_!SRCFILE:~0,1!_"=="_\_" (
		set IS_XREF=yes
	) else if "_!SRCFILE:~0,2!_"=="_.._" (
		set IS_XREF=yes
	) else if "_!SRCFILE:~1,1!_"=="_:_" (
		set IS_XREF=yes
	) else (
		set IS_XREF=no
	)

	rem // determine source file type (defaults to C) and remember Objective-C when we encounter some
	if "!SRCFILE:~-2!"==".c" (
		set SRCTYPE=C
	) else if "!SRCFILE:~-2!"==".m" (
		set SRCTYPE=C& set OBJC_LDFLAGS=-lobjc
	) else if "!SRCFILE:~-3!"==".cc" (
		set SRCTYPE=C++
	) else if "!SRCFILE:~-3!"==".mm" (
		set SRCTYPE=C++& set OBJC_LDFLAGS=-lobjc
	) else if "!SRCFILE:~-3!"==".mx" (
		set SRCTYPE=C++& set OBJC_LDFLAGS=-lobjc
	) else if "!SRCFILE:~-4!"==".cpp" (
		set SRCTYPE=C++
	) else if "!SRCFILE:~-4!"==".cxx" (
		set SRCTYPE=C++
	) else if "!SRCFILE:~-4!"==".mxx" (
		set SRCTYPE=C++& set OBJC_LDFLAGS=-lobjc
	) else if "!SRCFILE:~-6!"==".swift" (
		set SRCTYPE=SWIFT& set OBJC_LDFLAGS=-lobjc
	) else if "!SRCFILE:~-2!"==".s" (
		set SRCTYPE=ASM
	) else if "!SRCFILE:~-2!"==".a" (
		set SRCTYPE=LIB
	) else (
		set SRCTYPE=C
	)

	rem // for each arch for which to build this file...
	for %%a in (!ARCHS!) do (
		rem // acquire the current arch for which to build
		set ARCH=%%a

		rem // if another file produced a compiler error, jump out of the build loop
		if exist "!TARGET_BUILD_DIR!\!ARCH!\compilation-failed.txt" (
			echo detected compiler error, aborting multithreaded build.
			set BUILD_FAILED=true
			goto :endofloop
		)

		rem // work out object file path (put XRefs in a separate directory)
		if "!IS_XREF!"=="no" set OBJFILE=!TARGET_BUILD_DIR!\!ARCH!\!SRCFILE!.obj
		if "!IS_XREF!"=="yes" set OBJFILE=!TARGET_BUILD_DIR!\!ARCH!\__xrefs\%%~nxf.obj
		set OBJFILE=!OBJFILE:/=\!

		rem // create the directory where the object file will be written
		rem // hint: to create a path[!name], we create pathname recursively, then delete it (only the last child is deleted)
		mkdir "!OBJFILE!.tmp" > nul 2>&1
		rmdir /q "!OBJFILE!.tmp" > nul 2>&1
		if not "!errorlevel!"=="0" (
			echo error: unable to create object file "!OBJFILE!"'s path
			set BUILD_FAILED=true
			goto :endofloop
		)

		rem //////////////////////////
		rem // now compile source file

		if "!SRCTYPE!"=="LIB" (
			echo    + [!ARCH!] Including !SRCFILE!...
			rem // don't just feed the static library as an input file like I used to do, use the appropriate linker option for that (essentially to avoid duplicates).
			rem // Since many user-produced libraries don't respect the "libXXXX.extension" POSIX convention, we need to link them using their full filename. Whereas GNU ld has the -l:filename flag,
			rem // for Apple's ld64 this filename MUST be ending in .o for the linker to not add "libXXXX.a" when using the -l flag. This is kind of stupid but that's the way they want it.
			rem // So just create a symlink that will point to the actual library file with the name ending in .o, and feed that symlink to the linker.
			rem // create the directory where the static library symlink will be written if it doesn't exist yet, then drop a symlink in it
			set SRCFILE_BASENAME=%%~nf
			if "_!SRCFILE_BASENAME:~0,3!_"=="_lib_" (
				rem // this library conforms to the "libXXXX.extension" POSIX convention, tell the linker to link against that library directly
				set SRCFILE_PATH=%%~dpf
				set SRCFILE_PATH=!SRCFILE_PATH:\=/!
				set SRCFILE_PATH=!SRCFILE_PATH:~0,-1!
				echo -L"!SRCFILE_PATH!" -l"!SRCFILE_BASENAME:~3!" >> "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags"
			) else (
				rem // this library is not conformant to the "libXXXX.extension" POSIX convention, create a symlink to it with the .o extension
				set SRCFILE_NAME=%%~nxf
				mkdir "!TARGET_BUILD_DIR!\!ARCH!\__libs" > nul 2>&1
				if        "_!SRCFILE:~0,1!_"=="_\_" ( set IS_ABSOLUTE=true
				) else if "_!SRCFILE:~0,1!_"=="_/_" ( set IS_ABSOLUTE=true
				) else if "_!SRCFILE:~1,1!_"=="_:_" ( set IS_ABSOLUTE=true
				) else                                set IS_ABSOLUTE=false
				if "_!IS_ABSOLUTE!_"=="_true_" (
					rem // absolute path with a leading backslash, a leading slash or a drive letter
					echo|set /p="!SRCFILE:\=/!" > "!TARGET_BUILD_DIR!\!ARCH!\__libs\lib!SRCFILE_NAME!" 2>nul
				) else (
					rem // relative path: prepend our directory path so as to make it absolute (relative paths break when moved to a different directory)
					echo|set /p="%CD:\=/%/!SRCFILE:\=/!" > "!TARGET_BUILD_DIR!\!ARCH!\__libs\lib!SRCFILE_NAME!" 2>nul
				)
				set SRCFILE_NAME=
				set IS_ABSOLUTE=
				rem // then tell the linker to link against that library
				echo -L"!TARGET_BUILD_DIR!/!ARCH!/__libs" -l"!SRCFILE_BASENAME!" >> "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags"
			)
			rem // FIXME: the following doesn't work. Either creates duplicated classes, or fails to correctly expose all the necessary methods, giving runtime crashes. ATM -ObjC is unreplaceable.
			rem // does this static library contain Objective-C code ? if so, add the -force_load XXX flag to the linker. This will solve a lot of problems caused by users not setting
			rem // the -ObjC flag appropriately at the expense of a possibly bigger binary if none of these Objective-C functions from the static library are actually called from the app.
			rem //"!TOOLCHAIN_PATH!\nm.exe" -gpUj "!SRCFILE!"|findstr "_OBJC_" >nul && echo -force_load "!TARGET_BUILD_DIR!\!ARCH!\__libs\!SRCFILE_BASENAME!.o">> "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags"
		) else (
			rem // build the compiler flags file to work around the dreaded command-line size limit
			copy /y nul "!OBJFILE!.compflags" > nul
			if "!SRCTYPE!"=="C" (
				rem // C/Objective-C source file
				echo -U________ClangFlags__ExtraFrameworks_CPPFLAGS____________________>> "!OBJFILE!.compflags"
				rem // NOTE: forward slashes are MANDATORY!
				echo -F "!BUILD_DIR:\=/!/!CONFIGURATION!-!SDK_PLATFORM_NAME!/SuppliedFrameworks"      >> "!OBJFILE!.compflags"
				echo -F "!BUILD_DIR:\=/!/!CONFIGURATION!-!SDK_PLATFORM_NAME!/BuiltFrameworks"         >> "!OBJFILE!.compflags"
				echo -F "!BUILD_DIR:\=/!/!CONFIGURATION!-!SDK_PLATFORM_NAME!/XCFrameworkIntermediates">> "!OBJFILE!.compflags"
				echo -U________ClangFlags__TargetAgnostic_CPPFLAGS_____________________>> "!OBJFILE!.compflags"
				type "!TARGET_BUILD_DIR!\target-agnostic.compflags"                    >> "!OBJFILE!.compflags"
				echo -U________ClangFlags__TargetSpecific_CPPFLAGS_____________________>> "!OBJFILE!.compflags"
				if not "_!CPPFLAGS!_"=="__" echo !CPPFLAGS!>>                             "!OBJFILE!.compflags"
				echo -U________ClangFlags__LanguageSpecific_CFLAGS_____________________>> "!OBJFILE!.compflags"
				if not "_!CFLAGS!_"=="__" echo !CFLAGS!>>                                 "!OBJFILE!.compflags"
				rem // HACK: sometimes the pthread_impl.h definitions are missed out
				if "_!SDK_PLATFORM_NAME!_"=="_iphoneos_" (
					echo -U________ClangFlags__Hotfix___POSIX_LIB__________________________>> "!OBJFILE!.compflags"
					echo -D_PTHREAD_MUTEX_SIG_init=0x32AAABA7 1>>                             "!OBJFILE!.compflags"
					echo -D_PTHREAD_ERRORCHECK_MUTEX_SIG_init=0x32AAABA1 1>>                  "!OBJFILE!.compflags"
					echo -D_PTHREAD_RECURSIVE_MUTEX_SIG_init=0x32AAABA2 1>>                   "!OBJFILE!.compflags"
					echo -D_PTHREAD_FIRSTFIT_MUTEX_SIG_init=0x32AAABA3 1>>                    "!OBJFILE!.compflags"
					echo -D_PTHREAD_COND_SIG_init=0x3CB0B1BB 1>>                              "!OBJFILE!.compflags"
					echo -D_PTHREAD_ONCE_SIG_init=0x30B1BCBA 1>>                              "!OBJFILE!.compflags"
					echo -D_PTHREAD_RWLOCK_SIG_init=0x2DA8B3B4 1>>                            "!OBJFILE!.compflags"
					echo -DSCHED_OTHER=1 1>>                                                  "!OBJFILE!.compflags"
					echo -DSCHED_FIFO=4 1>>                                                   "!OBJFILE!.compflags"
					echo -DSCHED_RR=2 1>>                                                     "!OBJFILE!.compflags"
					echo -D__SCHED_PARAM_SIZE__=4 1>>                                         "!OBJFILE!.compflags"
				)
				echo -U________ClangFlags__FileSpecific__COMPILER_FLAGS________________>> "!OBJFILE!.compflags"
				if exist "!XCNAME!.xcodeproj\project.pbxproj" "!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -queryfile "!SRCFILE!" >> "!OBJFILE!.compflags"
				type "!TARGET_BUILD_DIR!\incpaths.compflags"                           >> "!OBJFILE!.compflags"
			) else if "!SRCTYPE!"=="C++" (
				rem // C++/Objective-C++ source file
				echo -U________ClangFlags__ExtraFrameworks_CPPFLAGS____________________>> "!OBJFILE!.compflags"
				rem // NOTE: forward slashes are MANDATORY!
				echo -F "!BUILD_DIR:\=/!/!CONFIGURATION!-!SDK_PLATFORM_NAME!/SuppliedFrameworks"      >> "!OBJFILE!.compflags"
				echo -F "!BUILD_DIR:\=/!/!CONFIGURATION!-!SDK_PLATFORM_NAME!/BuiltFrameworks"         >> "!OBJFILE!.compflags"
				echo -F "!BUILD_DIR:\=/!/!CONFIGURATION!-!SDK_PLATFORM_NAME!/XCFrameworkIntermediates">> "!OBJFILE!.compflags"
				echo -U________ClangFlags__TargetAgnostic_CPPFLAGS_____________________>> "!OBJFILE!.compflags"
				type "!TARGET_BUILD_DIR!\target-agnostic.compflags"                    >> "!OBJFILE!.compflags"
				echo -U________ClangFlags__TargetSpecific_CPPFLAGS_____________________>> "!OBJFILE!.compflags"
				if not "_!CPPFLAGS!_"=="__" echo !CPPFLAGS!>>                             "!OBJFILE!.compflags"
				echo -U________ClangFlags__LanguageSpecific_CXXFLAGS___________________>> "!OBJFILE!.compflags"
				if not "_!CXXFLAGS!_"=="__" echo !CXXFLAGS!>>                             "!OBJFILE!.compflags"
				rem // HACK: sometimes the pthread_impl.h definitions are missed out in some iOS SDKs
				if "_!SDK_PLATFORM_NAME!_"=="_iphoneos_" (
					echo -U________ClangFlags__Hotfix___POSIX_LIB__________________________>> "!OBJFILE!.compflags"
					echo -D_PTHREAD_MUTEX_SIG_init=0x32AAABA7 1>>                             "!OBJFILE!.compflags"
					echo -D_PTHREAD_ERRORCHECK_MUTEX_SIG_init=0x32AAABA1 1>>                  "!OBJFILE!.compflags"
					echo -D_PTHREAD_RECURSIVE_MUTEX_SIG_init=0x32AAABA2 1>>                   "!OBJFILE!.compflags"
					echo -D_PTHREAD_FIRSTFIT_MUTEX_SIG_init=0x32AAABA3 1>>                    "!OBJFILE!.compflags"
					echo -D_PTHREAD_COND_SIG_init=0x3CB0B1BB 1>>                              "!OBJFILE!.compflags"
					echo -D_PTHREAD_ONCE_SIG_init=0x30B1BCBA 1>>                              "!OBJFILE!.compflags"
					echo -D_PTHREAD_RWLOCK_SIG_init=0x2DA8B3B4 1>>                            "!OBJFILE!.compflags"
					echo -DSCHED_OTHER=1 1>>                                                  "!OBJFILE!.compflags"
					echo -DSCHED_FIFO=4 1>>                                                   "!OBJFILE!.compflags"
					echo -DSCHED_RR=2 1>>                                                     "!OBJFILE!.compflags"
					echo -D__SCHED_PARAM_SIZE__=4 1>>                                         "!OBJFILE!.compflags"
				)
				echo -U________ClangFlags__FileSpecific__COMPILER_FLAGS________________>> "!OBJFILE!.compflags"
				if exist "!XCNAME!.xcodeproj\project.pbxproj" "!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -queryfile "!SRCFILE!" >> "!OBJFILE!.compflags"
				rem // HACK: UNITY BUG: source files calling sched_yield *require* inclusion of pthread/sched.h !
				if not "_!SRCFILE:Classes/Unity/UnityWebRequest.mm=!_"=="_!SRCFILE!_" echo -include pthread/sched.h >> "!OBJFILE!.compflags"
				type "!TARGET_BUILD_DIR!\incpaths.compflags"                           >> "!OBJFILE!.compflags"
			) else if "!SRCTYPE!"=="SWIFT" (
				rem // Swift source file: construct the Swift file-specific compiler flags file
				set SRCFILE_ESCAPED=!SRCFILE_VERBATIM:\=\\!
				set SRCFILE_ESCAPED=!SRCFILE_ESCAPED:.=\.!
				"!TOOLCHAIN_PATH!\sed.exe" "s|^^!SRCFILE_ESCAPED!$|-primary-file !SRCFILE_ESCAPED!|" "!TARGET_BUILD_DIR!\swift-srclist.txt" > "!OBJFILE!.compflags"
				rem // if we're on any arm64 arch, ignore the top byte of 64-bit addresses for memory savings (and so as to enable signed pointers)
				if "_!ARCH:~0,5!_"=="_arm64_" echo -Xllvm -aarch64-use-tbi                        >> "!OBJFILE!.compflags"
				rem // enable ObjC interoperability, bringing in some knowledge of the Darwin ObjC runtime
				echo -enable-objc-interop                                                         >> "!OBJFILE!.compflags"
				rem // enable memory safety
				echo -enforce-exclusivity=checked                                                 >> "!OBJFILE!.compflags"
				rem // have color diagnostics just like Clang does
				echo -color-diagnostics                                                           >> "!OBJFILE!.compflags"
				rem // enable "let pattern = /.../" literals in Swift source code to be treated as regular expressions (available in Swift-5.8+)
				echo -enable-bare-slash-regex                                                     >> "!OBJFILE!.compflags"
				rem // pass the location where the Clang headers and layout files will be
				echo -resource-dir "!SDK_PATH!/../lib/swift"                                      >> "!OBJFILE!.compflags"
				rem // pass the location of the Swift modules in the SDK
				echo -I "!SDK_PATH!/usr/lib/swift"                                                >> "!OBJFILE!.compflags"
				rem // pass the location of our 3rd party frameworks (NOTE: FORWARD SLASHES ARE MANDATORY!)
				echo -F "!BUILD_DIR:\=/!/!CONFIGURATION!-!SDK_PLATFORM_NAME!/SuppliedFrameworks"      >> "!OBJFILE!.compflags"
				echo -F "!BUILD_DIR:\=/!/!CONFIGURATION!-!SDK_PLATFORM_NAME!/BuiltFrameworks"         >> "!OBJFILE!.compflags"
				echo -F "!BUILD_DIR:\=/!/!CONFIGURATION!-!SDK_PLATFORM_NAME!/XCFrameworkIntermediates">> "!OBJFILE!.compflags"
				rem // don't look for an entrypoint if not building a CRT program (FIXME: what about mixed-language CRT programs where the entrypoint is in C/ObjC code?)
				if not "_!ENTRYPOINT!_"=="_crt_" echo -parse-as-library                           >> "!OBJFILE!.compflags"
				rem // create a partial module, its documentation and source info for this Swift file
				echo -emit-module-path             "!OBJFILE!.swiftmodule"                        >> "!OBJFILE!.compflags"
				echo -emit-module-doc-path         "!OBJFILE!.swiftdoc"                           >> "!OBJFILE!.compflags"
				echo -emit-module-source-info-path "!OBJFILE!.swiftsourceinfo"                    >> "!OBJFILE!.compflags"
				rem // give the name of the module we're creating
				echo -module-name "!NAME!"                                                        >> "!OBJFILE!.compflags"
			) else if "!SRCTYPE!"=="ASM" (
				rem // assembly source file
				echo -U________ClangFlags__TargetAgnostic_CPPFLAGS_____________________>> "!OBJFILE!.compflags"
				type "!TARGET_BUILD_DIR!\target-agnostic.compflags"                    >> "!OBJFILE!.compflags"
				echo -U________ClangFlags__TargetSpecific_CPPFLAGS_____________________>> "!OBJFILE!.compflags"
				if not "_!CPPFLAGS!_"=="__" echo !CPPFLAGS!>>                             "!OBJFILE!.compflags"
				echo -U________ClangFlags__FileSpecific__COMPILER_FLAGS________________>> "!OBJFILE!.compflags"
				if exist "!XCNAME!.xcodeproj\project.pbxproj" "!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -queryfile "!SRCFILE!" >> "!OBJFILE!.compflags"
				type "!TARGET_BUILD_DIR!\incpaths.compflags"                           >> "!OBJFILE!.compflags"
			)
			rem // compare source and object mtimes and see if this file needs to be recompiled
			set COMPILE_REASON=
			if not exist "!OBJFILE!" (
				if not "_!DEBUG_ARG!_"=="__" echo    + [!ARCH!] INFO: compiling !OBJFILE!
				set COMPILE_REASON=initial
			)
			if "_!COMPILE_REASON!_"=="__" "!TOOLCHAIN_PATH!\is_newer.exe" "!SRCFILE!" "!OBJFILE!" 2>nul && (
				if not "_!DEBUG_ARG!_"=="__" echo    + [!ARCH!] INFO: recompiling !OBJFILE! because source file is newer
				set COMPILE_REASON=source changed
			)
			if "_!COMPILE_REASON!_"=="__" (
				fc "!OBJFILE!.compflags" "!OBJFILE!.compflags.bak" > nul 2>&1 || (
					if not "_!DEBUG_ARG!_"=="__" echo    + [!ARCH!] INFO: recompiling !OBJFILE! because compiler flags changed
					set COMPILE_REASON=flags changed
				)
			)
			if "_!COMPILE_REASON!_"=="__" (
				echo    + [!ARCH!] !SRCFILE!.obj is up to date
			) else (
				rem // if we're building on all cores, wait until one CPU core is free
				if not "_!WANT_MULTICORE!_"=="__" "!TOOLCHAIN_PATH!\waitfor.exe" clang:2,swiftc less_than !CORES!

				rem // compile this source file, either in the foreground or as a background task...
				if not "!SRCTYPE!"=="ASM" (
					if "_!COMPILE_REASON!_"=="_initial_" (
						echo    + [!ARCH!] Compiling !SRCFILE!...
					) else (
						echo    + [!ARCH!] Recompiling !SRCFILE! ^(!COMPILE_REASON!^)...
					)
				) else (
					if "_!COMPILE_REASON!_"=="_initial_" (
						echo    + [!ARCH!] Assembling !SRCFILE!...
					) else (
						echo    + [!ARCH!] Reassembling !SRCFILE! ^(!COMPILE_REASON!^)...
					)
				)

				rem // build the compiler command line and prepend "ccache" to it if we intend to cache the compilation results
				rem // NOTE: only use ccache for C-language compilations. Swiftc is excluded.
				if "!SRCTYPE!"=="SWIFT" (
					set COMPILER_CMDLINE="!TOOLCHAIN_PATH!\swiftc.exe" -frontend -target !ARCH!-apple-!TARGET_PLATFORM_NAME!!MINOSVER! -sdk "!SDK_PATH!" -target-sdk-version "!SDK_VERSION!" -module-cache-path "!SDK_PATH!\.llvm-modules" -o "!OBJFILE!" -c @"!OBJFILE!.compflags"
				) else if "_!WANT_CCACHE!_"=="_false_" (
					set COMPILER_CMDLINE="!TOOLCHAIN_PATH!\clang.exe" -target !ARCH!-apple-!TARGET_PLATFORM_NAME!!MINOSVER! --sysroot "!SDK_PATH!" -fmodules-cache-path="!SDK_PATH!\.llvm-modules" -o "!OBJFILE!" -c "!SRCFILE!" @"!OBJFILE!.compflags"
				) else (
					rem // if we use ccache, don't forget to set the cache directory using CCACHE_DIR
					set CCACHE_DIR=!SDK_PATH!\.ccache
					set CCACHE_COMPILERTYPE=clang
					set CCACHE_BASEDIR=%CD%
					set COMPILER_CMDLINE="!TOOLCHAIN_PATH!\ccache.exe" "!TOOLCHAIN_PATH!\clang.exe" -target !ARCH!-apple-!TARGET_PLATFORM_NAME!!MINOSVER! --sysroot "!SDK_PATH!" -fmodules-cache-path="!SDK_PATH!\.llvm-modules" -o "!OBJFILE!" -c "!SRCFILE!" @"!OBJFILE!.compflags"
				)

				if not "_!DEBUG_ARG!_"=="__" (
					rem // NOTE: in verbose mode, only append -v to clang, not swiftc when it runs in -frontend mode
					if not "!SRCTYPE!"=="SWIFT" set COMPILER_CMDLINE=!COMPILER_CMDLINE! -v
					echo -------------------------------------------------------------------------------
					echo Working directory: "!CD!"
					echo Compiler invoked with:
					echo !COMPILER_CMDLINE!
					echo.
					echo Compiler flags file [!OBJFILE!.compflags]:
					type "!OBJFILE!.compflags"
					echo -------------------------------------------------------------------------------
				)

				rem // now we can tell if among all the compiler flags, a -g (debug) was passed; if there's one, remember to create .dSYM file later
				rem // look for two cases: flag occuring at start of line, and flag in the middle of a line (after a space separator)
				rem // WARNING: findstr would take "-g" or " -g" as a FLAG PARAMETER and NOT a literal if we passed it as is! Hence the use of /c:"-g" (and find on the 2nd line)
				findstr /b /c:"-g" "!OBJFILE!.compflags" && set SHOULD_CREATE_DSYM=true
				find " -g" "!OBJFILE!.compflags" && set SHOULD_CREATE_DSYM=true

				rem // fire up a compiler then backup the compiler flags file for a future pass to be able to compare and decide whether to recompile or not
				if "_!WANT_MULTICORE!_"=="__" (
					!COMPILER_CMDLINE! || set BUILD_FAILED=true
					move /y "!OBJFILE!.compflags" "!OBJFILE!.compflags.bak" > nul 2>&1
					if "_!BUILD_FAILED!_"=="_true_" goto :endofloop
				) else (
					start /b "" "%COMSPEC%" /c "((!COMPILER_CMDLINE! || echo. > "!TARGET_BUILD_DIR!\!ARCH!\compilation-failed.txt") & move /y "!OBJFILE!.compflags" "!OBJFILE!.compflags.bak" > nul 2>&1)"
				)
				rem // and remember which file was the last built file
				set LAST_BUILT_OBJFILE=!OBJFILE!
			)
			rem // now append this object file to the list of object files
			echo !OBJFILE:\=/!>> "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt"
		)
		rem // end of "static or dynamic binary ?"
	)
	rem // end of "for each arch..."

	rem // cleanup
	set SRCFILE=
	set SRCTYPE=
	set IS_XREF=
	set OBJFILE=
)
rem // end of "for each source file..."

rem // at this point, the object files have been produced. Time to link / archive all of them together.

rem // for each arch...
for %%a in (!ARCHS!) do (
	set ARCH=%%a

	rem // did we actually compile something ? (some projects are resources-only) -- NOTE: verify that objlist.txt file size > 0 too
	if exist "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt" for %%S in ("!TARGET_BUILD_DIR!\!ARCH!\objlist.txt") do if not %%~zS lss 1 (

		rem // we compiled something; if we use several cores, wait for all compilers to finish their jobs
		if not "_!LAST_BUILT_OBJFILE!_"=="__" if not "_!WANT_MULTICORE!_"=="__" "!TOOLCHAIN_PATH!\waitfor.exe" ccache,clang:2,swiftc

		rem // are we building a static library or a binary ?
		if "_!ENTRYPOINT!_"=="_lib_" (
			rem // this is a static library, we must call the archiver
			echo    + [!ARCH!] Creating !OUTFILE! static library...

			del /f /q "!TARGET_BUILD_DIR!\!ARCH!\!OUTFILE!.!ARCH!" > nul 2>&1
			if not "_!DEBUG_ARG!_"=="__" echo Calling: ar qsv "!TARGET_BUILD_DIR!\!ARCH!\!OUTFILE!.!ARCH!" "@!TARGET_BUILD_DIR!\!ARCH!\objlist.txt"
			"!TOOLCHAIN_PATH!\ar.exe"                     qsv "!TARGET_BUILD_DIR!\!ARCH!\!OUTFILE!.!ARCH!" "@!TARGET_BUILD_DIR!\!ARCH!\objlist.txt" > "!TARGET_BUILD_DIR!\!ARCH!\!OUTFILE!.ar.log" 2>&1
			if not "!errorlevel!"=="0" (
				type "!TARGET_BUILD_DIR!\!ARCH!\!OUTFILE!.ar.log"
				del /f /q "!TARGET_BUILD_DIR!\!ARCH!\!OUTFILE!.!ARCH!" > nul 2>&1
				set BUILD_FAILED=true& goto :endofloop
			)
			if not "_!DEBUG_ARG!_"=="__" type "!TARGET_BUILD_DIR!\!ARCH!\!OUTFILE!.ar.log"

		) else (
			rem // this is not a static library, i.e. we must call the linker

			rem // entrypoint-related linker flags
			rem // also, in case we're linking for a 32-bit arch AND we have crt1, bundle1 and dylib1 in the SDK, link against them too as appropriate
			if "_!ENTRYPOINT!_"=="_crt_" (
				rem // when building an app, add a suitable rpath for dynamic libraries and frameworks - see https://developer.apple.com/documentation/bundleresources/placing_content_in_a_bundle
				set ENTRYPOINT_LDFLAGS=-rpath "@executable_path/Frameworks"
				if not "_!ARCH:~0,5!_"=="_arm64_" if exist "!SDK_PATH!\usr\lib\crt1.o"    set ENTRYPOINT_LDFLAGS=!ENTRYPOINT_LDFLAGS! -lcrt1.o
			) else if "_!ENTRYPOINT!_"=="_appex_" (
				rem // note to self: app extensions seem to be CRT binaries with just a custom entrypoint called _NSExtensionMain instead of _main
				rem // when building an app extension, add a suitable rpath for dynamic libraries and frameworks - see https://developer.apple.com/documentation/bundleresources/placing_content_in_a_bundle
				set ENTRYPOINT_LDFLAGS=-e _NSExtensionMain -rpath "@executable_path/Frameworks"
				if not "_!ARCH:~0,5!_"=="_arm64_" if exist "!SDK_PATH!\usr\lib\crt1.o"    set ENTRYPOINT_LDFLAGS=!ENTRYPOINT_LDFLAGS! -lcrt1.o
			) else if "_!ENTRYPOINT!_"=="_bundle_" (
				rem // when building a bundle, add a suitable rpath for dynamic libraries and frameworks - see https://developer.apple.com/documentation/bundleresources/placing_content_in_a_bundle
				set ENTRYPOINT_LDFLAGS=-bundle -rpath "@executable_path/Frameworks"
				if not "_!ARCH:~0,5!_"=="_arm64_" if exist "!SDK_PATH!\usr\lib\bundle1.o" set ENTRYPOINT_LDFLAGS=!ENTRYPOINT_LDFLAGS! -lbundle1.o
			) else if "_!ENTRYPOINT!_"=="_dylib_" (
				rem // when building a dylib, set its install_name and add a suitable rpath for dynamic libraries and frameworks - see https://developer.apple.com/documentation/bundleresources/placing_content_in_a_bundle
				set ENTRYPOINT_LDFLAGS=-dylib -install_name "@executable_path/Frameworks/!OUTFILE!" -rpath "@executable_path/Frameworks"
				if not "_!ARCH:~0,5!_"=="_arm64_" if exist "!SDK_PATH!\usr\lib\dylib1.o"  set ENTRYPOINT_LDFLAGS=!ENTRYPOINT_LDFLAGS! -ldylib1.o
			) else if "_!ENTRYPOINT!_"=="_framework_" (
				rem // when building a framework, set its install_name and add suitable rpaths for dynamic libraries and frameworks - see https://developer.apple.com/documentation/bundleresources/placing_content_in_a_bundle
				set ENTRYPOINT_LDFLAGS=-dylib -install_name "@executable_path/Frameworks/!NAME!.framework/!OUTFILE!" -rpath "@executable_path/Frameworks"
				if not "_!ARCH:~0,5!_"=="_arm64_" if exist "!SDK_PATH!\usr\lib\dylib1.o"  set ENTRYPOINT_LDFLAGS=!ENTRYPOINT_LDFLAGS! -ldylib1.o
			)

			rem // build the linker flags

			rem // we need at least a runtime library to link against (okay, sometimes we don't, but that's clearly out of scope of this build script)
			set LDFLAGS_DEFAULTLIBS=-L"!SDK_PATH!" -l!SDK_RUNTIME_LIB!

			rem // link against the C++ library provided by the SDK. As of 2020, the GNU libstdc++ is deprecated in favor of LLVM's libc++.
			rem // note that this library is needed for the remote debug console demangler, so don't make it optional.
			if exist "!SDK_PATH!\usr\lib\libc++.tbd"      set LDFLAGS_DEFAULTLIBS=!LDFLAGS_DEFAULTLIBS! -lc++
			if exist "!SDK_PATH!\usr\lib\libc++.dylib"    set LDFLAGS_DEFAULTLIBS=!LDFLAGS_DEFAULTLIBS! -lc++
			rem // if exist "!SDK_PATH!\usr\lib\libstdc++.tbd"   set LDFLAGS_DEFAULTLIBS=!LDFLAGS_DEFAULTLIBS! -lstdc++
			rem // if exist "!SDK_PATH!\usr\lib\libstdc++.dylib" set LDFLAGS_DEFAULTLIBS=!LDFLAGS_DEFAULTLIBS! -lstdc++

			rem // if our SDK has Swift, add the Swift directory in the linker's search path so that it can find Swift libs
			rem // FIXME: put these libraries in a more proper place during SDK migration
			if exist "!SDK_PATH!\usr\lib\swift\libswiftCore.*" (
rem				for /f "usebackq delims=" %%f in (`dir /b "!SDK_PATH!\libswiftCompatibility*.a" 2^>nul`) do (
rem					set SWIFTLIB=%%~nf
rem					set LDFLAGS_DEFAULTLIBS=!LDFLAGS_DEFAULTLIBS! -l!SWIFTLIB:~3!
rem				)
				set LDFLAGS_DEFAULTLIBS=!LDFLAGS_DEFAULTLIBS! -L/usr/lib/swift
rem	DEPRECATE		rem // in the Swift 5.0 compatibility library Apple seemingly forgot to include some symbols. They're present in 5.1
rem				if not exist "!SDK_PATH!\libswiftCompatibility51.a" set LDFLAGS_DEFAULTLIBS=!LDFLAGS_DEFAULTLIBS! -L"!SDK_PATH!/../Toolchain/extra" -lswiftCompatibilityMissingSymbols
			)

			rem // Also note that -lz -lsqlite3 seems to be a default on Xcode (FIXME: wrong ?? -- 2018-10-22: after feedback from A. Vorobets and some research, let's assume it was wrong.)
			rem set LDFLAGS_DEFAULTLIBS=!LDFLAGS_DEFAULTLIBS! -lz -lsqlite3
			echo -demangle -dead_strip !OBJC_LDFLAGS! -lSystem !ENTRYPOINT_LDFLAGS! !LDFLAGS_DEFAULTLIBS! >> "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags"
			if exist "!TOOLCHAIN_PATH!\..\lib\libomp.a" echo "!TOOLCHAIN_PATH!\..\lib\libomp.a" >> "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags"

			rem // add the standard frameworks path to the linker search paths
			echo -F"!SDK_PATH!\System\Library\Frameworks" >> "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags"
			if exist "!SDK_PATH!\System\Library\PrivateFrameworks\*" echo -F"!SDK_PATH!\System\Library\PrivateFrameworks">> "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags"

			rem // compare last built object and output file mtimes and see if this output file needs to be re-created
			set LINK_REASON=
			if not exist "!TARGET_BUILD_DIR!\!ARCH!\!OUTFILE!.!ARCH!" (
				if not "_!DEBUG_ARG!_"=="__" echo    + [!ARCH!] INFO: linking !OUTFILE!
				set LINK_REASON=initial
			)
			if "_!LINK_REASON!_"=="__" if not "_!LAST_BUILT_OBJFILE!_"=="__" (
				"!TOOLCHAIN_PATH!\is_newer.exe" "!LAST_BUILT_OBJFILE!" "!TARGET_BUILD_DIR!\!ARCH!\!OUTFILE!.!ARCH!" 2>nul && (
					if not "_!DEBUG_ARG!_"=="__" echo    + [!ARCH!] INFO: relinking !OUTFILE! because last object built is newer
					set LINK_REASON=input files were rebuilt
				)
			)
			rem // output file is newer than last build file, did the linker flags change ? compare target-specific.ldflags and target-agnostic.ldflags with their .bak copies
			if "_!LINK_REASON!_"=="__" (
				fc "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt" "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt.bak" > nul 2>&1 || (
					if not "_!DEBUG_ARG!_"=="__" echo    + [!ARCH!] INFO: relinking !OUTFILE! because objects list changed
					set LINK_REASON=input files list changed
				)
			)
			if "_!LINK_REASON!_"=="__" (
				fc "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags" "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags.bak" > nul 2>&1 || (
					if not "_!DEBUG_ARG!_"=="__" echo    + [!ARCH!] INFO: relinking !OUTFILE! because target-specific linker flags changed
					set LINK_REASON=target-specific flags changed
				)
			)
			if "_!LINK_REASON!_"=="__" (
				fc "!TARGET_BUILD_DIR!\target-agnostic.ldflags" "!TARGET_BUILD_DIR!\target-agnostic.ldflags.bak" > nul 2>&1 || (
					if not "_!DEBUG_ARG!_"=="__" echo    + [!ARCH!] INFO: relinking !OUTFILE! because target-agnostic linker flags changed
					set LINK_REASON=target-agnostic flags changed
				)
			)
			if "_!LINK_REASON!_"=="__" (
				fc "!TARGET_BUILD_DIR!\frameworks.ldflags" "!TARGET_BUILD_DIR!\frameworks.ldflags.bak" > nul 2>&1 || (
					if not "_!DEBUG_ARG!_"=="__" echo    + [!ARCH!] INFO: relinking !OUTFILE! because frameworks linker flags changed
					set LINK_REASON=frameworks changed
				)
			)
			if "_!LINK_REASON!_"=="__" (
				echo    + [!ARCH!] !OUTFILE! is up to date
			) else (
				rem // link the files just compiled
				if "_!LINK_REASON!_"=="_initial_" (
					echo    + [!ARCH!] Linking !OUTFILE!...
				) else (
					echo    + [!ARCH!] Relinking !OUTFILE! ^(!LINK_REASON!^)...
				)
				if not "_!WANT_LLVMLINKER!_"=="__" (
					rem // use LLVM's linker. Note that it does NOT support backslashes! So we need to convert ALL file paths!
					set LINKER=!TOOLCHAIN_PATH!\ld64.lld.exe
					set RESPONSE_FILE_ARG=@
					set SDK_PATH=!SDK_PATH:\=/!
					move "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt"             "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt.bak"             > nul && type "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt.bak"             | "!TOOLCHAIN_PATH!\sed.exe" "s|\\|/|g" > "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt"             && del /f /q "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt.bak"             > nul
					move "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags" "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags.bak" > nul && type "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags.bak" | "!TOOLCHAIN_PATH!\sed.exe" "s|\\|/|g" > "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags" && del /f /q "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags.bak" > nul
					move "!TARGET_BUILD_DIR!\target-agnostic.ldflags"        "!TARGET_BUILD_DIR!\target-agnostic.ldflags.bak"        > nul && type "!TARGET_BUILD_DIR!\target-agnostic.ldflags.bak"        | "!TOOLCHAIN_PATH!\sed.exe" "s|\\|/|g" > "!TARGET_BUILD_DIR!\target-agnostic.ldflags"        && del /f /q "!TARGET_BUILD_DIR!\target-agnostic.ldflags.bak"        > nul
					move "!TARGET_BUILD_DIR!\frameworks.ldflags"             "!TARGET_BUILD_DIR!\frameworks.ldflags.bak"             > nul && type "!TARGET_BUILD_DIR!\frameworks.ldflags.bak"             | "!TOOLCHAIN_PATH!\sed.exe" "s|\\|/|g" > "!TARGET_BUILD_DIR!\frameworks.ldflags"             && del /f /q "!TARGET_BUILD_DIR!\frameworks.ldflags.bak"             > nul
				) else (
					rem // use Apple's linker. Note that it's the only one that supports -allow_dead_duplicates ATM.
					set LINKER=!TOOLCHAIN_PATH!\ld.exe
					set RESPONSE_FILE_ARG=-optlist 
					rem // note the extra space --^
					echo -allow_dead_duplicates >> "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags"
				)
				if not "_!DEBUG_ARG!_"=="__" (
					echo -------------------------------------------------------------------------------
					echo Working directory: "!CD!"
					echo Linker invoked with:
					echo "!LINKER!" -arch !ARCH! -syslibroot "!SDK_PATH!" -platform_version !TARGET_PLATFORM_NAME! !MINOSVER_MAJOR!.!MINOSVER_MINOR!.!MINOSVER_REVIS! !SDK_VERSION! -filelist "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt" !RESPONSE_FILE_ARG!"!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags" !RESPONSE_FILE_ARG!"!TARGET_BUILD_DIR!\target-agnostic.ldflags" !RESPONSE_FILE_ARG!"!TARGET_BUILD_DIR!\frameworks.ldflags" -o "!TARGET_BUILD_DIR!\!ARCH!\!OUTFILE!.!ARCH!"
					echo.
					echo Link list file [!TARGET_BUILD_DIR!\!ARCH!\objlist.txt]:
					type "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt"
					echo.
					echo Linker options file [!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags]:
					type "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags"
					echo.
					echo Linker options file [!TARGET_BUILD_DIR!\target-agnostic.ldflags]:
					type "!TARGET_BUILD_DIR!\target-agnostic.ldflags"
					echo.
					echo Linker options file [!TARGET_BUILD_DIR!\frameworks.ldflags]:
					type "!TARGET_BUILD_DIR!\frameworks.ldflags"
					echo -------------------------------------------------------------------------------
				)
				"!LINKER!" -arch !ARCH! -syslibroot "!SDK_PATH!" -platform_version !TARGET_PLATFORM_NAME! !MINOSVER_MAJOR!.!MINOSVER_MINOR!.!MINOSVER_REVIS! !SDK_VERSION! -filelist "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt" !RESPONSE_FILE_ARG!"!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags" !RESPONSE_FILE_ARG!"!TARGET_BUILD_DIR!\target-agnostic.ldflags" !RESPONSE_FILE_ARG!"!TARGET_BUILD_DIR!\frameworks.ldflags" -o "!TARGET_BUILD_DIR!\!ARCH!\!OUTFILE!.!ARCH!" > "!TARGET_BUILD_DIR!\!ARCH!\!OUTFILE!.ld.log" 2>&1 && (
					rem // linking succeeded at first try, but we may want to see the linker messages anyway
					type "!TARGET_BUILD_DIR!\!ARCH!\!OUTFILE!.ld.log"
				) || (
					rem // linking failed, do we want to auto-resolve symbols ?
					if not "_!WANT_AUTORESOLVE!_"=="__" (
						rem // a linking error occured and auto-fix was requested. The linker should have hinted us what the missing flags were, so just read that
						type "!TARGET_BUILD_DIR!\!ARCH!\!OUTFILE!.ld.log"|findstr /c:"suggested extra flags:" > %TO_TEMPVAR%
						if "_!TEMPVAR!_"=="__" (
							rem // a linker error occured, and it was NOT a missing Objective-C class (or no extra flags were suggested). Dump the linker output and bail out.
							type "!TARGET_BUILD_DIR!\!ARCH!\!OUTFILE!.ld.log"
							set BUILD_FAILED=true& goto :endofloop
						) else (
							rem // we've identified some missing symbols. Try linking again with the suggested extra flags.
							set SUGGESTED_FLAGS=!TEMPVAR:~27!
							echo    + [!ARCH!] Relinking !OUTFILE! with extra flags: !SUGGESTED_FLAGS!
							if not "_!DEBUG_ARG!_"=="__" (
								echo -------------------------------------------------------------------------------
								echo Working directory: "!CD!"
								echo Linker invoked with:
								echo "!LINKER!" -arch !ARCH! -syslibroot "!SDK_PATH!" -platform_version !TARGET_PLATFORM_NAME! !MINOSVER_MAJOR!.!MINOSVER_MINOR!.!MINOSVER_REVIS! !SDK_VERSION! -filelist "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt" !RESPONSE_FILE_ARG!"!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags" !RESPONSE_FILE_ARG!"!TARGET_BUILD_DIR!\target-agnostic.ldflags" !RESPONSE_FILE_ARG!"!TARGET_BUILD_DIR!\frameworks.ldflags" !SUGGESTED_FLAGS! -o "!TARGET_BUILD_DIR!\!ARCH!\!OUTFILE!.!ARCH!"
								echo.
								echo Link list file [!TARGET_BUILD_DIR!\!ARCH!\objlist.txt]:
								type "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt"
								echo.
								echo Linker options file [!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags]:
								type "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags"
								echo.
								echo Linker options file [!TARGET_BUILD_DIR!\target-agnostic.ldflags]:
								type "!TARGET_BUILD_DIR!\target-agnostic.ldflags"
								echo.
								echo Linker options file [!TARGET_BUILD_DIR!\frameworks.ldflags]:
								type "!TARGET_BUILD_DIR!\frameworks.ldflags"
								echo -------------------------------------------------------------------------------
							)
							"!LINKER!" -arch !ARCH! -syslibroot "!SDK_PATH!" -platform_version !TARGET_PLATFORM_NAME! !MINOSVER_MAJOR!.!MINOSVER_MINOR!.!MINOSVER_REVIS! !SDK_VERSION! -filelist "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt" !RESPONSE_FILE_ARG!"!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags" !RESPONSE_FILE_ARG!"!TARGET_BUILD_DIR!\target-agnostic.ldflags" !RESPONSE_FILE_ARG!"!TARGET_BUILD_DIR!\frameworks.ldflags" !SUGGESTED_FLAGS! -o "!TARGET_BUILD_DIR!\!ARCH!\!OUTFILE!.!ARCH!"
							if not "!errorlevel!"=="0" set BUILD_FAILED=true& goto :endofloop
						)
					) else (
						rem // a linker error occured, but no auto-fix was requested. Dump the linker output and bail out.
						type "!TARGET_BUILD_DIR!\!ARCH!\!OUTFILE!.ld.log"
						set BUILD_FAILED=true& goto :endofloop
					)
				)

				rem // backup the linker flags files for a future pass to be able to compare and decide whether to relink or not
				move /y "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt"             "!TARGET_BUILD_DIR!\!ARCH!\objlist.txt.bak"             > nul 2>&1
				move /y "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags" "!TARGET_BUILD_DIR!\!ARCH!\target-specific.ldflags.bak" > nul 2>&1
				rem // these ones are shared between all archs, so they must NOT be moved, but copied instead
				copy /y /b "!TARGET_BUILD_DIR!\target-agnostic.ldflags"     "!TARGET_BUILD_DIR!\target-agnostic.ldflags.bak"        > nul 2>&1
				copy /y /b "!TARGET_BUILD_DIR!\frameworks.ldflags"          "!TARGET_BUILD_DIR!\frameworks.ldflags.bak"             > nul 2>&1
			)
		)

		rem // once this arch has been processed, remember that'll be one more to embed in the final fat file
		set THINFILES_INPUT=!THINFILES_INPUT! -arch !ARCH! "!TARGET_BUILD_DIR!\!ARCH!\!OUTFILE!.!ARCH!"

		rem // end of "something was compiled"
	)

	rem // end of "for each arch"
)

rem // if the build ended up problematically, jump to the error handler
:endofloop
rem // if we compiled anything AND we use several cores, wait for all compilers to finish their jobs (must call this again in case we were thrown here)
if not "_!LAST_BUILT_OBJFILE!_"=="__" if not "_!WANT_MULTICORE!_"=="__" "!TOOLCHAIN_PATH!\waitfor.exe" ccache,clang:2,swiftc
if "_!BUILD_FAILED!_"=="_true_" goto :exit_error

rem // did we actually link something ? (some projects are resources-only)
if not "_!THINFILES_INPUT!_"=="__" (

	rem // create the universal binary gathering all arches
	set OUTTYPE=binary
	if "_!ENTRYPOINT!_"=="_lib_"       set OUTTYPE=library
	if "_!ENTRYPOINT!_"=="_dylib_"     set OUTTYPE=library
	if "_!ENTRYPOINT!_"=="_framework_" set OUTTYPE=framework
	echo  + Creating universal !OUTTYPE! ^(!ARCHS: =+!^)...
	set OUTTYPE=
	mkdir "!OUTDIR!" > nul 2>&1
	del /f /q "!OUTDIR!\!OUTFILE!" > nul 2>&1
	del /f /q "!OUTDIR!\!OUTFILE!.lipo" > nul 2>&1
	if not "_!DEBUG_ARG!_"=="__" echo Calling: lipo !THINFILES_INPUT! -output "!OUTDIR!\!OUTFILE!" -create
	"!TOOLCHAIN_PATH!\lipo.exe"                     !THINFILES_INPUT! -output "!OUTDIR!\!OUTFILE!" -create
	if not "!errorlevel!"=="0" goto :exit_error

	rem // back up the debug symbols into a .dSYM database if we should
	if "_!SHOULD_CREATE_DSYM!_"=="_true_" (
		echo  + Creating !OUTFILE!.dSYM debug symbols database...
		if not "_!DEBUG_ARG!_"=="__" echo Calling: dsymutil "!OUTDIR!\!OUTFILE!" -o "!OUTDIR!.dSYM"
		"!TOOLCHAIN_PATH!\dsymutil.exe"                     "!OUTDIR!\!OUTFILE!" -o "!OUTDIR!.dSYM"
	)

	rem // strip output binary only if requested AND remote logging is disabled (remote logging needs debug symbols)
	if not "_!ENTRYPOINT!_"=="_lib_" if "_!WANT_STRIP!_"=="_true_" (
		rem // first pass: normal symbols stripping. Remove all local (unexported) symbols (-x), leaving only global (exported) symbols
		if "_!REMOTELOG_IP!_"=="__" (
			if not "_!DEBUG_ARG!_"=="__" echo Calling: strip -x "!OUTDIR!\!OUTFILE!"
			"!TOOLCHAIN_PATH!\strip.exe"                     -x "!OUTDIR!\!OUTFILE!"
			rem // DO NOT test for errors after strip, as the binary may contain no strippable symbols.
		)
		rem // second pass: bitcode stripping
		if "_!WANT_STRIP_BITCODE!_"=="_true_" (
			echo  + Stripping bitcode to reduce binary size...
			if not "_!DEBUG_ARG!_"=="__" echo Calling: bitcode_strip -r "!OUTDIR!\!OUTFILE!" -o "!OUTDIR!\!OUTFILE!"
			"!TOOLCHAIN_PATH!\bitcode_strip.exe"                     -r "!OUTDIR!\!OUTFILE!" -o "!OUTDIR!\!OUTFILE!"
			if not "!errorlevel!"=="0" goto :exit_error
		)
	)

) else echo Nothing to link here, skipping.& goto :exit_success


rem // cleanup
set XCDIR=
set CPPFLAGS=
set CFLAGS=
set CXXFLAGS=
set CXX_DIRFLAG=
set LDFLAGS=
set LDFLAGS_DEFAULTLIBS=
set OBJC_LDFLAGS=
set PCH=


rem ////////////////////////////
rem // STEP 3. COPY RESOURCES //
rem ////////////////////////////


rem // does this project have an Info.plist ? if so, gather info from it before creating the final one
if exist "!INFO_PLIST!" (
	rem // retrieve a few variables from the Info.plist as needed
	if "_!BUNDLE_IDENTIFIER!_"=="__" (
		rem // if not known already, read CFBundleIdentifier from Info.plist (through temp file) into a variable and strip quotes
		"!TOOLCHAIN_PATH!\plconvert.exe" "!INFO_PLIST!" /query CFBundleIdentifier > %TO_TEMPVAR%
		if not "_!TEMPVAR!_"=="__" set BUNDLE_IDENTIFIER=!TEMPVAR:"=!

		rem // if the bundle name was not found in the Info.plist, defaults to project name
		if "_!BUNDLE_IDENTIFIER!_"=="__" (
			set BUNDLE_IDENTIFIER=com.pmbaty.iosbuildenv.!NAME!
			echo WARNING: CFBundleIdentifier not found in Info.plist, defaulting to !BUNDLE_IDENTIFIER!.
		)
		rem // does the bundle identifier contain variables AND do we have a Xcode project file? if so, attempt to resolve them
		echo _!BUNDLE_IDENTIFIER!_|findstr "\$" 1>nul && if exist "!XCNAME!.xcodeproj\project.pbxproj" (
			echo !BUNDLE_IDENTIFIER!> "!TEMPDIR!\resolve.tmp"
			"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -expandvars "!TEMPDIR!\resolve.tmp" > %TO_TEMPVAR%
			set BUNDLE_IDENTIFIER=!TEMPVAR!
		)
		rem // fallback if the bundle identifier still contains a variable
		echo _!BUNDLE_IDENTIFIER!_|findstr "\$" 1>nul && (
			echo WARNING: CFBundleIdentifier in Info.plist containing unresolved variables ^(!BUNDLE_IDENTIFIER!^), defaulting to com.pmbaty.iosbuildenv.!NAME!.
			set BUNDLE_IDENTIFIER=com.pmbaty.iosbuildenv.!NAME!
		)
	)
	if "_!BUNDLE_NAME!_"=="__" (
		rem // if not known already, read CFBundleName from Info.plist (through temp file) into a variable and strip quotes
		"!TOOLCHAIN_PATH!\plconvert.exe" "!INFO_PLIST!" /query CFBundleName > %TO_TEMPVAR%
		if not "_!TEMPVAR!_"=="__" set BUNDLE_NAME=!TEMPVAR:"=!

		rem // if the bundle name was not found in the Info.plist, defaults to project name
		if "_!BUNDLE_NAME!_"=="__" (
			set BUNDLE_NAME=!NAME!
			echo Notice: CFBundleName not found in Info.plist, defaulting to !BUNDLE_NAME!.
		)
		rem // does the bundle name contain a variable AND do we have a Xcode project file? if so, attempt to resolve it
		echo _!BUNDLE_NAME!_|findstr "\$" 1>nul && if exist "!XCNAME!.xcodeproj\project.pbxproj" (
			echo !BUNDLE_NAME!> "!TEMPDIR!\resolve.tmp"
			"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -expandvars "!TEMPDIR!\resolve.tmp" > %TO_TEMPVAR%
			set BUNDLE_NAME=!TEMPVAR!
		)
		rem // fallback if the bundle name still contains a variable
		echo _!BUNDLE_NAME!_|findstr "\$" 1>nul && (
			echo Notice: CFBundleName in Info.plist containing unresolved variables ^(!BUNDLE_NAME!^), defaulting to !NAME!.
			set BUNDLE_NAME=!NAME!
		)
	)
	if "_!ENTRYPOINT!_"=="_crt_" if "_!BUNDLE_DISPLAYNAME!_"=="__" (
		rem // if building an app AND if not known already, read CFBundleDisplayName from Info.plist (through temp file) into a variable and strip quotes
		"!TOOLCHAIN_PATH!\plconvert.exe" "!INFO_PLIST!" /query CFBundleDisplayName > %TO_TEMPVAR%
		if not "_!TEMPVAR!_"=="__" set BUNDLE_DISPLAYNAME=!TEMPVAR:"=!

		rem // if the bundle display name was not found in the Info.plist, defaults to project name
		if "_!BUNDLE_DISPLAYNAME!_"=="__" (
			set BUNDLE_DISPLAYNAME=!NAME!
			echo Notice: CFBundleDisplayName not found in Info.plist, defaulting to !BUNDLE_DISPLAYNAME!.
		)
		rem // handles the case where the bundle display name contains a variable
		echo _!BUNDLE_DISPLAYNAME!_|findstr "\$\{PRODUCT_NAME\}" 1>nul && set BUNDLE_DISPLAYNAME=!BUNDLE_DISPLAYNAME:${PRODUCT_NAME}=%NAME%!
		rem // fallback if the bundle display name still contains a variable
		echo _!BUNDLE_DISPLAYNAME!_|findstr "\$" 1>nul && (
			echo Notice: CFBundleDisplayName in Info.plist containing unresolved variables ^(!BUNDLE_DISPLAYNAME!^), defaulting to !NAME!.
			set BUNDLE_DISPLAYNAME=!NAME!
		)
	)
	if "_!BUILD_NUMBER!_"=="__" (
		rem // if not known already, read CFBundleVersion (build number) from Info.plist (through temp file) into a variable and strip quotes
		"!TOOLCHAIN_PATH!\plconvert.exe" "!INFO_PLIST!" /query CFBundleVersion > %TO_TEMPVAR%
		if not "_!TEMPVAR!_"=="__" set BUILD_NUMBER=!TEMPVAR:"=!
		rem // does the build number contain a variable AND do we have a Xcode project file? if so, attempt to resolve it
		echo _!BUILD_NUMBER!_|findstr "\$" 1>nul && if exist "!XCNAME!.xcodeproj\project.pbxproj" (
			echo !BUILD_NUMBER!> "!TEMPDIR!\resolve.tmp"
			"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -expandvars "!TEMPDIR!\resolve.tmp" > %TO_TEMPVAR%
			set BUILD_NUMBER=!TEMPVAR!
		)
		rem // if the build number was not found in the Info.plist, fallback to 1.0.0
		if "_!BUILD_NUMBER!_"=="__" (
			set BUILD_NUMBER=1.0.0
			echo Notice: build number not found in Info.plist at CFBundleVersion, defaulting to !BUILD_NUMBER!
		)
		rem // if the build number was specified in the Info.plist but contains a variable, fallback to 1.0.0
		echo _!BUILD_NUMBER!_|findstr "\$" 1>nul && (
			echo Notice: build number in Info.plist at CFBundleVersion containing unresolved variables ^(!BUILD_NUMBER!^), defaulting to 1.0.0.
			set BUILD_NUMBER=1.0.0
		)
	)

	rem // we *NEED* CFBundleExecutable to point to the file we've just compiled, so overwrite it if necessary
	set EXECUTABLE_NAME=!OUTFILE!

	rem // the bundle package type is FMWK for frameworks, XPC! for app extensions, BNDL for preferences bundles and APPL for apps (default)
	if        "_!ENTRYPOINT!_"=="_framework_" ( set BUNDLE_PKGTYPE=FMWK
	) else if "_!ENTRYPOINT!_"=="_appex_"     ( set BUNDLE_PKGTYPE=XPC^^^!
	) else if "_!ENTRYPOINT!_"=="_bundle_"    ( set BUNDLE_PKGTYPE=BNDL
	) else                                    ( set BUNDLE_PKGTYPE=APPL
	)

	rem // UIDeviceFamily is a required key. Construct it from the FAMILY envvar value
	set DEVICE_FAMILY={integer}1{/integer}{integer}2{/integer}
	if "_!FAMILY!_"=="_iphone_" set DEVICE_FAMILY={integer}1{/integer}
	if "_!FAMILY!_"=="_ipad_"   set DEVICE_FAMILY={integer}2{/integer}

	rem // if this project is a Xcode project, expand the variables in the Info.plist, else just copy it as is
	if exist "!XCNAME!.xcodeproj\project.pbxproj" (
		if not "_!DEBUG_ARG!_"=="__" echo Calling: pbxprojparse "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" EXECUTABLE_NAME="!EXECUTABLE_NAME!" DEVELOPMENT_LANGUAGE=English -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -expandvars "!INFO_PLIST!"
		"!TOOLCHAIN_PATH!\pbxprojparse.exe"                     "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" EXECUTABLE_NAME="!EXECUTABLE_NAME!" DEVELOPMENT_LANGUAGE=English -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -expandvars "!INFO_PLIST!" > "!OUTDIR!\Info.plist" 2> "!TEMPDIR!\unresolved-variables.txt"
		set ERRORLVL=!errorlevel!
		for /f "usebackq delims=" %%a in ("!TEMPDIR!\unresolved-variables.txt") do echo %%a | findstr /v /i "output.txt \$\{EXECUTABLE_NAME\}"
	) else (
		copy /y /b "!INFO_PLIST!" "!OUTDIR!" > nul
		set ERRORLVL=!errorlevel!
	)
	if not "_!ERRORLVL!_"=="_0_" goto :exit_error
	set ERRORLVL=
)

rem // copy all the other resources (directories, then files). Note that the for loops expand wildcards.
if not "_!RES!_"=="__" (
	set RES=!RES:/=\!
	echo  + Copying resources...
)
for /d %%r in (!RES!) do (
	set ITEM_PATHNAME=%%~r
	rem // needs a 2nd check for already-expanded pathnames. Stupid Microsoft.
	if exist "!ITEM_PATHNAME!" pushd "!ITEM_PATHNAME!" 2>nul && popd && (
		set ITEM_BASENAME=%%~nxr
		if not "_!DEBUG_ARG!_"=="__" echo    Directory: !ITEM_PATHNAME! -^> !OUTDIR!\!ITEM_BASENAME!

		rem // copy that directory in place recursively
		rem // NOTE: when copying a DIRECTORY, the copy would fail if the target directory already exists as a FILE, so make sure the target doesn't exist in any form first
		rmdir /s /q "!OUTDIR!\!ITEM_BASENAME!" > nul 2>&1
		del /f /q "!OUTDIR!\!ITEM_BASENAME!" > nul 2>&1
		robocopy "!ITEM_PATHNAME!" "!OUTDIR!\!ITEM_BASENAME!" /is /e /njh /ndl /nfl /njs > nul 2>&1
		if not "!errorlevel!"=="1" goto :exit_error
		rem // NOTE: robocopy exits with errorlevel ***1*** on success! Microsofticly absurd, but that's the way it is.

		rem // cleanup
		set ITEM_BASENAME=
	)
	rem // cleanup
	set ITEM_PATHNAME=
)
for %%r in (!RES!) do (
	set ITEM_PATHNAME=%%~r
	rem // needs a 2nd check for already-expanded pathnames. Stupid Microsoft.
	if exist "!ITEM_PATHNAME!" pushd "!ITEM_PATHNAME!" 2>nul && popd || (
		set ITEM_BASENAME=%%~nxr
		if not "_!DEBUG_ARG!_"=="__" echo    File: !ITEM_PATHNAME! -^> !OUTDIR!\!ITEM_BASENAME!
		copy /y /b "!ITEM_PATHNAME!" "!OUTDIR!" > nul
		if not "!errorlevel!"=="0" goto :exit_error
		rem // cleanup
		set ITEM_BASENAME=
	)
	rem // cleanup
	set ITEM_PATHNAME=
)

rem // are we building a Xcode project ?
if exist "!XCNAME!.xcodeproj\project.pbxproj" (
	rem // was it a framework ? if so, we should construct the Headers and Modules directories so that it can be reused in Swift
	if "_!ENTRYPOINT!_"=="_framework_" (

		rem // copy each public header where it belongs
		for /f "usebackq delims=" %%a in ("!TARGET_BUILD_DIR!\headers-public\filelist.txt") do (
			set HEADER_ABSPATHNAME=%%~fa
			set HEADER_NAME=%%~nxa
			rem // create the Headers directory if it doesn't exist yet and copy the file there
			if not exist "!OUTDIR!\Headers\." mkdir "!OUTDIR!\Headers" > nul
			copy /y /b "!HEADER_ABSPATHNAME:/=\!" "!OUTDIR!\Headers" > nul
		)

		rem // copy each private header where it belongs
		for /f "usebackq delims=" %%a in ("!TARGET_BUILD_DIR!\headers-private\filelist.txt") do (
			set HEADER_ABSPATHNAME=%%~fa
			set HEADER_NAME=%%~nxa
			rem // create the Headers directory if it doesn't exist yet and copy the file there
			if not exist "!OUTDIR!\PrivateHeaders\." mkdir "!OUTDIR!\PrivateHeaders" > nul
			copy /y /b "!HEADER_ABSPATHNAME:/=\!" "!OUTDIR!\PrivateHeaders" > nul
		)

		rem // does this framework specify a module map file ?
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query MODULEMAP_FILE -line_format "!SRCROOT!/%%s" > %TO_TEMPVAR%
		if not "_!TEMPVAR!_"=="__" set MODULEMAP_FILE=!TEMPVAR!
		if not "_!MODULEMAP_FILE!_"=="__" (
			if exist "!MODULEMAP_FILE!" (
				echo    + Constructing module from supplied module map...
				mkdir "!OUTDIR!\Modules" > nul 2>&1
				copy /y /b "!MODULEMAP_FILE:/=\!" "!OUTDIR!\Modules\module.modulemap" > nul
				rem // now parse that module map file and bring in all public headers
				rem // have some Swift files been built ? if so, we need to merge their modules together and construct a proper Swift module
				rem // NOTE: we use the generic, always-present arm64 slice for the interface (it is assumed that no armv7 device supports Swift)
				if exist "!TARGET_BUILD_DIR!\arm64\objlist.txt.bak" findstr ".swift$" "!TARGET_BUILD_DIR!\swift-srclist.txt" > nul && (
					echo    + Merging Swift module...

					rem // create a specific module merge Swift compiler command line flags file and run swiftc

					rem // the list of .swiftmodule files to parse (NOTE: can't reliably detect end of line with findstr, this Microsoft shit expects CRLFs!)
					type "!TARGET_BUILD_DIR!\arm64\objlist.txt.bak"|findstr "\.swift\.obj"|"!TOOLCHAIN_PATH!\sed.exe" "s|\.obj$|.obj.swiftmodule|g"  > "!TARGET_BUILD_DIR!\mergemodule.compflags"
					rem // enable ObjC interoperability, bringing in some knowledge of the Darwin ObjC runtime
					echo -enable-objc-interop                                                                  >> "!TARGET_BUILD_DIR!\mergemodule.compflags"
					rem // have color diagnostics just like Clang does
					echo -color-diagnostics                                                                    >> "!TARGET_BUILD_DIR!\mergemodule.compflags"
					rem // enable "let pattern = /.../" literals in Swift source code to be treated as regular expressions (available in Swift-5.8+)
					echo -enable-bare-slash-regex                                                              >> "!TARGET_BUILD_DIR!\mergemodule.compflags"
					rem // don't run diagnostic passes
					echo -disable-diagnostic-passes                                                            >> "!TARGET_BUILD_DIR!\mergemodule.compflags"
					rem // don't optimize
					echo -disable-sil-perf-optzns                                                              >> "!TARGET_BUILD_DIR!\mergemodule.compflags"
					rem // enable memory safety
					echo -enforce-exclusivity=checked                                                          >> "!TARGET_BUILD_DIR!\mergemodule.compflags"
					rem // as it says
					rem echo -warn-on-potentially-unavailable-enum-case                                            >> "!TARGET_BUILD_DIR!\mergemodule.compflags"
					rem // pass the location where the Clang headers and layout files will be
					echo -resource-dir "!SDK_PATH!/../lib/swift"                                               >> "!TARGET_BUILD_DIR!\mergemodule.compflags"
					rem // pass the location of the Swift modules in the SDK
					echo -I "!SDK_PATH!/usr/lib/swift"                                                         >> "!TARGET_BUILD_DIR!\mergemodule.compflags"
					rem // pass the location of our 3rd party frameworks
					echo -F "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks"               >> "!TARGET_BUILD_DIR!\mergemodule.compflags"
					echo -F "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks"                  >> "!TARGET_BUILD_DIR!\mergemodule.compflags"
					echo -F "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\XCFrameworkIntermediates"         >> "!TARGET_BUILD_DIR!\mergemodule.compflags"
					rem // don't look for an entrypoint if not building a CRT program (FIXME: what about mixed-language CRT programs where the entrypoint is in C/ObjC code?)
					if not "_!ENTRYPOINT!_"=="_crt_" echo -parse-as-library                                    >> "!TARGET_BUILD_DIR!\mergemodule.compflags"
					rem // create a FULL module, its documentation and source info for these Swift files
					echo -emit-module-doc-path         "!OUTDIR!\Modules\!OUTFILE!.swiftdoc"                   >> "!TARGET_BUILD_DIR!\mergemodule.compflags"
					echo -emit-module-source-info-path "!OUTDIR!\Modules\!OUTFILE!.swiftsourceinfo"            >> "!TARGET_BUILD_DIR!\mergemodule.compflags"
					echo -emit-objc-header-path        "!OUTDIR!\Headers\!OUTFILE!.h"                          >> "!TARGET_BUILD_DIR!\mergemodule.compflags"
					rem // give the name of the module we're creating
					echo -module-name "!NAME!"                                                                 >> "!TARGET_BUILD_DIR!\mergemodule.compflags"
					rem // number of threads to use
					echo -num-threads !CORES!                                                                  >> "!TARGET_BUILD_DIR!\mergemodule.compflags"

					rem // call the Swift compiler and make sure it finishes successfully
					if not "_!DEBUG_ARG!_"=="__" echo Calling: swiftc -frontend -target arm64-apple-!TARGET_PLATFORM_NAME!!MINOSVER! -sdk "!SDK_PATH!" -target-sdk-version "!SDK_VERSION!" -module-cache-path "!SDK_PATH!\.llvm-modules" -o "!OUTDIR!\Modules\!OUTFILE!.swiftmodule" -merge-modules -emit-module @"!TARGET_BUILD_DIR!\mergemodule.compflags"
					"!TOOLCHAIN_PATH!\swiftc.exe"                     -frontend -target arm64-apple-!TARGET_PLATFORM_NAME!!MINOSVER! -sdk "!SDK_PATH!" -target-sdk-version "!SDK_VERSION!" -module-cache-path "!SDK_PATH!\.llvm-modules" -o "!OUTDIR!\Modules\!OUTFILE!.swiftmodule" -merge-modules -emit-module @"!TARGET_BUILD_DIR!\mergemodule.compflags"
					if not "!errorlevel!"=="0" goto :exit_error
				)

				rem // now identify the umbrella header and make sure it contains an #import directive for the main module header
				set UMBRELLAHEADER_NAME=
				findstr "umbrella header" "!OUTDIR!\Modules\module.modulemap" > %TO_TEMPVAR%
				if not "_!TEMPVAR!_"=="__" for /f usebackq^ tokens^=2^ delims^=^" %%a in ('echo !TEMPVAR!') do (set UMBRELLAHEADER_NAME=%%a)
				rem // is there *NO* local #import "xxx" directive in this umbrella header at all ? if so, fix that by making sure all public headers are imported
				type "!OUTDIR!\Headers\!UMBRELLAHEADER_NAME!"|findstr /c:^"import \^"^" > nul || (
					for /f %%a in ('dir /b "!OUTDIR!\Headers"') do (
						set HEADER_NAME=%%~nxa
						if not "_!HEADER_NAME!_"=="_!UMBRELLAHEADER_NAME!_" echo #import "!HEADER_NAME!">> "!OUTDIR!\Headers\!UMBRELLAHEADER_NAME!"
					)
				)

			) else echo WARNING: framework module map file !MODULEMAP_FILE! not found. The build will likely fail if this framework is depended upon by other Swift code.
		)
	)

	rem // check if it has localized resources: obtain the list of localizations
	"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -localizations -line_format " %%s" > %TO_TEMPVAR%
	if not "_!TEMPVAR!_"=="__" set LOCALIZATIONS=!TEMPVAR!
	for %%l in (!LOCALIZATIONS!) do (
		echo    + Adding localization resources for locale: %%l...
		rem // create the directory for this locale
		mkdir "!OUTDIR!\%%l.lproj" > nul 2>1
		rem // get the localized resources for this locale
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -localizedresources -variant %%l > "!TEMPDIR!\localization-files.txt"
		rem // loop through all of them
		for /f "usebackq tokens=*" %%f in ("!TEMPDIR!\localization-files.txt") do (
			set ITEM_PATHNAME=%%f
			copy /y /b "!ITEM_PATHNAME:/=\!" "!OUTDIR!\%%l.lproj" > nul
			if not "!errorlevel!"=="0" goto :exit_error
		)
		del /f /q "!TEMPDIR!\localization-files.txt" > nul 2>&1
	)
	rem // cleanup
	set ITEM_PATHNAME=
	set LOCALIZATIONS=
)

rem // finally, fix the Info.plist and build the asset catalogs
if exist "!OUTDIR!\Info.plist" (
	echo  + Creating Info.plist...

	rem // auto-fix a few values in the Info.plist to ensure build consistency
	set PLCONVERT_FLAGS=+CFBundleName=string:"!BUNDLE_NAME!"^
	                    +CFBundleIdentifier=string:"!BUNDLE_IDENTIFIER!"^
	                    +CFBundleExecutable=string:"!EXECUTABLE_NAME!"^
	                    +CFBundlePackageType=string:"!BUNDLE_PKGTYPE!"^
	                    +CFBundleVersion=string:"!BUILD_NUMBER!"^
	                    +CFBundleInfoDictionaryVersion=string:6.0^
	                    +MinimumOSVersion=string:"!MINOSVER_MAJOR!.!MINOSVER_MINOR!"^
	                    +UIDeviceFamily=array:"!DEVICE_FAMILY!"
	if not "_!DEBUG_ARG!_"=="__" echo Calling: plconvert "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" !PLCONVERT_FLAGS!
	"!TOOLCHAIN_PATH!\plconvert.exe"                     "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" !PLCONVERT_FLAGS!
	if not "!errorlevel!"=="0" goto :exit_error
	rem // cleanup
	set PLCONVERT_FLAGS=
	set DEVICE_FAMILY=

	rem // CFBundleShortVersionString MAY differ from CFBundleVersion, but MUST NOT contain a variable (in which case fallback to CFBundleVersion)
	"!TOOLCHAIN_PATH!\plconvert.exe" "!OUTDIR!\Info.plist" /query CFBundleShortVersionString > %TO_TEMPVAR%
	if not "_!TEMPVAR!_"=="__" set TEMPVAR=!TEMPVAR:"=!
	echo _!TEMPVAR!_|findstr "\$" 1>nul
	if "!errorlevel!"=="0" (
		rem // FIXME: collect these sort of important warnings and display them to the user after the build
		echo WARNING: CFBundleShortVersionString in Info.plist containing unresolved variables ^(!TEMPVAR!^), defaulting to build number !BUILD_NUMBER!.
		"!TOOLCHAIN_PATH!\plconvert.exe" "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" +CFBundleShortVersionString=string:"!BUILD_NUMBER!"
		if not "!errorlevel!"=="0" goto :exit_error
	)

	rem // read the machine OS build number from the SDK. Fallback to macOS 17E198 (10.13.4 'High Sierra').
	set TEMPVAR=& if exist "!SDK_PATH!\osx-build" set /p TEMPVAR=< "!SDK_PATH!\osx-build"
	if "_!TEMPVAR!_"=="__" (
		set TEMPVAR=17E198
		echo WARNING: macOS build number guessing failed, defaulting to !TEMPVAR!.
	)
	if not "_!DEBUG_ARG!_"=="__" echo Calling: plconvert "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" +BuildMachineOSBuild=string:"!TEMPVAR!"
	"!TOOLCHAIN_PATH!\plconvert.exe"                     "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" +BuildMachineOSBuild=string:"!TEMPVAR!"
	if not "!errorlevel!"=="0" goto :exit_error

	rem // record in the app's PList the target platform name, version and platform SDK name and that it was built with the Clang compiler toolchain
	set PLCONVERT_FLAGS=+CFBundleSupportedPlatforms=array:{string}iPhoneOS{/string}^
                            +DTPlatformName=string:"!SDK_PLATFORM_NAME!"^
                            +DTPlatformVersion=string:"!SDK_VERSION!"^
                            +DTSDKName=string:"!SDK_PLATFORM_NAME!!SDK_VERSION!"^
                            +DTCompiler=string:com.apple.compilers.llvm.clang.1_0
	if not "_!DEBUG_ARG!_"=="__" echo Calling: plconvert "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" !PLCONVERT_FLAGS!
	"!TOOLCHAIN_PATH!\plconvert.exe"                     "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" !PLCONVERT_FLAGS!
	if not "!errorlevel!"=="0" goto :exit_error
	rem // cleanup
	set PLCONVERT_FLAGS=

	rem // are we building an iOS application ? if so, add the LSRequiresIPhoneOS key
	if "_!TARGET_PLATFORM_NAME!_"=="_ios_" if "_!ENTRYPOINT!_"=="_crt_" (
		if not "_!DEBUG_ARG!_"=="__" echo Calling: plconvert "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" +LSRequiresIPhoneOS=bool:true
		"!TOOLCHAIN_PATH!\plconvert.exe"                     "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" +LSRequiresIPhoneOS=bool:true
	)

	rem // record in the app's Plist the SDK build tag that we read from the SDK's sdk-build (or version.plist as a fallback)
	if exist "!SDK_PATH!\sdk-build" (
		rem // the "sdk-build" file exists, good
		set /p TEMPVAR=< "!SDK_PATH!\sdk-build"
	) else (
		rem // fallback on the SDK's version.plist if it exists
		"!TOOLCHAIN_PATH!\plconvert.exe" "!SDK_PATH!\version.plist" -query ProductBuildVersion > %TO_TEMPVAR%
	)
	if not "_!DEBUG_ARG!_"=="__" echo Calling: plconvert "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" +DTPlatformBuild=string:"!TEMPVAR!" +DTSDKBuild=string:"!TEMPVAR!"
	"!TOOLCHAIN_PATH!\plconvert.exe"                     "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" +DTPlatformBuild=string:"!TEMPVAR!" +DTSDKBuild=string:"!TEMPVAR!"
	if not "!errorlevel!"=="0" goto :exit_error

	rem // record in the app's Plist the Xcode version number and build number
	"!TOOLCHAIN_PATH!\plconvert.exe" "!SDK_PATH!\xcode-version.plist" -query CFBundleShortVersionString > %TO_TEMPVAR%
	for /f "tokens=1,2,3 delims=." %%a in ("!TEMPVAR!") do (set XCODE_MAJOR=%%a& set XCODE_MINOR=%%b& set XCODE_REVIS=%%c)
	if "_!XCODE_MAJOR!_"=="__" set XCODE_MAJOR=0
	(if "_!XCODE_MINOR!_"=="__" set XCODE_MINOR=0) & call :dec_to_hex !XCODE_MINOR! XCODE_MINOR
	(if "_!XCODE_REVIS!_"=="__" set XCODE_REVIS=0) & call :dec_to_hex !XCODE_REVIS! XCODE_REVIS
	set TEMPVAR=0000!XCODE_MAJOR!!XCODE_MINOR!!XCODE_REVIS!
	set TEMPVAR=!TEMPVAR:~-4!
	if not "_!DEBUG_ARG!_"=="__" echo Calling: plconvert "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" +DTXcode=string:"!TEMPVAR!"
	"!TOOLCHAIN_PATH!\plconvert.exe"                     "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" +DTXcode=string:"!TEMPVAR!"
	if not "!errorlevel!"=="0" goto :exit_error

	"!TOOLCHAIN_PATH!\plconvert.exe" "!SDK_PATH!\xcode-version.plist" -query ProductBuildVersion > %TO_TEMPVAR%
	if not "_!DEBUG_ARG!_"=="__" echo Calling: plconvert "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" +DTXcodeBuild=string:"!TEMPVAR!"
	"!TOOLCHAIN_PATH!\plconvert.exe"                     "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" +DTXcodeBuild=string:"!TEMPVAR!"
	if not "!errorlevel!"=="0" goto :exit_error

	rem // auto-fix the device capabilities. Add the arch for which we built if it's a single arch.
	if "_!ARCHS: =!_"=="_!ARCHS!_" (
		"!TOOLCHAIN_PATH!\plconvert.exe" "!OUTDIR!\Info.plist" -query UIRequiredDeviceCapabilities 2>nul|findstr /v "arm" > "!TEMPDIR!\required-device-capabilities.txt"
		if "_!ARCHS!_"=="_armv6_" echo armv6>> "!TEMPDIR!\required-device-capabilities.txt"
		if "_!ARCHS!_"=="_armv7_" echo armv7>> "!TEMPDIR!\required-device-capabilities.txt"
		if "_!ARCHS!_"=="_arm64_" echo arm64>> "!TEMPDIR!\required-device-capabilities.txt"
		set TEMPVAR=& for /f "usebackq delims=" %%l in ("!TEMPDIR!\required-device-capabilities.txt") do set TEMPVAR=!TEMPVAR!{string}%%l{/string}
		if not "_!DEBUG_ARG!_"=="__" echo Calling: plconvert "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" +UIRequiredDeviceCapabilities=array:"!TEMPVAR!"
		"!TOOLCHAIN_PATH!\plconvert.exe"                     "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" +UIRequiredDeviceCapabilities=array:"!TEMPVAR!"
		if not "!errorlevel!"=="0" goto :exit_error
	)

	rem // are we building a Xcode project's main application ? if so, prepare the bundle icons and splash screens
	if not "_!XCNAME!_"=="__" if exist "!XCNAME!.xcodeproj\project.pbxproj" if "_!ENTRYPOINT!_"=="_crt_" (

		rem // make sure no Assets.car exists in the output directory
		if exist "!OUTDIR!\Assets.car" del /f /q "!OUTDIR!\Assets.car" > nul 2>&1

		rem // read the icon set name from the .pbxproj file, use AppIcon as fallback
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query ASSETCATALOG_COMPILER_APPICON_NAME > %TO_TEMPVAR%
		if not "_!TEMPVAR!_"=="__" set APPICONSET_SETNAME=!TEMPVAR!
		if "_!APPICONSET_SETNAME!_"=="__" set APPICONSET_SETNAME=AppIcon

		rem // read the launch image name from the .pbxproj file, use LaunchImage as fallback
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -query ASSETCATALOG_COMPILER_LAUNCHIMAGE_NAME > %TO_TEMPVAR%
		if not "_!TEMPVAR!_"=="__" set LAUNCHIMAGE_SETNAME=!TEMPVAR!
		if "_!LAUNCHIMAGE_SETNAME!_"=="__" set LAUNCHIMAGE_SETNAME=LaunchImage

		rem // guess the list of idioms to use
		set IDIOMS=!FAMILY!
		if "_!IDIOMS!_"=="_all_" set IDIOMS=iphone,ipad

		rem // list the asset catalogs from the .pbxproj file, and compile them all into the same Assets.car file
		set APPICONSET_CATALOG=
		set LAUNCHIMAGE_CATALOG=
		"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -assets -line_format " \"%%s\"" > %TO_TEMPVAR%
		if not "_!TEMPVAR!_"=="__" set ASSETCATALOGS=!TEMPVAR:/=\!
		for /d %%r in (!ASSETCATALOGS!) do (
			set ITEM_PATHNAME=%%~r
			if not exist "!ITEM_PATHNAME!" (
				rem // is this asset catalog referenced to by an ABSOLUTE pathname ? (Unity bug)
				if not "_!ITEM_PATHNAME:~1,1!_"=="_:_" (
					echo Warning: the asset catalog !ITEM_PATHNAME! referenced in this Xcode project cannot be found ^(bogus Xcode project^). Ignoring.
				) else (
					echo Warning: the asset catalog !ITEM_PATHNAME! referenced in this Xcode project cannot be found ^(bogus absolute path^). Attempting to fix...
					for /f "tokens=1-15 delims=\" %%a in ("!ITEM_PATHNAME!") do (
						                               set TEST_PATHNAME=%%a\%%b\%%c\%%d\%%e\%%f\%%g\%%h\%%i\%%j\%%k\%%l\%%m\%%n\%%o
						if not exist "!TEST_PATHNAME!" set TEST_PATHNAME=%%b\%%c\%%d\%%e\%%f\%%g\%%h\%%i\%%j\%%k\%%l\%%m\%%n\%%o
						if not exist "!TEST_PATHNAME!" set TEST_PATHNAME=%%c\%%d\%%e\%%f\%%g\%%h\%%i\%%j\%%k\%%l\%%m\%%n\%%o
						if not exist "!TEST_PATHNAME!" set TEST_PATHNAME=%%d\%%e\%%f\%%g\%%h\%%i\%%j\%%k\%%l\%%m\%%n\%%o
						if not exist "!TEST_PATHNAME!" set TEST_PATHNAME=%%e\%%f\%%g\%%h\%%i\%%j\%%k\%%l\%%m\%%n\%%o
						if not exist "!TEST_PATHNAME!" set TEST_PATHNAME=%%f\%%g\%%h\%%i\%%j\%%k\%%l\%%m\%%n\%%o
						if not exist "!TEST_PATHNAME!" set TEST_PATHNAME=%%g\%%h\%%i\%%j\%%k\%%l\%%m\%%n\%%o
						if not exist "!TEST_PATHNAME!" set TEST_PATHNAME=%%h\%%i\%%j\%%k\%%l\%%m\%%n\%%o
						if not exist "!TEST_PATHNAME!" set TEST_PATHNAME=%%i\%%j\%%k\%%l\%%m\%%n\%%o
						if not exist "!TEST_PATHNAME!" set TEST_PATHNAME=%%j\%%k\%%l\%%m\%%n\%%o
						if not exist "!TEST_PATHNAME!" set TEST_PATHNAME=%%k\%%l\%%m\%%n\%%o
						if not exist "!TEST_PATHNAME!" set TEST_PATHNAME=%%l\%%m\%%n\%%o
						if not exist "!TEST_PATHNAME!" set TEST_PATHNAME=%%m\%%n\%%o
						if not exist "!TEST_PATHNAME!" set TEST_PATHNAME=%%n\%%o
						if not exist "!TEST_PATHNAME!" set TEST_PATHNAME=%%o
						if not exist "!TEST_PATHNAME!" (
							echo Error: unable to figure out the asset catalog's relative path. Please fix this Xcode project.
							goto :exit_error
						)
						set ITEM_PATHNAME=!TEST_PATHNAME:\\=!
						echo Warning: Fixed asset catalog path: !ITEM_PATHNAME!
					)
				)
			)

			rem // needs a 2nd check for already-expanded pathnames. Stupid Microsoft.
			if exist "!ITEM_PATHNAME!" pushd "!ITEM_PATHNAME!" 2>nul && popd && (
				set ITEM_BASENAME=%%~nxr
				if exist "!ITEM_PATHNAME!\!APPICONSET_SETNAME!.appiconset\" set APPICONSET_CATALOG=!ITEM_PATHNAME!
				if exist "!ITEM_PATHNAME!\!LAUNCHIMAGE_SETNAME!.launchimage\" set LAUNCHIMAGE_CATALOG=!ITEM_PATHNAME!
				if not "_!DEBUG_ARG!_"=="__" echo    Asset catalog: !ITEM_PATHNAME! -^> !OUTDIR!\Assets.car
				set VERBOSE_CARTOOL_ARGS=
				if not "_!DEBUG_ARG!_"=="__" set VERBOSE_CARTOOL_ARGS=-v -nobanner
				if not "_!DEBUG_ARG!_"=="__" echo Calling: cartool -idioms !IDIOMS! -compile-add "!ITEM_PATHNAME!" -out "!OUTDIR!\Assets.car" !VERBOSE_CARTOOL_ARGS!
				"!TOOLCHAIN_PATH!\cartool.exe"                     -idioms !IDIOMS! -compile-add "!ITEM_PATHNAME!" -out "!OUTDIR!\Assets.car" !VERBOSE_CARTOOL_ARGS! 2>&1
				if not "!errorlevel!"=="0" goto :exit_error
				rem // cleanup
				set VERBOSE_CARTOOL_ARGS=
				set ITEM_BASENAME=
			)
			rem // cleanup
			set ITEM_PATHNAME=
		)
		rem // cleanup
		set ASSETCATALOGS=

		rem /////////////////////////////////////////
		rem // build the AppIcon assets catalog sheet (for legacy iOS versions only. Newest format was compiled inside Assets.car by cartool already)

		rem // identify the largest of the icon files, either from the existing AppIcon assets catalog sheet, or from the icons in the root directory
		set BIGGESTFILE=
		dir /b /o:-s "!APPICONSET_CATALOG!\!APPICONSET_SETNAME!.appiconset\*.png" > %TO_TEMPVAR%
		if not "_!TEMPVAR!_"=="__" set BIGGESTFILE=!APPICONSET_CATALOG!\!APPICONSET_SETNAME!.appiconset\!TEMPVAR!
		if "_!BIGGESTFILE!_"=="__" (
			dir /b /o:-s "!OUTDIR!\*Icon*.png" > %TO_TEMPVAR%
			if not "_!TEMPVAR!_"=="__" set BIGGESTFILE=!OUTDIR!\!TEMPVAR!
		)
		if "_!BIGGESTFILE!_"=="__" (
			echo Warning: no app icon found, will generate a dummy one ^(please add icons to your project^).
			set BIGGESTFILE=!TEMPDIR!\dummyicon.png
			"!TOOLCHAIN_PATH!\imconvert.exe" new "!BIGGESTFILE!"
			if not "!errorlevel!"=="0" goto :exit_error
		)

		rem // create all the necessary launch images, reusing those that already exist, and pointing at the widest of each sort when not
		set IPHONE_1X_29X29=& set IPAD_1X_29X29=
		set IPHONE_2X_40X40=& set IPAD_2X_40X40=
		set IPHONE_2X_58X58=& set IPAD_2X_58X58=
		set IPHONE_1X_57X57=
		set IPHONE_1X_60X60=& set IPHONE_3X_60X60=
		set IPHONE_3X_87X87=
		set IPHONE_2X_114X114=
		set IPHONE_2X_120X120=
		set IPHONE_3X_180X180=
		set IPAD_1X_20X20=
		set IPAD_1X_40X40=
		set IPAD_1X_50X50=
		set IPAD_1X_72X72=
		set IPAD_1X_76X76=
		set IPAD_2X_80X80=
		set IPAD_2X_100X100=
		set IPAD_2X_144X144=
		set IPAD_2X_152X152=
		set IPAD_2X_167X167=
		for %%f in ("!APPICONSET_CATALOG!\!APPICONSET_SETNAME!.appiconset\"*.bmp "!APPICONSET_CATALOG!\!APPICONSET_SETNAME!.appiconset\"*.png "!APPICONSET_CATALOG!\!APPICONSET_SETNAME!.appiconset\"*.jpg "!APPICONSET_CATALOG!\!APPICONSET_SETNAME!.appiconset\"*.jpeg) do (
			set FILENAME=%%f

			rem // figure out the dimensions of that image and identify width and height
			"!TOOLCHAIN_PATH!\imconvert.exe" "!FILENAME!" -getsize > %TO_TEMPVAR%
			if "!errorlevel!"=="0" (
				for /f "tokens=1,2 delims=x" %%a in ("!TEMPVAR!") do (set IMG_W=%%a& set IMG_H=%%b)

				rem // identify each possible image
				if !IMG_W! EQU 29   if !IMG_H! EQU 29   set IPHONE_1X_29X29=!FILENAME!& set IPAD_1X_29X29=!FILENAME!
				if !IMG_W! EQU 40   if !IMG_H! EQU 40   set IPHONE_2X_40X40=!FILENAME!& set IPAD_2X_40X40=!FILENAME!
				if !IMG_W! EQU 58   if !IMG_H! EQU 58   set IPHONE_2X_58X58=!FILENAME!& set IPAD_2X_58X58=!FILENAME!

				if !IMG_W! EQU 57   if !IMG_H! EQU 57   set IPHONE_1X_57X57=!FILENAME!
				if !IMG_W! EQU 60   if !IMG_H! EQU 60   set IPHONE_1X_60X60=!FILENAME!& set IPHONE_3X_60X60=!FILENAME!
				if !IMG_W! EQU 87   if !IMG_H! EQU 87   set IPHONE_3X_87X87=!FILENAME!
				if !IMG_W! EQU 114  if !IMG_H! EQU 114  set IPHONE_2X_114X114=!FILENAME!
				if !IMG_W! EQU 120  if !IMG_H! EQU 120  set IPHONE_2X_120X120=!FILENAME!& set IPHONE_3X_120X120=!FILENAME!
				if !IMG_W! EQU 180  if !IMG_H! EQU 180  set IPHONE_3X_180X180=!FILENAME!

				if !IMG_W! EQU 20   if !IMG_H! EQU 20   set IPAD_1X_20X20=!FILENAME!
				if !IMG_W! EQU 40   if !IMG_H! EQU 40   set IPAD_1X_40X40=!FILENAME!
				if !IMG_W! EQU 50   if !IMG_H! EQU 50   set IPAD_1X_50X50=!FILENAME!
				if !IMG_W! EQU 72   if !IMG_H! EQU 72   set IPAD_1X_72X72=!FILENAME!
				if !IMG_W! EQU 76   if !IMG_H! EQU 76   set IPAD_1X_76X76=!FILENAME!
				if !IMG_W! EQU 80   if !IMG_H! EQU 80   set IPAD_2X_80X80=!FILENAME!
				if !IMG_W! EQU 100  if !IMG_H! EQU 100  set IPAD_2X_100X100=!FILENAME!
				if !IMG_W! EQU 144  if !IMG_H! EQU 144  set IPAD_2X_144X144=!FILENAME!
				if !IMG_W! EQU 152  if !IMG_H! EQU 152  set IPAD_2X_152X152=!FILENAME!
				if !IMG_W! EQU 167  if !IMG_H! EQU 167  set IPAD_2X_167X167=!FILENAME!

				rem // cleanup
				set IMG_H=
				set IMG_W=
			) else (
				echo Warning: unidentifiable image in app icon set asset catalog sheet ^(!FILENAME!^). Ignoring.
			)
			rem // cleanup
			set FILENAME=
		)
		rem // don't print notifications about missing icons. cartool already did it.
		if "_!IPHONE_1X_29X29!_"=="__"   set IPHONE_1X_29X29=!BIGGESTFILE!& set IPAD_1X_29X29=!BIGGESTFILE!
		if "_!IPHONE_2X_40X40!_"=="__"   set IPHONE_2X_40X40=!BIGGESTFILE!& set IPAD_2X_40X40=!BIGGESTFILE!
		if "_!IPHONE_2X_58X58!_"=="__"   set IPHONE_2X_58X58=!BIGGESTFILE!& set IPAD_2X_58X58=!BIGGESTFILE!
		if "_!IPHONE_1X_57X57!_"=="__"   set IPHONE_1X_57X57=!BIGGESTFILE!
		if "_!IPHONE_1X_60X60!_"=="__"   set IPHONE_1X_60X60=!BIGGESTFILE!& set IPHONE_3X_60X60=!BIGGESTFILE!
		if "_!IPHONE_3X_87X87!_"=="__"   set IPHONE_3X_87X87=!BIGGESTFILE!
		if "_!IPHONE_2X_114X114!_"=="__" set IPHONE_2X_114X114=!BIGGESTFILE!
		if "_!IPHONE_2X_120X120!_"=="__" set IPHONE_2X_120X120=!BIGGESTFILE!
		if "_!IPHONE_3X_180X180!_"=="__" set IPHONE_3X_180X180=!BIGGESTFILE!
		if "_!IPAD_1X_20X20!_"=="__"     set IPAD_1X_20X20=!BIGGESTFILE!
		if "_!IPAD_1X_40X40!_"=="__"     set IPAD_1X_40X40=!BIGGESTFILE!
		if "_!IPAD_1X_50X50!_"=="__"     set IPAD_1X_50X50=!BIGGESTFILE!
		if "_!IPAD_1X_72X72!_"=="__"     set IPAD_1X_72X72=!BIGGESTFILE!
		if "_!IPAD_1X_76X76!_"=="__"     set IPAD_1X_76X76=!BIGGESTFILE!
		if "_!IPAD_2X_80X80!_"=="__"     set IPAD_2X_80X80=!BIGGESTFILE!
		if "_!IPAD_2X_100X100!_"=="__"   set IPAD_2X_100X100=!BIGGESTFILE!
		if "_!IPAD_2X_144X144!_"=="__"   set IPAD_2X_144X144=!BIGGESTFILE!
		if "_!IPAD_2X_152X152!_"=="__"   set IPAD_2X_152X152=!BIGGESTFILE!
		if "_!IPAD_2X_167X167!_"=="__"   set IPAD_2X_167X167=!BIGGESTFILE!

		rem // cleanup
		set BIGGESTFILE=

		rem // now create all variants, reusing the existing ones and creating the missing ones from the biggest icon
		rem // ref: https://developer.apple.com/library/archive/documentation/DeviceInformation/Reference/iOSDeviceCompatibility/Displays/Displays.html

		rem // BEWARE: KEEP A SPACE AFTER THE EQUAL SIGN!
		set ARRAY_IPHONE= 
		set ARRAY_IPAD= 

		rem // iPhone app icons for all known iPhone models
		if not "_!IDIOMS:iphone=!_"=="_!IDIOMS!_" (
			rem // 29x29 px "Icon-Small.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPHONE_1X_29X29!" -resize 29x29 "!OUTDIR!\Icon-Small.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPHONE=!ARRAY_IPHONE!{string}Icon-Small{/string}

			rem // 40x40 px "Icon-Notification@2x.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPHONE_2X_40X40!" -resize 40x40 "!OUTDIR!\Icon-Notification@2x.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPHONE=!ARRAY_IPHONE!{string}Icon-Notification@2x{/string}

			rem // 57x57 px "Icon.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPHONE_1X_57X57!" -resize 57x57 "!OUTDIR!\Icon.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPHONE=!ARRAY_IPHONE!{string}Icon{/string}

			rem // 58x58 px "Icon-Small@2x.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPHONE_2X_58X58!" -resize 58x58 "!OUTDIR!\Icon-Small@2x.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPHONE=!ARRAY_IPHONE!{string}Icon-Small@2x{/string}

			rem // 60x60 px "Icon-60.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPHONE_1X_60X60!" -resize 60x60 "!OUTDIR!\Icon-60.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPHONE=!ARRAY_IPHONE!{string}Icon-60{/string}

			rem // 60x60 px "Icon-Notification@3x.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPHONE_3X_60X60!" -resize 60x60 "!OUTDIR!\Icon-Notification@3x.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPHONE=!ARRAY_IPHONE!{string}Icon-Notification@3x{/string}

			rem // 87x87 px "Icon-Small@3x.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPHONE_3X_87X87!" -resize 87x87 "!OUTDIR!\Icon-Small@3x.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPHONE=!ARRAY_IPHONE!{string}Icon-Small@3x{/string}

			rem // 114x114 px "Icon@2x.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPHONE_2X_114X114!" -resize 114x114 "!OUTDIR!\Icon@2x.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPHONE=!ARRAY_IPHONE!{string}Icon@2x{/string}

			rem // 120x120 px "Icon-120.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPHONE_2X_120X120!" -resize 120x120 "!OUTDIR!\Icon-120.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPHONE=!ARRAY_IPHONE!{string}Icon-120{/string}

			rem // 180x180 px "Icon-180.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPHONE_3X_180X180!" -resize 180x180 "!OUTDIR!\Icon-180.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPHONE=!ARRAY_IPHONE!{string}Icon-180{/string}
		)

		rem // iPad app icons for all known iPad models
		if not "_!IDIOMS:ipad=!_"=="_!IDIOMS!_" (
			rem // 20x20 px "Icon-Notification.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPAD_1X_20X20!" -resize 20x20 "!OUTDIR!\Icon-Notification.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPAD=!ARRAY_IPAD!{string}Icon-Notification{/string}

			rem // 29x29 px "Icon-Small.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPAD_1X_29X29!" -resize 29x29 "!OUTDIR!\Icon-Small.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPAD=!ARRAY_IPAD!{string}Icon-Small{/string}

			rem // 40x40 px "Icon-Notification@2x.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPAD_2X_40X40!" -resize 40x40 "!OUTDIR!\Icon-Notification@2x.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPAD=!ARRAY_IPAD!{string}Icon-Notification@2x{/string}

			rem // 40x40 px "Icon-Small-40.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPAD_1X_40X40!" -resize 40x40 "!OUTDIR!\Icon-Small-40.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPAD=!ARRAY_IPAD!{string}Icon-Small-40{/string}

			rem // 50x50 px "Icon-Small-50.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPAD_1X_50X50!" -resize 50x50 "!OUTDIR!\Icon-Small-50.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPAD=!ARRAY_IPAD!{string}Icon-Small-50{/string}

			rem // 58x58 px "Icon-Small@2x.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPAD_2X_58X58!" -resize 58x58 "!OUTDIR!\Icon-Small@2x.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPAD=!ARRAY_IPAD!{string}Icon-Small@2x{/string}

			rem // 72x72 px "Icon-72.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPAD_1X_72X72!" -resize 72x72 "!OUTDIR!\Icon-72.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPAD=!ARRAY_IPAD!{string}Icon-72{/string}

			rem // 76x76 px "Icon-76.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPAD_1X_76X76!" -resize 76x76 "!OUTDIR!\Icon-76.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPAD=!ARRAY_IPAD!{string}Icon-76{/string}

			rem // 80x80 px "Icon-Small-80.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPAD_2X_80X80!" -resize 80x80 "!OUTDIR!\Icon-Small-80.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPAD=!ARRAY_IPAD!{string}Icon-Small-80{/string}

			rem // 100x100 px "Icon-Small-100.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPAD_2X_100X100!" -resize 100x100 "!OUTDIR!\Icon-Small-100.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPAD=!ARRAY_IPAD!{string}Icon-Small-100{/string}

			rem // 144x144 px "Icon-144.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPAD_2X_144X144!" -resize 144x144 "!OUTDIR!\Icon-144.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPAD=!ARRAY_IPAD!{string}Icon-144{/string}

			rem // 152x152 px "Icon-152.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPAD_2X_152X152!" -resize 152x152 "!OUTDIR!\Icon-152.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPAD=!ARRAY_IPAD!{string}Icon-152{/string}

			rem // 167x167 px "Icon-167.png"
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPAD_2X_167X167!" -resize 167x167 "!OUTDIR!\Icon-167.png"
			if not "!errorlevel!"=="0" goto :exit_error
			set ARRAY_IPAD=!ARRAY_IPAD!{string}Icon-167{/string}
		)

		rem // add the icons for the various iOS versions (CFBundleIconFile: iOS<3.2; CFBundleIconFiles: iOS<5.0; CFBundleIcons: iOS<11; CFBundleIconName: iOS11+)
		rem // NOTE: CFBundleIconName MUST be a subkey of CFBundleIcons/CFBundlePrimaryIcon for App Store Connect validation to pass, consistently with Apple's documentation
		"!TOOLCHAIN_PATH!\plconvert.exe" "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist"^
                                                 "+CFBundleIconFile=string:Icon"^
                                                 "+CFBundleIconFiles=array:!ARRAY_IPHONE!!ARRAY_IPAD!"^
                                                 "+CFBundleIcons=dict:{key}CFBundlePrimaryIcon{/key}{dict}{key}CFBundleIconName{/key}{string}!APPICONSET_SETNAME!{/string}{key}CFBundleIconFiles{/key}{array}!ARRAY_IPHONE!{/array}{/dict}"^
                                                 "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" "+CFBundleIcons~ipad=dict:{key}CFBundlePrimaryIcon{/key}{dict}{key}CFBundleIconName{/key}{string}!APPICONSET_SETNAME!{/string}{key}CFBundleIconFiles{/key}{array}!ARRAY_IPAD!{/array}{/dict}"
		if not "!errorlevel!"=="0" goto :exit_error

		rem ////////////////////////////////////////////////////////////
		rem // build the LaunchImage assets catalog sheet, if one exists

		if exist "!LAUNCHIMAGE_CATALOG!\!LAUNCHIMAGE_SETNAME!.launchimage\." (

			rem // identify the largest of the launch image files (portrait mode) from the existing LaunchImage assets catalog sheet
			dir /b /o:-s "!LAUNCHIMAGE_CATALOG!\!LAUNCHIMAGE_SETNAME!.launchimage\*-Portrait*.png" > %TO_TEMPVAR%
			if not "_!TEMPVAR!_"=="__" set BIGGESTFILE_PORTRAIT=!LAUNCHIMAGE_CATALOG!\!LAUNCHIMAGE_SETNAME!.launchimage\!TEMPVAR!
			if "_!BIGGESTFILE_PORTRAIT!_"=="__" if exist "!LAUNCHIMAGE_CATALOG!\!LAUNCHIMAGE_SETNAME!.launchimage\." (
				dir /b /o:-s "!LAUNCHIMAGE_CATALOG!\!LAUNCHIMAGE_SETNAME!.launchimage\*.png" > %TO_TEMPVAR%
				if not "_!TEMPVAR!_"=="__" set BIGGESTFILE_PORTRAIT=!LAUNCHIMAGE_CATALOG!\!LAUNCHIMAGE_SETNAME!.launchimage\!TEMPVAR!
			)
			if "_!BIGGESTFILE_PORTRAIT!_"=="__" (
				echo Warning: no launch images found, will generate a dummy one ^(please add launch images to your project^).
				set BIGGESTFILE_PORTRAIT=!TEMPDIR!\dummylaunchimage-portrait.png
				"!TOOLCHAIN_PATH!\imconvert.exe" new "!BIGGESTFILE_PORTRAIT!"
				if not "!errorlevel!"=="0" goto :exit_error
			)

			rem // identify the largest of the launch image files (landscape mode) from the existing LaunchImage assets catalog sheet
			dir /b /o:-s "!LAUNCHIMAGE_CATALOG!\!LAUNCHIMAGE_SETNAME!.launchimage\*-Landscape*.png" > %TO_TEMPVAR%
			if not "_!TEMPVAR!_"=="__" set BIGGESTFILE_LANDSCAPE=!LAUNCHIMAGE_CATALOG!\!LAUNCHIMAGE_SETNAME!.launchimage\!TEMPVAR!
			if "_!BIGGESTFILE_LANDSCAPE!_"=="__" if exist "!LAUNCHIMAGE_CATALOG!\!LAUNCHIMAGE_SETNAME!.launchimage\." (
				dir /b /o:-s "!LAUNCHIMAGE_CATALOG!\!LAUNCHIMAGE_SETNAME!.launchimage\*.png" > %TO_TEMPVAR%
				if not "_!TEMPVAR!_"=="__" set BIGGESTFILE_LANDSCAPE=!LAUNCHIMAGE_CATALOG!\!LAUNCHIMAGE_SETNAME!.launchimage\!TEMPVAR!
			)
			if "_!BIGGESTFILE_LANDSCAPE!_"=="__" (
				echo Warning: no launch images found, will generate a dummy one ^(please add launch images to your project^).
				set BIGGESTFILE_LANDSCAPE=!TEMPDIR!\dummylaunchimage-landscape.png
				"!TOOLCHAIN_PATH!\imconvert.exe" new "!BIGGESTFILE_LANDSCAPE!"
				if not "!errorlevel!"=="0" goto :exit_error
			)

			rem // create all the necessary launch images, reusing those that already exist, and pointing at the widest of each sort when not
			set IPHONE_480_PORT=
			set IPHONE_960_PORT=
			set IPHONE_2436_PORT=
			set IPHONE_2436_LAND=
			set IPHONE_2208_PORT=
			set IPHONE_2208_LAND=
			set IPHONE_1334_PORT=
			set IPHONE_1136_PORT=
			set IPAD_1024_PORT=
			set IPAD_1024_LAND=
			set IPAD_2048_PORT=
			set IPAD_2048_LAND=
			for %%f in ("!LAUNCHIMAGE_CATALOG!\!LAUNCHIMAGE_SETNAME!.launchimage\"*.bmp "!LAUNCHIMAGE_CATALOG!\!LAUNCHIMAGE_SETNAME!.launchimage\"*.png "!LAUNCHIMAGE_CATALOG!\!LAUNCHIMAGE_SETNAME!.launchimage\"*.jpg "!LAUNCHIMAGE_CATALOG!\!LAUNCHIMAGE_SETNAME!.launchimage\"*.jpeg) do (
				set FILENAME=%%f

				rem // figure out the dimensions of that image and identify width and height
				"!TOOLCHAIN_PATH!\imconvert.exe" "!FILENAME!" -getsize > %TO_TEMPVAR%
				if "!errorlevel!"=="0" (
					for /f "tokens=1,2 delims=x" %%a in ("!TEMPVAR!") do (set IMG_W=%%a& set IMG_H=%%b)

					rem // identify each possible image
					if !IMG_W! EQU 320  if !IMG_H! EQU 480  set IPHONE_480_PORT=!FILENAME!
					if !IMG_W! EQU 640  if !IMG_H! EQU 960  set IPHONE_960_PORT=!FILENAME!
					if !IMG_W! EQU 1125 if !IMG_H! EQU 2436 set IPHONE_2436_PORT=!FILENAME!
					if !IMG_W! EQU 2436 if !IMG_H! EQU 1125 set IPHONE_2436_LAND=!FILENAME!
					if !IMG_W! EQU 1242 if !IMG_H! EQU 2208 set IPHONE_2208_PORT=!FILENAME!
					if !IMG_W! EQU 2208 if !IMG_H! EQU 1242 set IPHONE_2208_LAND=!FILENAME!
					if !IMG_W! EQU 750  if !IMG_H! EQU 1334 set IPHONE_1334_PORT=!FILENAME!
					if !IMG_W! EQU 640  if !IMG_H! EQU 1136 set IPHONE_1136_PORT=!FILENAME!
					if !IMG_W! EQU 768  if !IMG_H! EQU 1024 set IPAD_1024_PORT=!FILENAME!
					if !IMG_W! EQU 1024 if !IMG_H! EQU 768  set IPAD_1024_LAND=!FILENAME!
					if !IMG_W! EQU 1536 if !IMG_H! EQU 2048 set IPAD_2048_PORT=!FILENAME!
					if !IMG_W! EQU 2048 if !IMG_H! EQU 1536 set IPAD_2048_LAND=!FILENAME!

					rem // cleanup
					set IMG_H=
					set IMG_W=
				) else (
					echo Warning: unidentifiable image in launch image asset catalog sheet ^(!FILENAME!^). Ignoring.
				)
				rem // cleanup
				set FILENAME=
			)
			if "_!IPHONE_480_PORT!_"=="__"  echo Notice: missing iPhone 320x480 variant in launch image set '!LAUNCHIMAGE_SETNAME!', falling back on best resolution portrait.&    set IPHONE_480_PORT=!BIGGESTFILE_PORTRAIT!
			if "_!IPHONE_960_PORT!_"=="__"  echo Notice: missing iPhone 640x960 variant in launch image set '!LAUNCHIMAGE_SETNAME!', falling back on best resolution portrait.&    set IPHONE_960_PORT=!BIGGESTFILE_PORTRAIT!
			if "_!IPHONE_2436_PORT!_"=="__" echo Notice: missing iPhone 1125x2436 variant in launch image set '!LAUNCHIMAGE_SETNAME!', falling back on best resolution portrait.&  set IPHONE_2436_PORT=!BIGGESTFILE_PORTRAIT!
			if "_!IPHONE_2436_LAND!_"=="__" echo Notice: missing iPhone 2436x1125 variant in launch image set '!LAUNCHIMAGE_SETNAME!', falling back on best resolution landscape.& set IPHONE_2436_LAND=!BIGGESTFILE_LANDSCAPE!
			if "_!IPHONE_2208_PORT!_"=="__" echo Notice: missing iPhone 1242x2208 variant in launch image set '!LAUNCHIMAGE_SETNAME!', falling back on best resolution portrait.&  set IPHONE_2208_PORT=!BIGGESTFILE_PORTRAIT!
			if "_!IPHONE_2208_LAND!_"=="__" echo Notice: missing iPhone 2208x1242 variant in launch image set '!LAUNCHIMAGE_SETNAME!', falling back on best resolution landscape.& set IPHONE_2208_LAND=!BIGGESTFILE_LANDSCAPE!
			if "_!IPHONE_1334_PORT!_"=="__" echo Notice: missing iPhone 750x1334 variant in launch image set '!LAUNCHIMAGE_SETNAME!', falling back on best resolution portrait.&   set IPHONE_1334_PORT=!BIGGESTFILE_PORTRAIT!
			if "_!IPHONE_1136_PORT!_"=="__" echo Notice: missing iPhone 640x1136 variant in launch image set '!LAUNCHIMAGE_SETNAME!', falling back on best resolution portrait.&   set IPHONE_1136_PORT=!BIGGESTFILE_PORTRAIT!
			if "_!IPAD_1024_PORT!_"=="__" echo Notice: missing iPad 768x1024 variant in launch image set '!LAUNCHIMAGE_SETNAME!', falling back on best resolution portrait.&   set IPAD_1024_PORT=!BIGGESTFILE_PORTRAIT!
			if "_!IPAD_1024_LAND!_"=="__" echo Notice: missing iPad 1024x768 variant in launch image set '!LAUNCHIMAGE_SETNAME!', falling back on best resolution landscape.&  set IPAD_1024_LAND=!BIGGESTFILE_LANDSCAPE!
			if "_!IPAD_2048_PORT!_"=="__" echo Notice: missing iPad 1536x2048 variant in launch image set '!LAUNCHIMAGE_SETNAME!', falling back on best resolution portrait.&  set IPAD_2048_PORT=!BIGGESTFILE_PORTRAIT!
			if "_!IPAD_2048_LAND!_"=="__" echo Notice: missing iPad 2048x1536 variant in launch image set '!LAUNCHIMAGE_SETNAME!', falling back on best resolution landscape.& set IPAD_2048_LAND=!BIGGESTFILE_LANDSCAPE!

			rem // now create all variants, reusing the existing ones and creating the missing ones from the biggest image
			rem // ref: https://developer.apple.com/library/archive/documentation/DeviceInformation/Reference/iOSDeviceCompatibility/Displays/Displays.html

			rem // BEWARE: KEEP A SPACE AFTER THE EQUAL SIGN!
			set ARRAY= 

			rem // iPhone launch screens for all known iPhone models
			if not "_!IDIOMS:iphone=!_"=="_!IDIOMS!_" (
				rem // 812 points: iPhone rounded Super Retina (3x) portrait and landscape (iPhone X)
				"!TOOLCHAIN_PATH!\imconvert.exe" "!IPHONE_2436_PORT!" -resize 1125x2436 "!OUTDIR!\LaunchImage-1100-Portrait-2436h@3x.png"
				if not "!errorlevel!"=="0" goto :exit_error
				"!TOOLCHAIN_PATH!\imconvert.exe" "!IPHONE_2436_LAND!" -resize 2436x1125 "!OUTDIR!\LaunchImage-1100-Landscape-2436h@3x.png"
				if not "!errorlevel!"=="0" goto :exit_error
				set ARRAY=!ARRAY!{dict}{key}UILaunchImageMinimumOSVersion{/key}{string}11.0{/string}{key}UILaunchImageName{/key}{string}LaunchImage-1100-Portrait-2436h{/string}{key}UILaunchImageOrientation{/key}{string}Portrait{/string}{key}UILaunchImageSize{/key}{string}{375, 812}{/string}{/dict}
				set ARRAY=!ARRAY!{dict}{key}UILaunchImageMinimumOSVersion{/key}{string}11.0{/string}{key}UILaunchImageName{/key}{string}LaunchImage-1100-Landscape-2436h{/string}{key}UILaunchImageOrientation{/key}{string}Landscape{/string}{key}UILaunchImageSize{/key}{string}{375, 812}{/string}{/dict}

				rem // 736 points: iPhone 16/9 Super Retina (3x) portrait and landscape (iPhone 6+, iPhone 6S+, iPhone 7+, iPhone 8+)
				"!TOOLCHAIN_PATH!\imconvert.exe" "!IPHONE_2208_PORT!" -resize 1242x2208 "!OUTDIR!\LaunchImage-800-Portrait-736h@3x.png"
				if not "!errorlevel!"=="0" goto :exit_error
				"!TOOLCHAIN_PATH!\imconvert.exe" "!IPHONE_2208_LAND!" -resize 2208x1242 "!OUTDIR!\LaunchImage-800-Landscape-736h@3x.png"
				if not "!errorlevel!"=="0" goto :exit_error
				set ARRAY=!ARRAY!{dict}{key}UILaunchImageMinimumOSVersion{/key}{string}8.0{/string}{key}UILaunchImageName{/key}{string}LaunchImage-800-Portrait-736h{/string}{key}UILaunchImageOrientation{/key}{string}Portrait{/string}{key}UILaunchImageSize{/key}{string}{414, 736}{/string}{/dict}
				set ARRAY=!ARRAY!{dict}{key}UILaunchImageMinimumOSVersion{/key}{string}8.0{/string}{key}UILaunchImageName{/key}{string}LaunchImage-800-Landscape-736h{/string}{key}UILaunchImageOrientation{/key}{string}Landscape{/string}{key}UILaunchImageSize{/key}{string}{414, 736}{/string}{/dict}

				rem // 667 points: iPhone 16/9 Retina (2x) portrait (iPhone 6, iPhone 6S, iPhone 7, iPhone 8). Landscape version is not generated by Xcode, thus so do I.
				"!TOOLCHAIN_PATH!\imconvert.exe" "!IPHONE_1334_PORT!" -resize 750x1334 "!OUTDIR!\LaunchImage-800-667h@2x.png"
				if not "!errorlevel!"=="0" goto :exit_error
				set ARRAY=!ARRAY!{dict}{key}UILaunchImageMinimumOSVersion{/key}{string}8.0{/string}{key}UILaunchImageName{/key}{string}LaunchImage-800-667h{/string}{key}UILaunchImageOrientation{/key}{string}Portrait{/string}{key}UILaunchImageSize{/key}{string}{375, 667}{/string}{/dict}

				rem // 480 points: iPhone 4/3 Retina (2x) portrait (iPhone 4, iPhone 4S). Landscape versions are not generated by Xcode, thus so do I.
				"!TOOLCHAIN_PATH!\imconvert.exe" "!IPHONE_960_PORT!" -resize 640x960 "!OUTDIR!\LaunchImage-700@2x.png"
				if not "!errorlevel!"=="0" goto :exit_error
				set ARRAY=!ARRAY!{dict}{key}UILaunchImageMinimumOSVersion{/key}{string}7.0{/string}{key}UILaunchImageName{/key}{string}LaunchImage-700{/string}{key}UILaunchImageOrientation{/key}{string}Portrait{/string}{key}UILaunchImageSize{/key}{string}{320, 480}{/string}{/dict}

				rem // 568 points: iPhone 16/9 Retina (2x) portrait and landscape (iPhone 5, iPhone 5S, iPhone 5C, iPhone SE)
				"!TOOLCHAIN_PATH!\imconvert.exe" "!IPHONE_1136_PORT!" -resize 640x1136 "!OUTDIR!\LaunchImage-700-568h@2x.png"
				if not "!errorlevel!"=="0" goto :exit_error
				set ARRAY=!ARRAY!{dict}{key}UILaunchImageMinimumOSVersion{/key}{string}7.0{/string}{key}UILaunchImageName{/key}{string}LaunchImage-700-568h{/string}{key}UILaunchImageOrientation{/key}{string}Portrait{/string}{key}UILaunchImageSize{/key}{string}{320, 568}{/string}{/dict}
			)

			rem // iPad launch screens for all known iPad models
			if not "_!IDIOMS:ipad=!_"=="_!IDIOMS!_" (
				rem // 1024 points: iPad 4/3 standard (1x) portrait and landscape (iPad 1, iPad 2, iPad Mini 1), and iPad 4/3 Retina (2x) portrait and landscape (iPad 3, iPad 4, iPad Air 1, iPad Air 2, iPad 2017, iPad Mini 2, iPad Mini 3, iPad Mini 4, iPad Pro 9.7')
				"!TOOLCHAIN_PATH!\imconvert.exe" "!IPAD_1024_PORT!" -resize 768x1024 "!OUTDIR!\LaunchImage-700-Portrait~ipad.png"
				if not "!errorlevel!"=="0" goto :exit_error
				"!TOOLCHAIN_PATH!\imconvert.exe" "!IPAD_1024_LAND!" -resize 1024x768 "!OUTDIR!\LaunchImage-700-Landscape~ipad.png"
				if not "!errorlevel!"=="0" goto :exit_error
				"!TOOLCHAIN_PATH!\imconvert.exe" "!IPAD_2048_PORT!" -resize 1536x2048 "!OUTDIR!\LaunchImage-700-Portrait@2x~ipad.png"
				if not "!errorlevel!"=="0" goto :exit_error
				"!TOOLCHAIN_PATH!\imconvert.exe" "!IPAD_2048_LAND!" -resize 2048x1536 "!OUTDIR!\LaunchImage-700-Landscape@2x~ipad.png"
				if not "!errorlevel!"=="0" goto :exit_error
				set ARRAY=!ARRAY!{dict}{key}UILaunchImageMinimumOSVersion{/key}{string}7.0{/string}{key}UILaunchImageName{/key}{string}LaunchImage-700-Portrait{/string}{key}UILaunchImageOrientation{/key}{string}Portrait{/string}{key}UILaunchImageSize{/key}{string}{768, 1024}{/string}{/dict}
				set ARRAY=!ARRAY!{dict}{key}UILaunchImageMinimumOSVersion{/key}{string}7.0{/string}{key}UILaunchImageName{/key}{string}LaunchImage-700-Landscape{/string}{key}UILaunchImageOrientation{/key}{string}Landscape{/string}{key}UILaunchImageSize{/key}{string}{768, 1024}{/string}{/dict}
			)

			rem // pre-iOS6 launch screens for iPhones, Retina iPhones, iPads and Retina iPads
			"!TOOLCHAIN_PATH!\imconvert.exe" "!IPHONE_480_PORT!" -resize 320x480 "!OUTDIR!\LaunchImage.png"
			if not "!errorlevel!"=="0" goto :exit_error
			rem // are we allowed to reuse the existing files as much as possible by symlinking to them ?
			if "_!WANT_SYMLINKS!_"=="_true_" (
				if exist "!OUTDIR!\LaunchImage-700@2x.png"                echo|set /p="LaunchImage-700@2x.png"                > "!OUTDIR!\LaunchImage@2x.png"
				if exist "!OUTDIR!\LaunchImage-700-568h@2x.png"           echo|set /p="LaunchImage-700-568h@2x.png"           > "!OUTDIR!\LaunchImage-568h@2x.png"
				if exist "!OUTDIR!\LaunchImage-700-Portrait~ipad.png"     echo|set /p="LaunchImage-700-Portrait~ipad.png"     > "!OUTDIR!\LaunchImage-Portrait~ipad.png"
				if exist "!OUTDIR!\LaunchImage-700-Landscape~ipad.png"    echo|set /p="LaunchImage-700-Landscape~ipad.png"    > "!OUTDIR!\LaunchImage-Landscape~ipad.png"
				if exist "!OUTDIR!\LaunchImage-700-Portrait@2x~ipad.png"  echo|set /p="LaunchImage-700-Portrait@2x~ipad.png"  > "!OUTDIR!\LaunchImage-Portrait@2x~ipad.png"
				if exist "!OUTDIR!\LaunchImage-700-Landscape@2x~ipad.png" echo|set /p="LaunchImage-700-Landscape@2x~ipad.png" > "!OUTDIR!\LaunchImage-Landscape@2x~ipad.png"
			) else (
				if exist "!OUTDIR!\LaunchImage-700@2x.png"                copy /y /b "!OUTDIR!\LaunchImage-700@2x.png"                "!OUTDIR!\LaunchImage@2x.png"                > nul
				if exist "!OUTDIR!\LaunchImage-700-568h@2x.png"           copy /y /b "!OUTDIR!\LaunchImage-700-568h@2x.png"           "!OUTDIR!\LaunchImage-568h@2x.png"           > nul
				if exist "!OUTDIR!\LaunchImage-700-Portrait~ipad.png"     copy /y /b "!OUTDIR!\LaunchImage-700-Portrait~ipad.png"     "!OUTDIR!\LaunchImage-Portrait~ipad.png"     > nul
				if exist "!OUTDIR!\LaunchImage-700-Landscape~ipad.png"    copy /y /b "!OUTDIR!\LaunchImage-700-Landscape~ipad.png"    "!OUTDIR!\LaunchImage-Landscape~ipad.png"    > nul
				if exist "!OUTDIR!\LaunchImage-700-Portrait@2x~ipad.png"  copy /y /b "!OUTDIR!\LaunchImage-700-Portrait@2x~ipad.png"  "!OUTDIR!\LaunchImage-Portrait@2x~ipad.png"  > nul
				if exist "!OUTDIR!\LaunchImage-700-Landscape@2x~ipad.png" copy /y /b "!OUTDIR!\LaunchImage-700-Landscape@2x~ipad.png" "!OUTDIR!\LaunchImage-Landscape@2x~ipad.png" > nul
			)

			rem // add the launch screens for all iOS versions, legacy (UILaunchImageFile: 6.0<=iOS<7.0) and modern (UILaunchImages: iOS7+)
			"!TOOLCHAIN_PATH!\plconvert.exe" "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" "+UILaunchImageFile=string:LaunchImage" "+UILaunchImages=array:!ARRAY!"
			if not "!errorlevel!"=="0" echo Error: unable to add iOS launch images from !LAUNCHIMAGE_CATALOG!\!LAUNCHIMAGE_SETNAME!.launchimage in Info.plist.& goto :exit_error
		)

		rem // convert all .xib and .storyboard files into .nib files, and auto-fix the UILaunchStoryboardName* key values on the fly if needed
		echo  + Compiling Interface Builder launch storyboards...
		rem // iPhone, iPod and iPad variants
		for %%v in (iphone ipod ipad all) do (
			set VARIANT=%%v
			set VARIANT_SUFFIX=~!VARIANT!
			if "_!VARIANT!_"=="_all_" set VARIANT_SUFFIX=
			"!TOOLCHAIN_PATH!\plconvert.exe" "!OUTDIR!\Info.plist" /query UILaunchStoryboardName!VARIANT_SUFFIX! > %TO_TEMPVAR%
			if not "_!TEMPVAR!_"=="__" (
				if not "_!TEMPVAR:.storyboard=!_"=="_!TEMPVAR!_" ( echo Warning: the UILaunchStoryboardName!VARIANT_SUFFIX! key value in your Info.plist should not have any extension. Auto-fixing.& set "TEMPVAR=!TEMPVAR:.storyboard=!" )
				if not "_!TEMPVAR:.xib=!_"=="_!TEMPVAR!_"        ( echo Warning: the UILaunchStoryboardName!VARIANT_SUFFIX! key value in your Info.plist should not have any extension. Auto-fixing.& set "TEMPVAR=!TEMPVAR:.xib=!" )
				if exist "!TEMPVAR!.storyboard" (
					echo    + [!VARIANT!] !OUTDIR!\!TEMPVAR!.storyboardc
					rmdir /s /q "!OUTDIR!\!TEMPVAR!.storyboardc" > nul 2>&1
					copy /y /b "!TEMPVAR!.storyboard" "!OUTDIR!\!TEMPVAR!.storyboard" > nul 2>&1
					if not "_!DEBUG_ARG!_"=="__" echo Calling: nibtool -compile "!OUTDIR!\!TEMPVAR!.storyboard"
					"!TOOLCHAIN_PATH!\nibtool.exe"                     -compile "!OUTDIR!\!TEMPVAR!.storyboard"
					if not "!errorlevel!"=="0" goto :exit_error
					del /f /q "!OUTDIR!\!TEMPVAR!.storyboard" > nul 2>&1
					if not "_!DEBUG_ARG!_"=="__" echo Calling: plconvert "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" "+UILaunchStoryboardName!VARIANT_SUFFIX!=string:!TEMPVAR!"
					"!TOOLCHAIN_PATH!\plconvert.exe"                     "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" "+UILaunchStoryboardName!VARIANT_SUFFIX!=string:!TEMPVAR!"
					if not "!errorlevel!"=="0" goto :exit_error
				) else if exist "!TEMPVAR!.xib" (
					echo    + [!VARIANT!] !OUTDIR!\!TEMPVAR!.nib
					del /f /q "!OUTDIR!\!TEMPVAR!.nib" > nul 2>&1
					copy /y /b "!TEMPVAR!.xib" "!OUTDIR!\!TEMPVAR!.xib" > nul 2>&1
					if not "_!DEBUG_ARG!_"=="__" echo Calling: nibtool -compile "!OUTDIR!\!TEMPVAR!.xib"
					"!TOOLCHAIN_PATH!\nibtool.exe"                     -compile "!OUTDIR!\!TEMPVAR!.xib"
					if not "!errorlevel!"=="0" goto :exit_error
					del /f /q "!OUTDIR!\!TEMPVAR!.xib" > nul 2>&1
					if not "_!DEBUG_ARG!_"=="__" echo Calling: plconvert "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" "+UILaunchStoryboardName!VARIANT_SUFFIX!=string:!TEMPVAR!"
					"!TOOLCHAIN_PATH!\plconvert.exe"                     "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" "+UILaunchStoryboardName!VARIANT_SUFFIX!=string:!TEMPVAR!"
					if not "!errorlevel!"=="0" goto :exit_error
				)
			)
			rem // cleanup
			set VARIANT_SUFFIX=
			set VARIANT=
		)

		rem // cleanup
		set LAUNCHIMAGE_CATALOG=
		set LAUNCHIMAGE_SETNAME=
		set APPICONSET_CATALOG=
		set APPICONSET_SETNAME=
		set ARRAY_IPHONE=
		set ARRAY_IPAD=
		set IDIOMS=
		set IPAD_1X_20X20=
		set IPAD_1X_29X29=
		set IPAD_1X_40X40=
		set IPAD_1X_50X50=
		set IPAD_1X_72X72=
		set IPAD_1X_76X76=
		set IPAD_2X_100X100=
		set IPAD_2X_144X144=
		set IPAD_2X_152X152=
		set IPAD_2X_167X167=
		set IPAD_2X_40X40=
		set IPAD_2X_58X58=
		set IPAD_2X_80X80=
		set IPHONE_1X_29X29=
		set IPHONE_1X_57X57=
		set IPHONE_1X_60X60=
		set IPHONE_2X_114X114=
		set IPHONE_2X_120X120=
		set IPHONE_2X_40X40=
		set IPHONE_2X_58X58=
		set IPHONE_3X_120X120=
		set IPHONE_3X_180X180=
		set IPHONE_3X_60X60=
		set IPHONE_3X_87X87=

		rem /////////////////////////////////////////////////////////
	)

	rem // build the legacy PkgInfo file
	"!TOOLCHAIN_PATH!\plconvert.exe" "!OUTDIR!\Info.plist" /query CFBundlePackageType > %TO_TEMPVAR%
	echo|set /p="!TEMPVAR!"> "!OUTDIR!\PkgInfo"
	"!TOOLCHAIN_PATH!\plconvert.exe" "!OUTDIR!\Info.plist" /query CFBundleSignature > %TO_TEMPVAR%
	if not "_!TEMPVAR!_"=="__" echo|set /p="!TEMPVAR!">> "!OUTDIR!\PkgInfo"
	if     "_!TEMPVAR!_"=="__" echo|set /p="????">> "!OUTDIR!\PkgInfo"

	rem // now convert the Info.plist to binary format
	if not "_!DEBUG_ARG!_"=="__" echo Calling: plconvert "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" -binary
	"!TOOLCHAIN_PATH!\plconvert.exe"                     "!OUTDIR!\Info.plist" "!OUTDIR!\Info.plist" -binary
	if not "!errorlevel!"=="0" goto :exit_error
)

rem // have we copied resources, should we crush PNG files and do some exist in the output directory ?
if not "_!RES!_"=="__" if "_!WANT_PNGCRUSH!_"=="_true_" dir /b "!OUTDIR!\*.png" > "!TEMPDIR!\pnglist.txt" 2>&1 && (

	echo  + Optimizing PNG files in !OUTDIR! for iOS...

	rem // cycle through all PNG files and crush them (BEWARE NOT TO USE !OUTDIR! HERE!!!)...
	for /f "usebackq delims=" %%f in ("!TEMPDIR!\pnglist.txt") do (
		if not "_!DEBUG_ARG!_"=="__" echo Calling: pngcrush -iphone -speed -ow "%OUTDIR%\%%f"
		"!TOOLCHAIN_PATH!\pngcrush.exe"                     -iphone -speed -ow "%OUTDIR%\%%f" > nul 2>&1
	)
	del /f /q "!TEMPDIR!\pnglist.txt" > nul 2>&1

	rem // this is the recursive version. We no longer use it because some embedded 3rd party bundles expect to find their .png files untouched (which makes a lot of sense.)
	rem // for /r "%OUTDIR%" %%f in (*.png) do "!TOOLCHAIN_PATH!\pngcrush.exe" -iphone -speed -ow "%%f" > nul 2>&1
)

rem // cleanup
set SRCROOT=
set RES=


rem //////////////////////////////////
rem // STEP 4. EMBED EXTRA BINARIES //
rem //////////////////////////////////


rem // for each extra binary to embed, do so
if exist "!TEMPDIR!\EMBEDDED_BINARIES.txt" for /f "usebackq delims=" %%a in ("!TEMPDIR!\EMBEDDED_BINARIES.txt") do (
	set EMBEDDED_BINARY=%%a
	rem // remove possible leading and trailing spaces in each entry
	set EMBEDDED_BINARY=###!EMBEDDED_BINARY!###
	set EMBEDDED_BINARY=!EMBEDDED_BINARY: ###=!
	set EMBEDDED_BINARY=!EMBEDDED_BINARY:### =!
	if not "_!EMBEDDED_BINARY!_"=="__" set EMBEDDED_BINARY=!EMBEDDED_BINARY:###=!
	rem // now try to figure out where this embedded binary can be
	if exist "!EMBEDDED_BINARY!" (
		rem // extra binary found under its own path, embed directly
		call :embed_binary "!EMBEDDED_BINARY!" in "!OUTDIR!\!OUTFILE!"
	) else if exist "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks\!EMBEDDED_BINARY!" (
		rem // extra binary found in the built frameworks, dereference symlink and embed
		type "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks\!EMBEDDED_BINARY!" > %TO_TEMPVAR%
		call :embed_binary "!TEMPVAR!" in "!OUTDIR!\!OUTFILE!"
	) else if exist "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks\!EMBEDDED_BINARY!" (
		rem // extra binary found in the supplied frameworks, dereference symlink and embed
		type "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks\!EMBEDDED_BINARY!" > %TO_TEMPVAR%
		call :embed_binary "!TEMPVAR!" in "!OUTDIR!\!OUTFILE!"
	) else if exist "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries\!EMBEDDED_BINARY!" (
		rem // extra binary found in the built libraries, dereference symlink and embed
		type "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries\!EMBEDDED_BINARY!" > %TO_TEMPVAR%
		call :embed_binary "!TEMPVAR!" in "!OUTDIR!\!OUTFILE!"
	) else if exist "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedLibraries\!EMBEDDED_BINARY!" (
		rem // extra binary found in the supplied libraries, dereference symlink and embed
		type "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedLibraries\!EMBEDDED_BINARY!" > %TO_TEMPVAR%
		call :embed_binary "!TEMPVAR!" in "!OUTDIR!\!OUTFILE!"
	) else (
		rem // extra binary not found in the obvious paths, let's do a more thorough search in the build directory
		set OLDDIR=!CD!
		set EMBEDDED_BINARY_NAME=%%~nxa
		cd /d "!BUILD_DIR!"
		dir *"!EMBEDDED_BINARY_NAME!" /b /s 2>nul > %TO_TEMPVAR%
		cd /d "!OLDDIR!"
		set OLDDIR=
		if not "_!TEMPVAR!_"=="__" (
			call :embed_binary "!TEMPVAR!" in "!OUTDIR!\!OUTFILE!"
		) else (
			echo ERROR: the specified embedded binary !EMBEDDED_BINARY! was not found.
			goto :exit_error
		)
	)
	rem // cleanup
	set EMBEDDED_BINARY=
)
del /f /q "!TEMPDIR!\EMBEDDED_BINARIES.txt" > nul 2>&1

rem // have we created a bundle ?
if exist "!OUTDIR!\Info.plist" if exist "!OUTDIR!\!OUTFILE!" (

	rem // is this bundle a full-featured app ? (note: this excludes app extensions and other preference bundles)
	if "_!ENTRYPOINT!_"=="_crt_" (
		rem // make sure dynamic binaries present in BuiltLibraries and BuiltFrameworks are there. If we built them, it's for a good reason.
		if exist "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries\." for /f "usebackq delims=" %%a in (`dir /b "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries\*.dylib" 2^>nul`) do (
			set EMBEDDED_LIBRARY=%%a
			rem // dereference symlink
			type "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries\!EMBEDDED_LIBRARY!" > %TO_TEMPVAR%
			rem // make sure that library is a dynamic one: dynamic libraries don't contain the !<arch> archive tag marker (FIXME: possible false negatives here)
			if exist "!TEMPVAR!" findstr "\^!\<arch\>" "!TEMPVAR!" > nul 2>&1 || (
				rem // this library is a dynamic one, let's embed it
				call :embed_binary "!TEMPVAR!" in "!OUTDIR!\!OUTFILE!"
			)
		)
		if exist "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks\." for /f "usebackq delims=" %%a in (`dir /b "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks\*.framework" 2^>nul`) do (
			set EMBEDDED_FRAMEWORK=%%a
			set EMBEDDED_FRAMEWORK_NAME=%%~na
			rem // dereference symlink
			type "!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks\!EMBEDDED_FRAMEWORK!" > %TO_TEMPVAR%
			rem // make sure that framework is a dynamic one: dynamic frameworks don't contain the !<arch> archive tag marker (FIXME: possible false negatives here)
			if exist "!TEMPVAR!\!EMBEDDED_FRAMEWORK_NAME!" findstr "\^!\<arch\>" "!TEMPVAR!\!EMBEDDED_FRAMEWORK_NAME!" > nul 2>&1 || (
				rem // this framework is a dynamic one, let's embed it
				call :embed_binary "!TEMPVAR!" in "!OUTDIR!\!OUTFILE!"
			)
			rem // cleanup
			set EMBEDDED_FRAMEWORK_NAME=
			set EMBEDDED_FRAMEWORK=
		)
	)

	rem // now fix the load paths in all binaries: make sure the main executable has a rpath to find them and fix their own rpaths so that they point at the right location
	rem // NOTE: when calling "otool" for dumping data, target only the last built arch (which !ARCH! conveniently still holds)

	rem // initialize the list of rpaths to add
	copy /y nul "!TARGET_BUILD_DIR!\rpaths.txt" > nul 2>&1
	rem // NOTE: before 20220922, we added just the necessary rpaths. It's much simpler to add the common ones systematically.
	"!TOOLCHAIN_PATH!\nm.exe" -a "!OUTDIR!\!OUTFILE!" 2>&1|findstr "_swift" > nul && (
		rem // 20231116 - only add this rpath if we actually built Swift code in this target, else App Store Connect will mistakenly believe that the app uses Swift
		rem // without being able to guess the Swift version, and end up claiming that the Payload/SwiftSupport folder doesn't exist with the old Swift dylibs in it.
		echo /usr/lib/swift>>              "!TARGET_BUILD_DIR!\rpaths.txt"
	)
	if "_!SDK_PLATFORM_NAME!_"=="_iphoneos_" (
		echo @executable_path/Frameworks>> "!TARGET_BUILD_DIR!\rpaths.txt"
	) else if "_!SDK_PLATFORM_NAME!_"=="_macosx_" (
		echo @executable_path/Libraries>>  "!TARGET_BUILD_DIR!\rpaths.txt"
		echo @executable_path/Frameworks>> "!TARGET_BUILD_DIR!\rpaths.txt"
	)

	rem // do we have extra libraries ? if so, for each extra library in there...
	if exist "!OUTDIR!\Frameworks\." for /f "usebackq delims=" %%f in (`dir /b "!OUTDIR!\Frameworks\*.dylib" 2^>nul`) do (
		set EMBEDDED_BINARY_NAME=%%~nxf
		set EMBEDDED_BINARY_TYPE=dylib
		set EMBEDDED_BINARY_PATHNAME=!OUTDIR!\Frameworks\!EMBEDDED_BINARY_NAME!

		rem // see whether other libraries/frameworks this binary depends on need to have their path changed
		if not "_!DEBUG_ARG!_"=="__" echo Calling: otool -L "!EMBEDDED_BINARY_PATHNAME!" -arch !ARCH!^|more +1^|findstr "\\" ^> "!TEMPDIR!\fixable_dependencies.txt"
		"!TOOLCHAIN_PATH!\otool.exe"                     -L "!EMBEDDED_BINARY_PATHNAME!" -arch !ARCH! |more +1 |findstr "\\"  > "!TEMPDIR!\fixable_dependencies.txt"
		for /f "usebackq tokens=1 delims=( " %%a in ("!TEMPDIR!\fixable_dependencies.txt") do (
			rem // get the full pathname to fix, and the name of the concerned library without its ".!ARCH!" extension
			set FIXABLE_PATHNAME=%%a
			set FIXABLE_NAME=%%~na

			rem // is that dependency NOT in place yet ?
			if not exist "!OUTDIR!\Frameworks\!FIXABLE_NAME!.framework\!FIXABLE_NAME!" if not exist "!OUTDIR!\Frameworks\!FIXABLE_NAME!.dylib" (
				call :fetch_dependency_or_die "!FIXABLE_NAME!"
			)

			rem // identify dependency type and relative pathname
			if exist "!OUTDIR!\Frameworks\!FIXABLE_NAME!.framework\!FIXABLE_NAME!" (
				set FIXABLE_DEPENDENCY_TYPE=framework
				set FIXABLE_DEPENDENCY_RELATIVE_PATHNAME=Frameworks\!FIXABLE_NAME!.framework\!FIXABLE_NAME!
			) else if exist "!OUTDIR!\Frameworks\!FIXABLE_NAME!.dylib" (
				set FIXABLE_DEPENDENCY_TYPE=dylib
				set FIXABLE_DEPENDENCY_RELATIVE_PATHNAME=Frameworks\!FIXABLE_NAME!.dylib
			)

			rem // and finally fix the load paths in this embedded binary
			echo  + Setting load path for !FIXABLE_NAME! !FIXABLE_DEPENDENCY_TYPE! in embedded !EMBEDDED_BINARY_TYPE! !EMBEDDED_BINARY_NAME!...
			if not "_!DEBUG_ARG!_"=="__" echo Calling: install_name_tool -change "!FIXABLE_PATHNAME!" "@executable_path/!FIXABLE_DEPENDENCY_RELATIVE_PATHNAME:\=/!" "!EMBEDDED_BINARY_PATHNAME!"
			"!TOOLCHAIN_PATH!\install_name_tool.exe"                     -change "!FIXABLE_PATHNAME!" "@executable_path/!FIXABLE_DEPENDENCY_RELATIVE_PATHNAME:\=/!" "!EMBEDDED_BINARY_PATHNAME!"
			if not "!errorlevel!"=="0" goto :exit_error
		)

		rem // now list the rpaths in this library that we should add to the main binary
		if not "_!DEBUG_ARG!_"=="__" echo Calling: otool -l "!EMBEDDED_BINARY_PATHNAME!" -arch !ARCH!^|findstr /c:"\(offset 12\)"^|findstr "^^ path" ^> "!TEMPDIR!\candidate_rpaths.txt"
		"!TOOLCHAIN_PATH!\otool.exe"                     -l "!EMBEDDED_BINARY_PATHNAME!" -arch !ARCH! |findstr /c:"\(offset 12\)" |findstr "^^ path"  > "!TEMPDIR!\candidate_rpaths.txt"
		for /f "usebackq tokens=2 delims=( " %%a in ("!TEMPDIR!\candidate_rpaths.txt") do (
			rem // get the full pathname of this rpath and make sure it does NOT exist yet in the rpaths to add, in which case add it
			set RPATH_PATHNAME=%%a
			findstr /l /c:"!RPATH_PATHNAME!" "!TARGET_BUILD_DIR!\rpaths.txt" > nul 2>&1 || echo !RPATH_PATHNAME!>> "!TARGET_BUILD_DIR!\rpaths.txt"
		)
	)

	rem // do we have extra frameworks ? if so, for each extra framework in there...
	if exist "!OUTDIR!\Frameworks\." for /f "usebackq delims=" %%f in (`dir /b "!OUTDIR!\Frameworks\*.framework" 2^>nul`) do (
		set EMBEDDED_BINARY_NAME=%%~nf
		set EMBEDDED_BINARY_TYPE=framework
		set EMBEDDED_BINARY_PATHNAME=!OUTDIR!\Frameworks\!EMBEDDED_BINARY_NAME!.framework\!EMBEDDED_BINARY_NAME!

		rem // see whether other libraries/frameworks this binary depends on need to have their path changed
		if not "_!DEBUG_ARG!_"=="__" echo Calling: otool -L "!EMBEDDED_BINARY_PATHNAME!" -arch !ARCH!^|more +1^|findstr "\\" ^> "!TEMPDIR!\fixable_dependencies.txt"
		"!TOOLCHAIN_PATH!\otool.exe"                     -L "!EMBEDDED_BINARY_PATHNAME!" -arch !ARCH!|more +1|findstr "\\" > "!TEMPDIR!\fixable_dependencies.txt"
		for /f "usebackq tokens=1 delims=( " %%a in ("!TEMPDIR!\fixable_dependencies.txt") do (
			rem // get the full pathname to fix, and the name of the concerned framework without its ".!ARCH!" extension
			set FIXABLE_PATHNAME=%%a
			set FIXABLE_NAME=%%~na

			rem // is that dependency NOT in place yet ?
			if not exist "!OUTDIR!\Frameworks\!FIXABLE_NAME!.framework\!FIXABLE_NAME!" if not exist "!OUTDIR!\Frameworks\!FIXABLE_NAME!.dylib" (
				call :fetch_dependency_or_die "!FIXABLE_NAME!"
			)

			rem // identify dependency type and relative pathname
			if exist "!OUTDIR!\Frameworks\!FIXABLE_NAME!.framework\!FIXABLE_NAME!" (
				set FIXABLE_DEPENDENCY_TYPE=framework
				set FIXABLE_DEPENDENCY_RELATIVE_PATHNAME=Frameworks\!FIXABLE_NAME!.framework\!FIXABLE_NAME!
			) else if exist "!OUTDIR!\Frameworks\!FIXABLE_NAME!.dylib" (
				set FIXABLE_DEPENDENCY_TYPE=dylib
				set FIXABLE_DEPENDENCY_RELATIVE_PATHNAME=Frameworks\!FIXABLE_NAME!.dylib
			)

			rem // and finally fix the load paths in this embedded binary
			echo  + Setting load path for !FIXABLE_NAME! !FIXABLE_DEPENDENCY_TYPE! in embedded !EMBEDDED_BINARY_TYPE! !EMBEDDED_BINARY_NAME!...
			if not "_!DEBUG_ARG!_"=="__" echo Calling: install_name_tool -change "!FIXABLE_PATHNAME!" "@executable_path/!FIXABLE_DEPENDENCY_RELATIVE_PATHNAME:\=/!" "!EMBEDDED_BINARY_PATHNAME!"
			"!TOOLCHAIN_PATH!\install_name_tool.exe"                     -change "!FIXABLE_PATHNAME!" "@executable_path/!FIXABLE_DEPENDENCY_RELATIVE_PATHNAME:\=/!" "!EMBEDDED_BINARY_PATHNAME!"
			if not "!errorlevel!"=="0" goto :exit_error
		)

		rem // now list the rpaths in this framework that we should add to the main binary
		if not "_!DEBUG_ARG!_"=="__" echo Calling: otool -l "!EMBEDDED_BINARY_PATHNAME!" -arch !ARCH!^|findstr /c:"\(offset 12\)"^|findstr "^^ path" ^> "!TEMPDIR!\candidate_rpaths.txt"
		"!TOOLCHAIN_PATH!\otool.exe"                     -l "!EMBEDDED_BINARY_PATHNAME!" -arch !ARCH! |findstr /c:"\(offset 12\)" |findstr "^^ path"  > "!TEMPDIR!\candidate_rpaths.txt"
		for /f "usebackq tokens=2 delims=( " %%a in ("!TEMPDIR!\candidate_rpaths.txt") do (
			rem // get the full pathname of this rpath and make sure it does NOT exist yet in the rpaths to add, in which case add it
			set RPATH_PATHNAME=%%a
			findstr /l /c:"!RPATH_PATHNAME!" "!TARGET_BUILD_DIR!\rpaths.txt" > nul 2>&1 || echo !RPATH_PATHNAME!>> "!TARGET_BUILD_DIR!\rpaths.txt"
		)
	)

	rem // does it look like we need some Swift standard libraries ?
	rem //if not "_!DEBUG_ARG!_"=="__" echo Calling: otool -L "!OUTDIR!\!OUTFILE!" -arch !ARCH!^|more +1^|findstr "@rpath/libswift"
	rem //"!TOOLCHAIN_PATH!\otool.exe"                     -L "!OUTDIR!\!OUTFILE!" -arch !ARCH!|more +1|findstr "@rpath/libswift" > nul && (
	rem //	rem // if so, append /usr/lib/swift to the binary's load paths if we aren't already about to add it
	rem //	rem // since it'll be in the first position, it ensure user libraries that may begin with "libswift" will be loaded only if the corresponding system libraries don't exist
	rem //	set RPATH_PATHNAME=/usr/lib/swift
	rem //	findstr /l /c:"!RPATH_PATHNAME!" "!TARGET_BUILD_DIR!\rpaths.txt" > nul 2>&1 || (
	rem //		ren "!TARGET_BUILD_DIR!\rpaths.txt" rpaths.txt.old
	rem //		echo !RPATH_PATHNAME!> "!TARGET_BUILD_DIR!\rpaths.txt"
	rem //		type "!TARGET_BUILD_DIR!\rpaths.txt.old" >> "!TARGET_BUILD_DIR!\rpaths.txt"
	rem //		del /f /q "!TARGET_BUILD_DIR!\rpaths.txt.old" > nul 2>&1
	rem //	)
	rem //)

	rem // now fix the load paths in the main binary

	rem // list the current rpaths, back them up and remove them all
	if not "_!DEBUG_ARG!_"=="__" echo Calling: otool -l "!OUTDIR!\!OUTFILE!" -arch !ARCH!^|findstr /c:"\(offset 12\)"^|findstr "^^ path" ^> "!TEMPDIR!\candidate_rpaths.txt"
	"!TOOLCHAIN_PATH!\otool.exe"                     -l "!OUTDIR!\!OUTFILE!" -arch !ARCH! |findstr /c:"\(offset 12\)" |findstr "^^ path"  > "!TEMPDIR!\candidate_rpaths.txt"
	for /f "usebackq tokens=2 delims=( " %%a in ("!TEMPDIR!\candidate_rpaths.txt") do (
		rem // get the full pathname of this rpath
		set RPATH_PATHNAME=%%a
		rem // add it at the END of our list of final rpaths to add in the main binary
		echo !RPATH_PATHNAME!>> "!TARGET_BUILD_DIR!\rpaths.txt"
		rem // and remove it from the main binary (until we add it again in the right order)
		if not "_!DEBUG_ARG!_"=="__" echo Calling: install_name_tool -delete_rpath "!RPATH_PATHNAME!" "!OUTDIR!\!OUTFILE!"
		"!TOOLCHAIN_PATH!\install_name_tool.exe"                     -delete_rpath "!RPATH_PATHNAME!" "!OUTDIR!\!OUTFILE!"
	)
	rem // now add the rpaths that we decided we should add, in the right order this time
	for /f "usebackq delims=" %%a in ("!TARGET_BUILD_DIR!\rpaths.txt") do (
		rem // get the full pathname of this rpath
		set RPATH_PATHNAME=%%a
		rem // see if we need to add this particular rpath load command to the main binary
		if not "_!DEBUG_ARG!_"=="__" echo Calling: otool -l "!OUTDIR!\!OUTFILE!" -arch !ARCH!^|findstr /c:"\(offset 12\)"^|findstr "^^ path"^|findstr /c:"!RPATH_PATHNAME!"
		"!TOOLCHAIN_PATH!\otool.exe"                     -l "!OUTDIR!\!OUTFILE!" -arch !ARCH! |findstr /c:"\(offset 12\)" |findstr "^^ path" |findstr /c:"!RPATH_PATHNAME!" > nul 2>&1 || (
			rem // this search path was not found, add it to our main application executable
			if not "_!DEBUG_ARG!_"=="__" echo Calling: install_name_tool -add_rpath "!RPATH_PATHNAME!" "!OUTDIR!\!OUTFILE!"
			"!TOOLCHAIN_PATH!\install_name_tool.exe"                     -add_rpath "!RPATH_PATHNAME!" "!OUTDIR!\!OUTFILE!"
			if not "!errorlevel!"=="0" goto :exit_error
		)
	)

	rem // see whether other libraries/frameworks this binary depends on need to have their path changed
	if not "_!DEBUG_ARG!_"=="__" echo Calling: otool -L "!OUTDIR!\!OUTFILE!" -arch !ARCH!^|more +1^|findstr "@rpath \\" ^> "!TEMPDIR!\fixable_dependencies.txt"
	"!TOOLCHAIN_PATH!\otool.exe"                     -L "!OUTDIR!\!OUTFILE!" -arch !ARCH! |more +1 |findstr "@rpath \\"  > "!TEMPDIR!\fixable_dependencies.txt"
	for /f "usebackq tokens=1 delims=( " %%a in ("!TEMPDIR!\fixable_dependencies.txt") do (
		rem // get the full pathname to fix, and the name of the concerned library or framework without its extension
		set FIXABLE_PATHNAME=%%a
		set FIXABLE_NAME=%%~na

		rem // is it a Swift standard library ?
		if "_!FIXABLE_NAME:~0,8!_"=="_libswift_" if exist "!SDK_PATH!\usr\lib\swift\!FIXABLE_NAME!.tbd" (

			rem // if so, that's already been taken care of, as we've made sure /usr/lib/swift was in the binary load paths

		rem // else it's not a Swift standard library
		) else (
			rem // is that dependency NOT in place yet ?
			if not exist "!OUTDIR!\Frameworks\!FIXABLE_NAME!.framework\!FIXABLE_NAME!" if not exist "!OUTDIR!\Frameworks\!FIXABLE_NAME!.dylib" (
				call :fetch_dependency_or_die "!FIXABLE_NAME!"
			)

			rem // identify dependency type and relative pathname
			if exist "!OUTDIR!\Frameworks\!FIXABLE_NAME!.framework\!FIXABLE_NAME!" (
				set FIXABLE_DEPENDENCY_TYPE=framework
				set FIXABLE_DEPENDENCY_RELATIVE_PATHNAME=Frameworks\!FIXABLE_NAME!.framework\!FIXABLE_NAME!
			) else (
				set FIXABLE_DEPENDENCY_TYPE=dylib
				set FIXABLE_DEPENDENCY_RELATIVE_PATHNAME=Frameworks\!FIXABLE_NAME!.dylib
			)

			rem // and finally fix the load paths in this embedded binary
			echo  + Setting load path for !FIXABLE_NAME! !FIXABLE_DEPENDENCY_TYPE!...
			if not "_!DEBUG_ARG!_"=="__" echo Calling: install_name_tool -change "!FIXABLE_PATHNAME!" "@executable_path/!FIXABLE_DEPENDENCY_RELATIVE_PATHNAME:\=/!" "!OUTDIR!\!OUTFILE!"
			"!TOOLCHAIN_PATH!\install_name_tool.exe"                     -change "!FIXABLE_PATHNAME!" "@executable_path/!FIXABLE_DEPENDENCY_RELATIVE_PATHNAME:\=/!" "!OUTDIR!\!OUTFILE!"
			if not "!errorlevel!"=="0" goto :exit_error
		)
	)
	del /f /q "!TEMPDIR!\fixable_dependencies.txt" > nul 2>&1
	del /f /q "!TEMPDIR!\candidate_rpaths.txt" > nul 2>&1

	rem // finally, validate that all dependencies are here, recursively
	rem // FIXME: this single step should replace all the cruft above someday -- the sooner the better!
	rem // is this bundle a full-featured app ? (note: this excludes app extensions and other preference bundles)
	if "_!ENTRYPOINT!_"=="_crt_" (
		echo  + Validating dependencies...
		set VERBOSE_VALIDATEDEPS_ARG=
		if not "_!DEBUG_ARG!_"=="__" set VERBOSE_VALIDATEDEPS_ARG=-v
		rem // collect all CPP flags used and dump them to a file (to filter library/framework search paths)
		copy /y nul "!TEMPDIR!\cppflagsfile.txt" > nul 2>&1
		for /f "tokens=2* delims==" %%l in ('set^|findstr "^IOS_CPPFLAGS_"') do echo %%l >> "!TEMPDIR!\cppflagsfile.txt"
		rem // cycle through all architectures built...
		for %%a in (!ARCHS!) do (
			set ARCH=%%a
:validate_dependencies_again
			rem // process the main app's dependencies
			if not "_!DEBUG_ARG!_"=="__" echo Calling: validatedeps "!OUTDIR!\!OUTFILE!" -s "!SDK_PATH!" -a "!ARCH!" -x "!OUTDIR!\!OUTFILE!" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedLibraries" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks" -L"!SDK_PATH!\swift-5.0\!SDK_PLATFORM_NAME!" -p "!TEMPDIR!\cppflagsfile.txt" -r ^> "!TEMPDIR!\validatedeps.txt" 2^>^&1
			"!TOOLCHAIN_PATH!\validatedeps.exe"                     "!OUTDIR!\!OUTFILE!" -s "!SDK_PATH!" -a "!ARCH!" -x "!OUTDIR!\!OUTFILE!" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedLibraries" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks" -L"!SDK_PATH!\swift-5.0\!SDK_PLATFORM_NAME!" -p "!TEMPDIR!\cppflagsfile.txt" -r > "!TEMPDIR!\validatedeps.txt" 2>&1
			rem // also process the other libraries if some exist
			if exist "!OUTDIR!\Frameworks\." for /f "usebackq delims=" %%f in (`dir /b "!OUTDIR!\Frameworks\*.dylib" 2^>nul`) do (
				set LIBRARY_NAME=%%f
				if not "_!DEBUG_ARG!_"=="__" echo Calling: validatedeps "!OUTDIR!\Frameworks\!LIBRARY_NAME!" -s "!SDK_PATH!" -a "!ARCH!" -x "!OUTDIR!\!OUTFILE!" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedLibraries" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks" -L"!SDK_PATH!\swift-5.0\!SDK_PLATFORM_NAME!" -p "!TEMPDIR!\cppflagsfile.txt" -r ^>^> "!TEMPDIR!\validatedeps.txt" 2^>^&1
				"!TOOLCHAIN_PATH!\validatedeps.exe"                     "!OUTDIR!\Frameworks\!LIBRARY_NAME!" -s "!SDK_PATH!" -a "!ARCH!" -x "!OUTDIR!\!OUTFILE!" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedLibraries" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks" -L"!SDK_PATH!\swift-5.0\!SDK_PLATFORM_NAME!" -p "!TEMPDIR!\cppflagsfile.txt" -r >> "!TEMPDIR!\validatedeps.txt" 2>&1
			)
			rem // also process the other frameworks if some exist
			if exist "!OUTDIR!\Frameworks\." for /f "usebackq delims=" %%f in (`dir /b "!OUTDIR!\Frameworks\*.framework" 2^>nul`) do (
				set FRAMEWORK_NAME=%%~nf
				if not "_!DEBUG_ARG!_"=="__" echo Calling: validatedeps "!OUTDIR!\Frameworks\!FRAMEWORK_NAME!.framework\!FRAMEWORK_NAME!" -s "!SDK_PATH!" -a "!ARCH!" -x "!OUTDIR!\!OUTFILE!" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedLibraries" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks" -L"!SDK_PATH!\swift-5.0\!SDK_PLATFORM_NAME!" -p "!TEMPDIR!\cppflagsfile.txt" -r ^>^> "!TEMPDIR!\validatedeps.txt" 2^>^&1
				"!TOOLCHAIN_PATH!\validatedeps.exe"                     "!OUTDIR!\Frameworks\!FRAMEWORK_NAME!.framework\!FRAMEWORK_NAME!" -s "!SDK_PATH!" -a "!ARCH!" -x "!OUTDIR!\!OUTFILE!" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedLibraries" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks" -L"!SDK_PATH!\swift-5.0\!SDK_PLATFORM_NAME!" -p "!TEMPDIR!\cppflagsfile.txt" -r >> "!TEMPDIR!\validatedeps.txt" 2>&1
			)
			rem // DO NOTE: errorlevels don't work after redirections
			set NOTEMPTY=0
			for /f %%i in ("!TEMPDIR!\validatedeps.txt") do set NOTEMPTY=%%~zi
			if !NOTEMPTY! gtr 0  (
				set HAS_INSTALLED_SOME=false
				rem // some dependencies are not installed yet. See if we can install them, and run the tool again
				findstr " possibly located at: " "!TEMPDIR!\validatedeps.txt" > "!TEMPDIR!\candidate_dependencies.txt"
				findstr "                  in: " "!TEMPDIR!\validatedeps.txt" > "!TEMPDIR!\candidate_callingbinaries.txt"
				set DEPENDENCY_NUMBER=0
				for /f "usebackq delims=" %%i in ("!TEMPDIR!\candidate_dependencies.txt") do (
					set DEPENDENCY_PATH=%%i
					set DEPENDENCY_PATH=!DEPENDENCY_PATH:~22!
					for %%F in ("!DEPENDENCY_PATH!") do set DEPENDENCY_NAME=%%~nxF
					if not "_!DEPENDENCY_PATH:.framework=!_"=="_!DEPENDENCY_PATH!_" for %%F in ("!DEPENDENCY_PATH!") do (
						rem // if dependency points to a framework binary, have the framework directory path instead (and chop off the trailing backslash)
						set DEPENDENCY_PATH=%%~dpF
						set DEPENDENCY_PATH=!DEPENDENCY_PATH:~0,-1!
					)
					more +!DEPENDENCY_NUMBER! "!TEMPDIR!\candidate_callingbinaries.txt" > %TO_TEMPVAR%
					set CALLING_BINARY=!TEMPVAR:~22!
					set /a DEPENDENCY_NUMBER=!DEPENDENCY_NUMBER!+1
					rem // do not bother embedding Swift libraries (every Apple platform has them now), but do embed those that aren't in the Frameworks directory yet
					if not exist "!SDK_PATH!\swift-5.0\!SDK_PLATFORM_NAME!\!DEPENDENCY_NAME!" (
						if not exist "!OUTDIR!\Frameworks\!DEPENDENCY_NAME!" if not exist "!OUTDIR!\Frameworks\!DEPENDENCY_NAME!.framework\!DEPENDENCY_NAME!" (
							call :embed_binary "!DEPENDENCY_PATH!" in "!CALLING_BINARY!"
							if "_!RETVAL!_"=="_0_" set HAS_INSTALLED_SOME=true
						)
					)
				)
				if "_!HAS_INSTALLED_SOME!_"=="_true_" goto :validate_dependencies_again
				rem // now run the tool again and display its output to the terminal -- we should have zero unresolved dependency, else it's an error
				if not "_!DEBUG_ARG!_"=="__" echo Calling: validatedeps "!OUTDIR!\!OUTFILE!" -s "!SDK_PATH!" -a "!ARCH!" -x "!OUTDIR!\!OUTFILE!" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedLibraries" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks" -L"!SDK_PATH!\swift-5.0\!SDK_PLATFORM_NAME!" -p "!TEMPDIR!\cppflagsfile.txt" -r !VERBOSE_VALIDATEDEPS_ARG!
				"!TOOLCHAIN_PATH!\validatedeps.exe"                     "!OUTDIR!\!OUTFILE!" -s "!SDK_PATH!" -a "!ARCH!" -x "!OUTDIR!\!OUTFILE!" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedLibraries" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks" -L"!SDK_PATH!\swift-5.0\!SDK_PLATFORM_NAME!" -p "!TEMPDIR!\cppflagsfile.txt" -r !VERBOSE_VALIDATEDEPS_ARG!
				if errorlevel 1 echo Error: unresolved dependencies in !OUTFILE! for arch !ARCH!. This app will crash on startup if not all the library and framework dependencies are present.& goto :exit_error
				rem // also process the other libraries if some exist
				if exist "!OUTDIR!\Frameworks\." for /f "usebackq delims=" %%f in (`dir /b "!OUTDIR!\Frameworks\*.dylib" 2^>nul`) do (
					set LIBRARY_NAME=%%f
					if not "_!DEBUG_ARG!_"=="__" echo Calling: validatedeps "!OUTDIR!\Frameworks\!LIBRARY_NAME!" -s "!SDK_PATH!" -a "!ARCH!" -x "!OUTDIR!\!OUTFILE!" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedLibraries" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks" -L"!SDK_PATH!\swift-5.0\!SDK_PLATFORM_NAME!" -p "!TEMPDIR!\cppflagsfile.txt" -r !VERBOSE_VALIDATEDEPS_ARG!
					"!TOOLCHAIN_PATH!\validatedeps.exe"                     "!OUTDIR!\Frameworks\!LIBRARY_NAME!" -s "!SDK_PATH!" -a "!ARCH!" -x "!OUTDIR!\!OUTFILE!" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedLibraries" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks" -L"!SDK_PATH!\swift-5.0\!SDK_PLATFORM_NAME!" -p "!TEMPDIR!\cppflagsfile.txt" -r !VERBOSE_VALIDATEDEPS_ARG!
					if errorlevel 1 echo Error: unresolved dependencies in !OUTFILE!\Frameworks\!LIBRARY_NAME! for arch !ARCH!. This app will crash on startup if not all the library and framework dependencies are present.& goto :exit_error
				)
				rem // also process the other frameworks if some exist
				if exist "!OUTDIR!\Frameworks\." for /f "usebackq delims=" %%f in (`dir /b "!OUTDIR!\Frameworks\*.framework" 2^>nul`) do (
					set FRAMEWORK_NAME=%%~nf
					if not "_!DEBUG_ARG!_"=="__" echo Calling: validatedeps "!OUTDIR!\Frameworks\!FRAMEWORK_NAME!.framework\!FRAMEWORK_NAME!" -s "!SDK_PATH!" -a "!ARCH!" -x "!OUTDIR!\!OUTFILE!" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedLibraries" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks" -L"!SDK_PATH!\swift-5.0\!SDK_PLATFORM_NAME!" -p "!TEMPDIR!\cppflagsfile.txt" -r !VERBOSE_VALIDATEDEPS_ARG!
					"!TOOLCHAIN_PATH!\validatedeps.exe"                     "!OUTDIR!\Frameworks\!FRAMEWORK_NAME!.framework\!FRAMEWORK_NAME!" -s "!SDK_PATH!" -a "!ARCH!" -x "!OUTDIR!\!OUTFILE!" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedLibraries" -L"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltLibraries" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\SuppliedFrameworks" -F"!BUILD_DIR!\!CONFIGURATION!-!SDK_PLATFORM_NAME!\BuiltFrameworks" -L"!SDK_PATH!\swift-5.0\!SDK_PLATFORM_NAME!" -p "!TEMPDIR!\cppflagsfile.txt" -r !VERBOSE_VALIDATEDEPS_ARG!
					if errorlevel 1 echo Error: unresolved dependencies in !OUTFILE!\Frameworks\!FRAMEWORK_NAME!.framework\!FRAMEWORK_NAME! for arch !ARCH!. This app will crash on startup if not all the library and framework dependencies are present.& goto :exit_error
				)
			)
		)
		del /f /q "!TEMPDIR!\cppflagsfile.txt" > nul 2>&1
		set VERBOSE_VALIDATEDEPS_ARG=
	)
)


rem ////////////////////////////////
rem // STEP 5. PRE-PACKAGE SCRIPT //
rem ////////////////////////////////


rem // do we have a pre-package script to run ?
if not "_!PREPKG_SCRIPT!_"=="__" (
	echo  + Running pre-packaging script...
	if not "_!DEBUG_ARG!_"=="__" echo Calling: "!PREPKG_SCRIPT!"
	call "!PREPKG_SCRIPT!"
)


rem //////////////////////////
rem // STEP 6. CODE SIGNING //
rem //////////////////////////


rem // are we NOT building a static library ?
if not "_!ENTRYPOINT!_"=="_lib_" (
	rem // if a provisioning profile has been specified, embed it
	if not "_!PROVISION!_"=="__" (
		echo  + Embedding provisioning profile...
		copy /y /b "!PROVISION!" "!OUTDIR!\embedded.mobileprovision" > nul 2>&1
		if not "!errorlevel!"=="0" goto :exit_error
	)

	rem // tell what we're about to do (only warn about pseudo-signatures when we're building a top-level project)
	if "_!IS_BUILDING_DEPENDENCY!_"=="__" (
		if "_!IDENTITY!_"=="__" echo  + Pseudo-signing code ^(no identity supplied, jailbreak only^)...
		if "_!IDENTITY!_"=="_::_" echo  + Pseudo-signing code ^(no identity supplied, jailbreak only^)...
	)
	if not "_!IDENTITY!_"=="__" if not "_!IDENTITY!_"=="_::_" (
		if not "_!SIGNER!_"=="__" ( echo  + Signing code as !SIGNER! with team ID !TEAMID!...
		) else                    ( echo  + Signing code with digital identity...
		)
	)

	rem // if we have an extra entitlements .plist, prepare a non-empty entitlements flag
	set ENTITLEMENTS_FLAG=
	if not "_!ENTITLEMENTS_PLIST!_"=="__" (
		rem // if this project is a Xcode project, expand the variables in the entitlements PList, else just copy it as is
		if exist "!XCNAME!.xcodeproj\project.pbxproj" (
			rem // use the Xcode project file AND the Info.plist (if we have one) to resolve variables, and inject the TeamIdentifierPrefix variable value too
			set VARFILE_FLAG=
			if exist "!OUTDIR!\Info.plist" set VARFILE_FLAG=-varfile "!OUTDIR!\Info.plist"
			"!TOOLCHAIN_PATH!\pbxprojparse.exe" "!XCNAME!.xcodeproj" SDKROOT= BUILD_DIR="!BUILD_DIR:\=/!" EXECUTABLE_NAME="!EXECUTABLE_NAME!" -target "!TARGET_NAME!" -configuration "!CONFIGURATION!" -expandvars "!ENTITLEMENTS_PLIST!" !VARFILE_FLAG! TeamIdentifierPrefix=!TEAMID!. > "!ENTITLEMENTS_PLIST!.final" 2> "!TEMPDIR!\unresolved-variables.txt"
			set ERRORLVL=!errorlevel!
			for /f "usebackq delims=" %%a in ("!TEMPDIR!\unresolved-variables.txt") do echo %%a | findstr /v /i "output.txt \$\{EXECUTABLE_NAME\}"
		) else (
			copy /y /b "!ENTITLEMENTS_PLIST!" "!ENTITLEMENTS_PLIST!.final" > nul
			set ERRORLVL=!errorlevel!
		)
		if not "_!ERRORLVL!_"=="_0_" goto :exit_error
		set ERRORLVL=
		set ENTITLEMENTS_FLAG=+!ENTITLEMENTS_PLIST!.final
	)

	rem // files ending in .meta cause the code signatures to be invalid, even with macOS codesign, for some elusive reason,
	rem // so strip them out if some are detected
	set OLDDIR=!CD!
	cd "!OUTDIR!"
	rem // note that we need to call findstr to filter out .meta PRECISELY else dir would also return files ending in .metaXXX
	for /f "usebackq delims=" %%f in (`dir /s /b *.meta 2^>nul^|findstr "\.meta$"`) do (
		rem // echo Notice: deleting ancilliary .meta file before code signature: %%f
		del /q "%%f" > nul
	)
	rem // also remove all PkgInfo files except the one at the root of this bundle
	rem // note that !CD! here MUST be surrounded by bangs else it would point to OLDDIR!
	for /f "usebackq delims=" %%f in (`dir /s /b PkgInfo 2^>nul`) do if not "%%f"=="!CD!\PkgInfo" (
		rem // echo Notice: deleting extra PkgInfo file: %%f
		del /q "%%f" > nul
	)
	cd "!OLDDIR!"

	rem // perform the code signature (and check whether we succeeded). Note that in case of a pseudo-signature, an arbitrary
	rem // team ID flag (constructed earlier) is passed in the invokation parameters to replace the team ID from the missing certificate.
	if not "_!DEBUG_ARG!_"=="__" echo Calling: ldid -k"!KEYCHAIN_PATH!" -S"!IDENTITY!" -t!TEAMID! "-E!ENTITLEMENTS_FLAG!" "!OUTDIR!\!OUTFILE!"
	"!TOOLCHAIN_PATH!\ldid.exe"                     -k"!KEYCHAIN_PATH!" -S"!IDENTITY!" -t!TEAMID! "-E!ENTITLEMENTS_FLAG!" "!OUTDIR!\!OUTFILE!"
	if not "!errorlevel!"=="0" ( del /f /q "!ENTITLEMENTS_PLIST!.final" > nul 2>&1 & goto :exit_error )
	del /f /q "!ENTITLEMENTS_PLIST!.final" > nul 2>&1
)


rem ///////////////////////
rem // STEP 7. BUILD IPA //
rem ///////////////////////


if not "_!WANT_IPA!_"=="__" (
	echo  + Building iOS package...
	mkdir "Packages" > nul 2>&1
	del /f /q "Packages\!NAME!.ipa" > nul 2>&1

	if not "_!DEBUG_ARG!_"=="__" echo Calling: makeipa "Packages\!NAME!.ipa" "!OUTDIR!"
	"!TOOLCHAIN_PATH!\makeipa.exe"                     "Packages\!NAME!.ipa" "!OUTDIR!"
	if not "!errorlevel!"=="0" goto :exit_error

	rem // if a .dSYM directory exists aside this .ipa, copy it too for the user to notice
	if exist "!OUTDIR!.dSYM\." (
		rem // copy the .dSYM directory in place recursively
		rem // NOTE: when copying a DIRECTORY, the copy would fail if the target directory already exists as a FILE, so make sure the target doesn't exist in any form first
		rmdir /s /q "!PACKAGES!\!NAME!.dSYM" > nul 2>&1
		del /f /q "!PACKAGES!\!NAME!.dSYM" > nul 2>&1
		robocopy "!OUTDIR!.dSYM" "!PACKAGES!\!NAME!.dSYM" /is /e /njh /ndl /nfl /njs > nul 2>&1
		if not "!errorlevel!"=="1" goto :exit_error
		rem // NOTE: robocopy exits with errorlevel ***1*** on success! Microsofticly absurd, but that's the way it is.
	)
)


rem ////////////////////
rem // STEP 8. DEPLOY //
rem ////////////////////


if not "_!WANT_DEPLOY!_"=="__" (
	echo  + Deploying to device...
	if "_!IDENTITY!_"=="__" echo WARNING: deploying a pseudo-signed ipa. Jailbreak + AppSync are mandatory^!
	if "_!IDENTITY!_"=="_::_" echo WARNING: deploying a pseudo-signed ipa. Jailbreak + AppSync are mandatory^!
	rem // WARNING: no quotes around WANT_DEPLOY because this variable may contain several args and we want to pass them individually
	if not "_!DEBUG_ARG!_"=="__" echo Calling: ideployota "Packages\!NAME!.ipa" !WANT_DEPLOY!
	"!TOOLCHAIN_PATH!\ideployota.exe"                     "Packages\!NAME!.ipa" !WANT_DEPLOY!
	if not "!errorlevel!"=="0" goto :exit_error
)


rem ///////////////////////
rem // STEP 9. BUILD DEB //
rem ///////////////////////


if not "_!WANT_DEB!_"=="__" (
	echo  + Building Cydia package...
	mkdir "Packages" > nul 2>&1
	mkdir "!NAME!\DEBIAN" > nul 2>&1
	mkdir "!NAME!\Applications" > nul 2>&1
	del /f /q "Packages\!NAME!.deb" > nul 2>&1
	rmdir /s /q "!NAME!\Applications\!OUTDIR!" > nul 2>&1
	move /y "!OUTDIR!" "!NAME!\Applications" > nul 2>&1
	if not "!errorlevel!"=="0" goto :exit_error
	rem // default fallbacks for bundle identifier, display name and build number
	if "_!BUNDLE_IDENTIFIER!_"=="__" set BUNDLE_IDENTIFIER=com.pmbaty.iosbuildenv.!NAME!
	if "_!BUNDLE_DISPLAYNAME!_"=="__" set BUNDLE_DISPLAYNAME=!NAME!
	if "_!BUILD_NUMBER!_"=="__" set BUILD_NUMBER=1.0
	rem // use our own echo here to not end lines with CRLF...
	"!TOOLCHAIN_PATH!\echo.exe" "Package: !BUNDLE_IDENTIFIER!" > "!NAME!\DEBIAN\control"
	"!TOOLCHAIN_PATH!\echo.exe" "Name: !BUNDLE_DISPLAYNAME!" >> "!NAME!\DEBIAN\control"
	"!TOOLCHAIN_PATH!\echo.exe" "Version: !BUILD_NUMBER!" >> "!NAME!\DEBIAN\control"
	"!TOOLCHAIN_PATH!\echo.exe" "Priority: optional" >> "!NAME!\DEBIAN\control"
	"!TOOLCHAIN_PATH!\echo.exe" "Architecture: !SDK_PLATFORM_NAME!-arm" >> "!NAME!\DEBIAN\control"
	"!TOOLCHAIN_PATH!\cat.exe" "cydia-package.cfg" >> "!NAME!\DEBIAN\control"
	"!TOOLCHAIN_PATH!\echo.exe" "Icon: file:///Applications/!OUTDIR!/icon.png" >> "!NAME!\DEBIAN\control"
	"!TOOLCHAIN_PATH!\makedeb.exe" "!NAME!"
	set RETVAL=!errorlevel!
	move /y "!NAME!.deb" "Packages" > nul 2>&1
	move /y "!NAME!\Applications\!OUTDIR!" . > nul 2>&1
	rmdir /s /q "!NAME!" > nul 2>&1
	if not "!RETVAL!"=="0" goto :exit_error
)


rem /////////////////////
rem // STEP 10. FINISH //
rem /////////////////////


echo  + Done.
echo.
if not "_!BUILDSTART_TIME!_"=="__" (
	rem // compute and display total build time, and save it to a file in the build directory
	set BUILDSTOP_TIME=%TIME%
	for /f "tokens=1-4 delims=:.," %%a in ("!BUILDSTART_TIME!") do set BUILDSTART_H=%%a& set /a BUILDSTART_M=100%%b %% 100& set /a BUILDSTART_S=100%%c %% 100&set /a BUILDSTART_L=100%%d %% 100
	for /f "tokens=1-4 delims=:.," %%a in ("!BUILDSTOP_TIME!")  do set  BUILDSTOP_H=%%a& set /a  BUILDSTOP_M=100%%b %% 100& set /a  BUILDSTOP_S=100%%c %% 100&set /a  BUILDSTOP_L=100%%d %% 100
	set /a BUILDTIME_H=!BUILDSTOP_H!-!BUILDSTART_H!
	set /a BUILDTIME_M=!BUILDSTOP_M!-!BUILDSTART_M!
	set /a BUILDTIME_S=!BUILDSTOP_S!-!BUILDSTART_S!
	set /a BUILDTIME_L=!BUILDSTOP_L!-!BUILDSTART_L!
	if !BUILDTIME_L! lss 0 set /a BUILDTIME_S=!BUILDTIME_S!-1
	if !BUILDTIME_S! lss 0 set /a BUILDTIME_M=!BUILDTIME_M!-1& set /a BUILDTIME_S=60!BUILDTIME_S!
	if !BUILDTIME_M! lss 0 set /a BUILDTIME_H=!BUILDTIME_H!-1& set /a BUILDTIME_M=60!BUILDTIME_M!
	if !BUILDTIME_H! lss 0 set /a BUILDTIME_H=24!BUILDTIME_H!
	set /a BUILDTIME=!BUILDTIME_H!*3600+!BUILDTIME_M!*60+!BUILDTIME_S!
	echo On %DATE% %TIME% project %NAME% was built in !BUILDTIME! seconds ^(!BUILDTIME_H! h !BUILDTIME_M! min !BUILDTIME_S! s^)>> buildtimes.log
	echo Build time: !BUILDTIME! seconds ^(!BUILDTIME_H! h !BUILDTIME_M! min !BUILDTIME_S! s^)
)
echo Output directory: !OUTDIR!
if not "_!WANT_IPA!_"=="__" echo IPA package:      Packages\!NAME!.ipa
if not "_!WANT_DEB!_"=="__" echo Cydia package:    Packages\!NAME!.deb

rem // see what sort of code signer we have; if we have the demo code signer, display the demo message
"!TOOLCHAIN_PATH!\ldid.exe" -v 2>&1 | findstr "passphrase" > %TO_TEMPVAR%
if "_!TEMPVAR!_"=="__" (
	echo.
	echo LIMITATIONS OF THIS DEMO VERSION:
	echo - Apps built with this version will periodically display a popup message.
	echo - Deployment on non-jailbroken devices is supported only in the full version.
	echo.
	echo To get rid of these limitations, please obtain the full version. Thank you^!
)

goto exit_success

:exit_sdk_missing
echo.
echo In order to use this builder, you need to copy some files from your iOS SDK
echo from the Mac side of your computer to the Windows side.
echo.
echo Please follow the instructions given in the readme file. This task is as
echo simple as opening a file and only requires to be done once.
echo.
call :open_url "%SCRIPT_DIRECTORY%readme.html"

:exit_error
if not "_!DEBUG_ARG!_"=="__" (
	echo :exit_error called. Check possible error messages above this line ^^^^^^
	echo ========================= CURRENT WORKING DIRECTORY =========================
	cd
	echo ========================== ENVIRONMENT DUMP BEGIN ===========================
	set
	echo ========================== END OF ENVIRONMENT DUMP ==========================
)
rmdir /s /q "!TEMPDIR!" > nul 2>&1
endlocal
exit /b 1

:exit_success
:end
rmdir /s /q "!TEMPDIR!" > nul 2>&1
endlocal
exit /b 0

:embed_binary
	rem // subroutine that copies an embedded binary or framework to its final location
	rem // usage: call :embed_binary DEPENDENCY_PATHNAME in CALLINGBINARY_PATHNAME
	rem // ex:    call :embed_binary path/to/SomeFramework.framework in path/to/main/app
	rem // ex:    call :embed_binary path/to/libSomeLibrary.dylib in path/to/main/app/Frameworks/SomeFramework.framework/SomeFramework

	rem // collect 1st argument: dependency pathname
	rem // strip quotes from item
	set _EMBEDDED_BINARY=###%1###
	set _EMBEDDED_BINARY=!_EMBEDDED_BINARY:"###=!
	set _EMBEDDED_BINARY=!_EMBEDDED_BINARY:###"=!
	if not "_!_EMBEDDED_BINARY!_"=="__" set _EMBEDDED_BINARY=!_EMBEDDED_BINARY:###=!
	rem // as we're running from a function, a double dequote is necessary
	set _EMBEDDED_BINARY=###!_EMBEDDED_BINARY!###
	set _EMBEDDED_BINARY=!_EMBEDDED_BINARY:"###=!
	set _EMBEDDED_BINARY=!_EMBEDDED_BINARY:###"=!
	if not "_!_EMBEDDED_BINARY!_"=="__" set _EMBEDDED_BINARY=!_EMBEDDED_BINARY:###=!
	rem // convert forward slashes to backslashes
	if not "_!_EMBEDDED_BINARY!_"=="__" set _EMBEDDED_BINARY=!_EMBEDDED_BINARY:/=\!

	rem // ignore 2nd argument ("in")

	rem // collect 3rd argument: calling binary pathname
	rem // strip quotes from item
	set _CALLING_BINARY=###%3###
	set _CALLING_BINARY=!_CALLING_BINARY:"###=!
	set _CALLING_BINARY=!_CALLING_BINARY:###"=!
	if not "_!_CALLING_BINARY!_"=="__" set _CALLING_BINARY=!_CALLING_BINARY:###=!
	rem // as we're running from a function, a double dequote is necessary
	set _CALLING_BINARY=###!_CALLING_BINARY!###
	set _CALLING_BINARY=!_CALLING_BINARY:"###=!
	set _CALLING_BINARY=!_CALLING_BINARY:###"=!
	if not "_!_CALLING_BINARY!_"=="__" set _CALLING_BINARY=!_CALLING_BINARY:###=!
	rem // convert forward slashes to backslashes
	if not "_!_CALLING_BINARY!_"=="__" set _CALLING_BINARY=!_CALLING_BINARY:/=\!

	rem echo :embed_binary called for: [!_EMBEDDED_BINARY!]

	rem // see if the passed pathname is a symbolic link, in which case dereference it
	(
		set _DEREFERENCED_PATHNAME=& set /p _DEREFERENCED_PATHNAME=< "!_EMBEDDED_BINARY!"
		if exist "!_DEREFERENCED_PATHNAME!" set _EMBEDDED_BINARY=!_DEREFERENCED_PATHNAME!
	) > nul 2>&1

	rem // get the embedded binary extension (if any)
	for %%a in (!_EMBEDDED_BINARY!) do set _EMBEDDED_BINARY_TYPE=%%~xa

	rem // assume we will not succeed until told otherwise
	set RETVAL=1

	rem // is it an app extension, i.e. does the extension end in .appex ?
	if "_!_EMBEDDED_BINARY_TYPE!_"=="_.appex_" (

		rem // guess the app extensions's path and name from its pathname
		set _EMBEDDED_APPEX_PATH=!_EMBEDDED_BINARY!
		for %%a in (!_EMBEDDED_APPEX_PATH!) do set _EMBEDDED_APPEX_NAME=%%~na

		rem // this is an app extension, let's embed it
		echo  + Embedding app extension: !_EMBEDDED_APPEX_NAME!.appex...

		rem // get the app extension's binary name from its Info.plist
		"!TOOLCHAIN_PATH!\plconvert.exe" "!_EMBEDDED_APPEX_PATH!\Info.plist" -query CFBundleExecutable > %TO_TEMPVAR%
		set _EMBEDDED_APPEX_BINARYNAME=!TEMPVAR!

		rem // create the app extension directory in the final app if it doesn't exist yet
		mkdir "!OUTDIR!\PlugIns" > nul 2>&1

		rem // copy the app extension's files (including the Info.plist) but the main binary in place
		rem // NOTE: when copying a DIRECTORY, the copy would fail if the target directory already exists as a FILE, so make sure the target doesn't exist in any form first
		rem // NOTE: robocopy exits with errorlevel ***1*** on success! Microsofticly absurd, but that's the way it is.
		rmdir /s /q "!OUTDIR!\PlugIns\!_EMBEDDED_APPEX_NAME!.appex" > nul 2>&1
		del /f /q "!OUTDIR!\PlugIns\!_EMBEDDED_APPEX_NAME!.appex" > nul 2>&1
		robocopy "!_EMBEDDED_APPEX_PATH!" "!OUTDIR!\PlugIns\!_EMBEDDED_APPEX_NAME!.appex" /is /e /njh /ndl /nfl /njs > nul
		if not "!errorlevel!"=="1" goto :exit_error
		del /f /q "!OUTDIR!\PlugIns\!_EMBEDDED_APPEX_NAME!.appex\!_EMBEDDED_APPEX_BINARYNAME!" > nul 2>&1

		rem // see if the binary to embed is fat, in which case strip it from the unnecessary arches, else simply copy it in place
		"!TOOLCHAIN_PATH!\lipo.exe" -info "!_EMBEDDED_APPEX_PATH!\!_EMBEDDED_APPEX_BINARYNAME!" > %TO_TEMPVAR%
		if not "_!TEMPVAR:~0,7!_"=="_Non-fat_" (
			rem // extract only the archs we need so as to merge them in the app extension binary, then merge them all in place
			set _LIPO_FLAGS=
			for %%a in (!ARCHS!) do (
				if not "_!DEBUG_ARG!_"=="__" echo Calling: lipo "!_EMBEDDED_APPEX_PATH!\!_EMBEDDED_APPEX_BINARYNAME!" -thin %%a -output "!TEMPDIR!\embedded-binary.%%a"
				"!TOOLCHAIN_PATH!\lipo.exe"                     "!_EMBEDDED_APPEX_PATH!\!_EMBEDDED_APPEX_BINARYNAME!" -thin %%a -output "!TEMPDIR!\embedded-binary.%%a"
				if not "!errorlevel!"=="0" goto :exit_error
				set _LIPO_FLAGS=!_LIPO_FLAGS! -arch %%a "!TEMPDIR!\embedded-binary.%%a"
			)
			if not "_!DEBUG_ARG!_"=="__" echo Calling: lipo !_LIPO_FLAGS! -output "!OUTDIR!\PlugIns\!_EMBEDDED_APPEX_NAME!.appex\!_EMBEDDED_APPEX_BINARYNAME!" -create
			"!TOOLCHAIN_PATH!\lipo.exe"                     !_LIPO_FLAGS! -output "!OUTDIR!\PlugIns\!_EMBEDDED_APPEX_NAME!.appex\!_EMBEDDED_APPEX_BINARYNAME!" -create
			if not "!errorlevel!"=="0" goto :exit_error
			del /f /q "!TEMPDIR!\embedded-binary."*
		) else (
			copy /y /b "!_EMBEDDED_APPEX_PATH!\!_EMBEDDED_APPEX_BINARYNAME!" "!OUTDIR!\PlugIns\!_EMBEDDED_APPEX_NAME!.appex\!_EMBEDDED_APPEX_BINARYNAME!" > nul 2>&1
		)

		rem // (optional) strip the embedded app extension to reduce code size
		if "_!WANT_STRIP!_"=="_true_" (
			if "_!REMOTELOG_IP!_"=="__" (
				rem // first pass: normal symbols stripping. Remove all local (unexported) symbols (-x), leaving only global (exported) symbols
				rem if "_!WANT_DSYM!_"=="_true_" "!TOOLCHAIN_PATH!\dsymutil.exe" "!OUTDIR!\PlugIns\!_EMBEDDED_APPEX_NAME!.appex\!_EMBEDDED_APPEX_BINARYNAME!" -o "!OUTDIR!\PlugIns\!_EMBEDDED_APPEX_NAME!.appex.dSYM"
				if not "_!DEBUG_ARG!_"=="__" echo Calling: strip -x "!OUTDIR!\PlugIns\!_EMBEDDED_APPEX_NAME!.appex\!_EMBEDDED_APPEX_BINARYNAME!"
				"!TOOLCHAIN_PATH!\strip.exe"                     -x "!OUTDIR!\PlugIns\!_EMBEDDED_APPEX_NAME!.appex\!_EMBEDDED_APPEX_BINARYNAME!" > nul 2>&1
				rem // DO NOT test for errors after strip, as the binary may contain no strippable symbols.
			)
			if "_!WANT_STRIP_BITCODE!_"=="_true_" (
				rem // second pass: bitcode stripping
				if not "_!DEBUG_ARG!_"=="__" echo Calling: bitcode_strip -r "!OUTDIR!\PlugIns\!_EMBEDDED_APPEX_NAME!.appex\!_EMBEDDED_APPEX_BINARYNAME!" -o "!OUTDIR!\PlugIns\!_EMBEDDED_APPEX_NAME!.appex\!_EMBEDDED_APPEX_BINARYNAME!"
				"!TOOLCHAIN_PATH!\bitcode_strip.exe"                     -r "!OUTDIR!\PlugIns\!_EMBEDDED_APPEX_NAME!.appex\!_EMBEDDED_APPEX_BINARYNAME!" -o "!OUTDIR!\PlugIns\!_EMBEDDED_APPEX_NAME!.appex\!_EMBEDDED_APPEX_BINARYNAME!"
				if not "!errorlevel!"=="0" goto :exit_error
			)
		)

		rem // signal our caller that we succeeded
		set RETVAL=0

	rem // else is it a framework, i.e. does the extension end in .framework ?
	) else if "_!_EMBEDDED_BINARY_TYPE!_"=="_.framework_" (

		rem // guess the framework's path and name from its pathname
		set _EMBEDDED_FRAMEWORK_PATH=!_EMBEDDED_BINARY!
		for %%a in (!_EMBEDDED_FRAMEWORK_PATH!) do set _EMBEDDED_FRAMEWORK_NAME=%%~na

		rem // get the framework's binary name from its Info.plist
		"!TOOLCHAIN_PATH!\plconvert.exe" "!_EMBEDDED_FRAMEWORK_PATH!\Info.plist" -query CFBundleExecutable > %TO_TEMPVAR%
		set _EMBEDDED_FRAMEWORK_BINARYNAME=!TEMPVAR!

		rem // make sure the binary is a DYNAMIC framework (and not a static one)
		findstr "\^!\<arch\>" "!_EMBEDDED_FRAMEWORK_PATH!\!_EMBEDDED_FRAMEWORK_BINARYNAME!" > nul 2>&1 && (
			rem // this is a STATIC framework. Notify the user that we can't embed that
			echo WARNING: ignoring static framework "!_EMBEDDED_FRAMEWORK_PATH!" while looking for dynamically-loadable binary to embed.
		) || (
			rem // this framework is a dynamic one, let's embed it
			echo  + Embedding framework: !_EMBEDDED_FRAMEWORK_NAME!.framework...

			rem // create the framework directory in the final app if it doesn't exist yet
			mkdir "!OUTDIR!\Frameworks" > nul 2>&1

			rem // see if we need to add this particular rpath load command to the calling binary
			if not "_!DEBUG_ARG!_"=="__" echo Calling: otool -l "!_CALLING_BINARY!" -arch !ARCH!^|findstr /c:"\(offset 12\)"^|findstr "^^ path"^|findstr /c:"@executable_path/Frameworks"
			"!TOOLCHAIN_PATH!\otool.exe"                     -l "!_CALLING_BINARY!" -arch !ARCH! |findstr /c:"\(offset 12\)" |findstr "^^ path" |findstr /c:"@executable_path/Frameworks" > nul 2>&1 || (
				rem // this search path was not found, add it to the calling binary
				if not "_!DEBUG_ARG!_"=="__" echo Calling: install_name_tool -add_rpath "@executable_path/Frameworks" "!_CALLING_BINARY!"
				"!TOOLCHAIN_PATH!\install_name_tool.exe"                     -add_rpath "@executable_path/Frameworks" "!_CALLING_BINARY!"
				if not "!errorlevel!"=="0" goto :exit_error
			)

			rem // copy the framework's files (including the Info.plist) but the main binary in place
			rem // NOTE: when copying a DIRECTORY, the copy would fail if the target directory already exists as a FILE, so make sure the target doesn't exist in any form first
			rem // NOTE: robocopy exits with errorlevel ***1*** on success! Microsofticly absurd, but that's the way it is.
			rmdir /s /q "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework" > nul 2>&1
			del /f /q "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework" > nul 2>&1
			robocopy "!_EMBEDDED_FRAMEWORK_PATH!" "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework" /is /e /njh /ndl /nfl /njs > nul 2>&1
			if not "!errorlevel!"=="1" goto :exit_error
			del /f /q "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\!_EMBEDDED_FRAMEWORK_BINARYNAME!" > nul 2>&1
			rmdir /s /q "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\Headers" > nul 2>&1
			rmdir /s /q "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\Modules" > nul 2>&1

			rem // does this framework bundle contain extra libraries ? if so, move them to the appropriate Frameworks directory at the bottom level if they don't exist already
			if exist "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\Libraries\." (
				for /f "usebackq delims=" %%a in (`dir /b "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\Libraries\*" 2^>nul`) do if not exist "!OUTDIR!\Frameworks\%%a" (
					echo    + Relocating dependent library: %%a
					move /y "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\Libraries\%%a" "!OUTDIR!\Frameworks\" > nul
				)
			) & rmdir /s /q "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\Libraries" > nul 2>&1

			rem // does this framework bundle contain extra frameworks ? if so, move them to the appropriate Frameworks directory at the bottom level if they don't exist already
			if exist "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\Frameworks\." (
				for /f "usebackq delims=" %%a in (`dir /b "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\Frameworks\*" 2^>nul`) do if not exist "!OUTDIR!\Frameworks\%%a\." (
					echo    + Relocating dependent framework: %%a
					move /y "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\Frameworks\%%a" "!OUTDIR!\Frameworks\" > nul
				)
			) & rmdir /s /q "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\Frameworks" > nul 2>&1

			rem // see if the framework to embed is fat, in which case strip it from the unnecessary arches, else simply copy it in place
			"!TOOLCHAIN_PATH!\lipo.exe" -info "!_EMBEDDED_FRAMEWORK_PATH!\!_EMBEDDED_FRAMEWORK_BINARYNAME!" > %TO_TEMPVAR%
			if not "_!TEMPVAR:~0,7!_"=="_Non-fat_" (
				rem // extract only the archs we need so as to merge them in the framework binary, then merge them all in place
				set _LIPO_FLAGS=
				for %%a in (!ARCHS!) do (
					if not "_!DEBUG_ARG!_"=="__" echo Calling: lipo "!_EMBEDDED_FRAMEWORK_PATH!\!_EMBEDDED_FRAMEWORK_BINARYNAME!" -thin %%a -output "!TEMPDIR!\embedded-binary.%%a"
					"!TOOLCHAIN_PATH!\lipo.exe"                     "!_EMBEDDED_FRAMEWORK_PATH!\!_EMBEDDED_FRAMEWORK_BINARYNAME!" -thin %%a -output "!TEMPDIR!\embedded-binary.%%a"
					if not "!errorlevel!"=="0" pause &goto :exit_error
					set _LIPO_FLAGS=!_LIPO_FLAGS! -arch %%a "!TEMPDIR!\embedded-binary.%%a"
				)
				if not "_!DEBUG_ARG!_"=="__" echo Calling: lipo !_LIPO_FLAGS! -output "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\!_EMBEDDED_FRAMEWORK_BINARYNAME!" -create
				"!TOOLCHAIN_PATH!\lipo.exe"                     !_LIPO_FLAGS! -output "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\!_EMBEDDED_FRAMEWORK_BINARYNAME!" -create
				if not "!errorlevel!"=="0" goto :exit_error
				del /f /q "!TEMPDIR!\embedded-binary."*
			) else (
				copy /y /b "!_EMBEDDED_FRAMEWORK_PATH!\!_EMBEDDED_FRAMEWORK_BINARYNAME!" "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\!_EMBEDDED_FRAMEWORK_BINARYNAME!" > nul 2>&1
			)

			rem // (optional) strip the embedded framework to reduce code size
			if "_!WANT_STRIP!_"=="_true_" (
				if "_!REMOTELOG_IP!_"=="__" (
					rem // first pass: normal symbols stripping. Remove all local (unexported) symbols (-x), leaving only global (exported) symbols
					rem if "_!WANT_DSYM!_"=="_true_" "!TOOLCHAIN_PATH!\dsymutil.exe" "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\!_EMBEDDED_FRAMEWORK_BINARYNAME!" -o "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\!_EMBEDDED_FRAMEWORK_BINARYNAME!.dSYM"
					if not "_!DEBUG_ARG!_"=="__" echo Calling: strip -x "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\!_EMBEDDED_FRAMEWORK_BINARYNAME!"
					"!TOOLCHAIN_PATH!\strip.exe"                     -x "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\!_EMBEDDED_FRAMEWORK_BINARYNAME!" > nul 2>&1
					rem // DO NOT test for errors after strip, as the binary may contain no strippable symbols.
				)
				if "_!WANT_STRIP_BITCODE!_"=="_true_" (
					rem // second pass: bitcode stripping
					if not "_!DEBUG_ARG!_"=="__" echo Calling: bitcode_strip -r "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\!_EMBEDDED_FRAMEWORK_BINARYNAME!" -o "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\!_EMBEDDED_FRAMEWORK_BINARYNAME!"
					"!TOOLCHAIN_PATH!\bitcode_strip.exe"                     -r "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\!_EMBEDDED_FRAMEWORK_BINARYNAME!" -o "!OUTDIR!\Frameworks\!_EMBEDDED_FRAMEWORK_NAME!.framework\!_EMBEDDED_FRAMEWORK_BINARYNAME!"
					if not "!errorlevel!"=="0" goto :exit_error
				)
			)
		)

		rem // signal our caller that we succeeded
		set RETVAL=0

	rem // else is it a library, i.e. does the extension end in .dylib ?
	) else if "_!_EMBEDDED_BINARY_TYPE!_"=="_.dylib_" (

		rem // guess the library's name from its pathname
		set _EMBEDDED_LIBRARY_PATHNAME=!_EMBEDDED_BINARY!
		for %%a in (!_EMBEDDED_LIBRARY_PATHNAME!) do set _EMBEDDED_LIBRARY_NAME=%%~nxa
		echo  + Embedding library: !_EMBEDDED_LIBRARY_NAME!...

		rem // create the libraries directory in the final app if it doesn't exist yet
		mkdir "!OUTDIR!\Frameworks" > nul 2>&1

		rem // see if we need to add this particular rpath load command to the calling binary
		if not "_!DEBUG_ARG!_"=="__" echo Calling: otool -l "!_CALLING_BINARY!" -arch !ARCH!^|findstr /c:"\(offset 12\)"^|findstr "^^ path"^|findstr /c:"@executable_path/Frameworks"
		"!TOOLCHAIN_PATH!\otool.exe"                     -l "!_CALLING_BINARY!" -arch !ARCH! |findstr /c:"\(offset 12\)" |findstr "^^ path" |findstr /c:"@executable_path/Frameworks" > nul 2>&1 || (
			rem // this search path was not found, add it to the calling binary
			if not "_!DEBUG_ARG!_"=="__" echo Calling: install_name_tool -add_rpath "@executable_path/Frameworks" "!_CALLING_BINARY!"
			"!TOOLCHAIN_PATH!\install_name_tool.exe"                     -add_rpath "@executable_path/Frameworks" "!_CALLING_BINARY!"
			if not "!errorlevel!"=="0" goto :exit_error
		)

		rem // see if the library to embed is fat, in which case strip it from the unnecessary arches, else simply copy it in place
		"!TOOLCHAIN_PATH!\lipo.exe" -info "!_EMBEDDED_LIBRARY_PATHNAME!" > %TO_TEMPVAR%
		if not "_!TEMPVAR:~0,7!_"=="_Non-fat_" (
			rem // extract only the archs we need so as to merge them in the library file, then merge them all in place
			set _LIPO_FLAGS=
			for %%a in (!ARCHS!) do (
				if not "_!DEBUG_ARG!_"=="__" echo Calling: lipo "!_EMBEDDED_LIBRARY_PATHNAME!" -thin %%a -output "!TEMPDIR!\embedded-binary.%%a"
				"!TOOLCHAIN_PATH!\lipo.exe"                     "!_EMBEDDED_LIBRARY_PATHNAME!" -thin %%a -output "!TEMPDIR!\embedded-binary.%%a"
				if not "!errorlevel!"=="0" goto :exit_error
				set _LIPO_FLAGS=!_LIPO_FLAGS! -arch %%a "!TEMPDIR!\embedded-binary.%%a"
			)
			if not "_!DEBUG_ARG!_"=="__" echo Calling: lipo !_LIPO_FLAGS! -output "!OUTDIR!\Frameworks\!_EMBEDDED_LIBRARY_NAME!" -create
			"!TOOLCHAIN_PATH!\lipo.exe"                     !_LIPO_FLAGS! -output "!OUTDIR!\Frameworks\!_EMBEDDED_LIBRARY_NAME!" -create
			if not "!errorlevel!"=="0" goto :exit_error
			del "!TEMPDIR!\embedded-binary."*
		)
		if not exist "!OUTDIR!\Frameworks\!_EMBEDDED_LIBRARY_NAME!" copy /y /b "!_EMBEDDED_LIBRARY_PATHNAME!" "!OUTDIR!\Frameworks\!_EMBEDDED_LIBRARY_NAME!" > nul 2>&1

		rem // (optional) strip the embedded library to reduce code size
		if "_!WANT_STRIP!_"=="_true_" (
			if "_!REMOTELOG_IP!_"=="__" (
				rem // first pass: normal symbols stripping. Remove all local (unexported) symbols (-x), leaving only global (exported) symbols
				rem if "_!WANT_DSYM!_"=="_true_" "!TOOLCHAIN_PATH!\dsymutil.exe" "!OUTDIR!\Frameworks\!_EMBEDDED_LIBRARY_NAME!" -o "!OUTDIR!\Frameworks\!_EMBEDDED_LIBRARY_NAME!.dSYM"
				if not "_!DEBUG_ARG!_"=="__" echo Calling: strip -x "!OUTDIR!\Frameworks\!_EMBEDDED_LIBRARY_NAME!"
				"!TOOLCHAIN_PATH!\strip.exe"                     -x "!OUTDIR!\Frameworks\!_EMBEDDED_LIBRARY_NAME!" > nul 2>&1
				rem // DO NOT test for errors after strip, as the binary may contain no strippable symbols.
			)
			if "_!WANT_STRIP_BITCODE!_"=="_true_" (
				rem // second pass: bitcode stripping
				if not "_!DEBUG_ARG!_"=="__" echo Calling: bitcode_strip -r "!OUTDIR!\Frameworks\!_EMBEDDED_LIBRARY_NAME!" -o "!OUTDIR!\Frameworks\!_EMBEDDED_LIBRARY_NAME!"
				"!TOOLCHAIN_PATH!\bitcode_strip.exe"                     -r "!OUTDIR!\Frameworks\!_EMBEDDED_LIBRARY_NAME!" -o "!OUTDIR!\Frameworks\!_EMBEDDED_LIBRARY_NAME!"
				if not "!errorlevel!"=="0" goto :exit_error
			)
		)

		rem // signal our caller that we succeeded
		set RETVAL=0

	rem // else is it a header, i.e. does the extension end in .h ?
	) else if "_!_EMBEDDED_BINARY_TYPE!_"=="_.h_" (

		rem // guess the header's name from its pathname
		set _EMBEDDED_HEADER_PATHNAME=!_EMBEDDED_BINARY!
		for %%a in (!_EMBEDDED_HEADER_PATHNAME!) do set _EMBEDDED_HEADER_NAME=%%~nxa
		echo  + Embedding header: !_EMBEDDED_HEADER_NAME!...

		rem // create the headers directory in the final app if it doesn't exist yet
		mkdir "!OUTDIR!\Headers" > nul 2>&1

		rem // copy this header there
		copy /y /b "!_EMBEDDED_HEADER_PATHNAME!" "!OUTDIR!\Headers\!_EMBEDDED_HEADER_NAME!" > nul 2>&1
		if not "!errorlevel!"=="0" goto :exit_error

		rem // signal our caller that we succeeded
		set RETVAL=0
	)

	rem // release local variables
	set _CALLING_BINARY=
	set _EMBEDDED_BINARY=
	set _EMBEDDED_BINARY_TYPE=
	set _EMBEDDED_APPEX_PATH=
	set _EMBEDDED_APPEX_NAME=
	set _EMBEDDED_APPEX_BINARYNAME=
	set _EMBEDDED_FRAMEWORK_PATH=
	set _EMBEDDED_FRAMEWORK_NAME=
	set _EMBEDDED_FRAMEWORK_BINARYNAME=
	set _EMBEDDED_LIBRARY_PATHNAME=
	set _EMBEDDED_LIBRARY_NAME=
	set _EMBEDDED_HEADER_PATHNAME=
	set _EMBEDDED_HEADER_NAME=
	set _LIPO_FLAGS=
goto :eof

:fetch_dependency_or_die
	rem // subroutine that locates a dependency in the built products directory and installs it in the output directory or die if not found

	rem // strip quotes from item
	set _FIXABLE_NAME=###%1###
	set _FIXABLE_NAME=!_FIXABLE_NAME:"###=!
	set _FIXABLE_NAME=!_FIXABLE_NAME:###"=!
	if not "_!_FIXABLE_NAME!_"=="__" set _FIXABLE_NAME=!_FIXABLE_NAME:###=!
	rem // as we're running from a function, a double dequote is necessary
	set _FIXABLE_NAME=###!_FIXABLE_NAME!###
	set _FIXABLE_NAME=!_FIXABLE_NAME:"###=!
	set _FIXABLE_NAME=!_FIXABLE_NAME:###"=!
	if not "_!_FIXABLE_NAME!_"=="__" set _FIXABLE_NAME=!_FIXABLE_NAME:###=!

	rem echo :fetch_dependency_or_die called for: [!_FIXABLE_NAME!]

	rem // dependency not in place, let's look for it in the built products directories
	if not exist "!OUTDIR!\Frameworks\!_FIXABLE_NAME!.framework\!_FIXABLE_NAME!" if not exist "!OUTDIR!\Frameworks\!_FIXABLE_NAME!.dylib" (
		set _OLDDIR=!CD!
		rem // first look for frameworks built in the build directory, excluding dependent frameworks
		cd /d "!BUILD_DIR!"
		dir *"!_FIXABLE_NAME!.framework" /b /s 2>nul|findstr /v /c:"\\Frameworks\\!_FIXABLE_NAME!\.framework" > "!TEMPDIR!\candidate-dependencies.txt"
		cd /d "!_OLDDIR!"
		for /f "usebackq delims=" %%p in ("!TEMPDIR!\candidate-dependencies.txt") do if exist "%%p\!_FIXABLE_NAME!" if not exist "!OUTDIR!\Frameworks\!_FIXABLE_NAME!.framework\!_FIXABLE_NAME!" (
			call :embed_binary "%%p"
		)
		if not exist "!OUTDIR!\Frameworks\!_FIXABLE_NAME!.framework\!_FIXABLE_NAME!" (
			rem // if not found, look for dylibs built in the build directory, excluding dependent libraries
			cd /d "!BUILD_DIR!"
			dir *"!_FIXABLE_NAME!.dylib" /b /s 2>nul|findstr /v /c:"\\Frameworks\\!_FIXABLE_NAME!\.dylib" > "!TEMPDIR!\candidate-dependencies.txt"
			cd /d "!_OLDDIR!"
			for /f "usebackq delims=" %%p in ("!TEMPDIR!\candidate-dependencies.txt") do if exist "%%p" if not exist "!OUTDIR!\Frameworks\!_FIXABLE_NAME!.dylib!" (
				call :embed_binary "%%p"
			)
		)
		del /f /s "!TEMPDIR!\candidate-dependencies.txt" >nul 2>&1
		set _OLDDIR=
	)
	if not exist "!OUTDIR!\Frameworks\!_FIXABLE_NAME!.framework\!_FIXABLE_NAME!" if not exist "!OUTDIR!\Frameworks\!_FIXABLE_NAME!.dylib" (
		rem // if not found, look in the linker's library and framework search paths. As our sed is crippled, we must preprocess a lot
		rem // start by prefixing and suffixing our strings with a space character - a newline will do.
		echo.> "!TEMPDIR!\scrap1.txt"
		type "!TARGET_BUILD_DIR!\target-agnostic.ldflags" >> "!TEMPDIR!\scrap1.txt"
		echo.>> "!TEMPDIR!\scrap1.txt"
		type "!TARGET_BUILD_DIR!\frameworks.ldflags"      >> "!TEMPDIR!\scrap1.txt"
		echo.>> "!TEMPDIR!\scrap1.txt"
		rem // now serialize all this to one single line and convert all carriage returns into spaces. This will make sure aLL linker flags will be surrounded by at least one space.
		copy /y nul "!TEMPDIR!\EOF" /a > nul & for /F "usebackq" %%f in ("!TEMPDIR!\EOF") do set "_EOFCHAR=%%f"
		copy /y nul "!TEMPDIR!\scrap2.txt" > nul 2>&1 & for /f "usebackq delims= eol=" %%l in ("!TEMPDIR!\scrap1.txt") do (
			echo  %%l %_EOFCHAR%> "!TEMPDIR!\EOF" & copy /y "!TEMPDIR!\EOF" /a "!TEMPDIR!\OUT" /b > nul & type "!TEMPDIR!\OUT"
		) >> "!TEMPDIR!\scrap2.txt" & del /f /q "!TEMPDIR!\EOF" "!TEMPDIR!\OUT"
		set _EOFCHAR=
		rem // now extract the libraries and framework search paths out of this mess
		(
			rem // this is just ridiculous. Spending hours trying to stuff a few bytes right into sed...
			echo s; -L"([^"]+^)"| -F"([^^^"]+^)"| -L([^ ]+)| -F([^ ]+);\nSEARCHPATH:\1\2\3\4\n;g
		) > "!TEMPDIR!\sedscript.txt"
		type "!TEMPDIR!\scrap2.txt"|"!TOOLCHAIN_PATH!\sed.exe" -r -f "!TEMPDIR!\sedscript.txt"|findstr "^SEARCHPATH:" > "!TEMPDIR!\fwsearchpaths.txt"
		del /f /q "!TEMPDIR!\scrap1.txt" "!TEMPDIR!\scrap2.txt" > nul 2>&1
		rem // look in those search paths now
		for /f "usebackq delims= eol=" %%l in ("!TEMPDIR!\fwsearchpaths.txt") do (
			set _SEARCH_PATH=%%l
			set _SEARCH_PATH=!_SEARCH_PATH:~11!
			set _SEARCH_PATH=!_SEARCH_PATH:/=\!
			rem // dependency not found yet AND existing in this search path ? if so, embed it
			if not exist "!OUTDIR!\Frameworks\!_FIXABLE_NAME!.framework\!_FIXABLE_NAME!" if not exist "!OUTDIR!\Frameworks\!_FIXABLE_NAME!.dylib" if exist "!_SEARCH_PATH!\!_FIXABLE_NAME!.framework\!_FIXABLE_NAME!" (
				call :embed_binary "!_SEARCH_PATH!\!_FIXABLE_NAME!.framework"
			)
			if not exist "!OUTDIR!\Frameworks\!_FIXABLE_NAME!.framework\!_FIXABLE_NAME!" if not exist "!OUTDIR!\Frameworks\!_FIXABLE_NAME!.dylib" if exist "!_SEARCH_PATH!\!_FIXABLE_NAME!.dylib" (
				call :embed_binary "!_SEARCH_PATH!\!_FIXABLE_NAME!.dylib"
			)
		)
		del /f /q "!TEMPDIR!\fwsearchpaths.txt" > nul 2>&1
	)
	if not exist "!OUTDIR!\Frameworks\!_FIXABLE_NAME!.framework\!_FIXABLE_NAME!" if not exist "!OUTDIR!\Frameworks\!_FIXABLE_NAME!.dylib" (
		rem // we need to look for it elsewhere, do it the brute-force way
		echo Warning: dependency !_FIXABLE_NAME! is not among the built products. Looking for a precompiled binary...
		for /d /r "." %%p in (!_FIXABLE_NAME!.framework) do if not exist "!OUTDIR!\Frameworks\!_FIXABLE_NAME!.framework\!_FIXABLE_NAME!" if not exist "!OUTDIR!\Frameworks\!_FIXABLE_NAME!.dylib" if exist "%%p\!_FIXABLE_NAME!" (
			call :embed_binary "%%p"
		)
		for /r "." %%p in (!_FIXABLE_NAME!.dylib) do if not exist "!OUTDIR!\Frameworks\!_FIXABLE_NAME!.framework\!_FIXABLE_NAME!" if not exist "!OUTDIR!\Frameworks\!_FIXABLE_NAME!.dylib" if exist "%%p" (
			call :embed_binary "%%p"
		)
	)

	rem // have we still NOT found it ?
	if not exist "!OUTDIR!\Frameworks\!_FIXABLE_NAME!.framework\!_FIXABLE_NAME!" if not exist "!OUTDIR!\Frameworks\!_FIXABLE_NAME!.dylib" (
		if "_!EMBEDDED_BINARY_NAME!_"=="__" (
			echo ERROR: !NAME! depends on unidentified library or framework: !_FIXABLE_NAME!
		) else (
			echo ERROR: embedded dependency !EMBEDDED_BINARY_NAME! depends on unidentified library or framework: !_FIXABLE_NAME!
		)
		goto :exit_error
	)

	rem // release local variables
	set _FIXABLE_NAME=
	set _SEARCH_PATH=
goto :eof

:open_url
	rem // subroutine that opens the first argument in the user's default browser, whatever it is, using a temporary .url file.
	rem // If the passed URL contains "://", it's opened verbatim, else it's considered local and "file:///" is prefixed.
	set _URL=%1
	set _URL=!_URL:"=!
	if  "_!_URL:://=!_"=="_!_URL!_" set _URL=file:///!_URL:\=/!
	rem // NOTE ABOUT EDGE: THERE IS NO WAY TO OPEN A LOCAL FILE AT A PARTICULAR ANCHOR IN EDGE AND MICROSOFT WON'T FIX IT.
	rem // THEIR RECOMMENDED WAY IS TO USE A LOCAL NodeJS WEBSERVER. What a bunch of brats.
	rem // test if Edge is present. If so, use the old Internet Explorer that's guaranteed to be here for compatibility reasons.
	if exist "%LOCALAPPDATA%\Microsoft\WindowsApps\MicrosoftEdge.exe" (
		start iexplore -extoff "!_URL!"
	) else (
		rem // Edge not present, fine. Use the user's default browser
		echo [InternetShortcut]> "!TEMPDIR!\link.url"
		echo URL=!_URL!>> "!TEMPDIR!\link.url"
		start /I "" "!TEMPDIR!\link.url"
	)
	rem // leave some time for the browser to wake up and open the file, before it gets deleted at the end of this script
	timeout /t 5 /nobreak > nul
	rem // release local variables
	set _URL=
goto :eof

:dirname
	rem // subroutine that extracts the path of the pathname passed in the first argument and returns it in the second
	set "%~2=%~dp1"
goto :eof

:dec_to_hex
	rem // subroutine that converts the first argument passed to it from decimal to hex and returns it in the second
	set _HEX_LOOKUP=0123456789ABCDEF
	set _RESULT=
	if "_%1"=="_" set "%2=0"& goto :eof
	set /a _A=%1 || exit /b 1
:dec_to_hex_loop
	set /a _B=!_A! %% 16
	set /a _A=!_A! / 16
	set _RESULT=!_HEX_LOOKUP:~%_B%,1!!_RESULT!
	if !_A! GTR 0 goto :dec_to_hex_loop
	set "%2=!_RESULT!"
	rem // release local variables
	set _HEX_LOOKUP=
	set _RESULT=
	set _A=
	set _B=
goto :eof

rem /////////////////////////////////////////////////////////////////////////////////////////////////////
rem // I should rewrite that in something faster, such as Brainf*ck or C-INTERCAL someday. Definitely. //
rem /////////////////////////////////////////////////////////////////////////////////////////////////////
